#ifndef _ECS_CHAMP_CHAINE_H_
#define _ECS_CHAMP_CHAINE_H_

/*============================================================================
 *  Prototypes des fonctions de base
 *   associes  une liste chane de structures `ecs_champ_t' dcrivant
 *   un champ
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <bft_file.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_param_perio_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_publ.h"


/*============================================================================
 *                       Prototypes de fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction librant la portion d'une liste chane de structures `ecs_champ_t'
 *    partir d'un noeud dont le pointeur est donn en argument.
 *  Le noeud est  NULL au retour de la fonction
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__detruit
(
 ecs_champ_t * * this_champ_noeud
) ;


/*----------------------------------------------------------------------------
 *  Fonction imprimant  partir d'un noeud `ecs_champ_t' donn
 *   une liste chane de champs
 *   sur le flux dcrit par la structure `bft_file_t'
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__imprime
(
 const ecs_champ_t *const this_champ_noeud ,
       ecs_int_t          imp_col          ,
 const ecs_int_t          nbr_imp          ,
       bft_file_t  *const fic_imp
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la taille en octets
 *   d'une chane de structures `ecs_champ_t' dont la tte est donne
 *----------------------------------------------------------------------------*/

float ecs_champ_chaine__ret_taille
(
 const ecs_champ_t *const this_champ_noeud
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui retourne dans une liste chane de champs
  *   dont un noeud est donn la rfrence au champ suivant.
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__ret_suivant
(
 ecs_champ_t *const this_champ
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui recherche dans une liste chane de champs
 *   dont la tte de la liste est donne,
 *   le champ correspondant au nom fourni en argument
 *
 *  La fonction retourne :
 *   -    la rfrence du champ trouv,
 *   - ou NULL si aucun champ n'a t trouv
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__trouve_nom
(
       ecs_champ_t *const this_champ_tete , /* --> Tte de liste des champs   */
                                            /*     ou est recherche le champ  */
 const char        *const nom_champ         /* --> Nom  rechercher           */
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui recherche dans une liste chane de champs
 *   dont la tte de la liste est donne,
 *   le premier champ correspondant au statut fourni en argument
 *
 *  La fonction retourne :
 *   -    la rfrence du champ trouv,
 *   - ou NULL si aucun champ n'a t trouv
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__trouve_statut
(
 ecs_champ_t         *const this_champ_tete , /* --> Tte de liste des champs */
                                              /*     o on recherche le champ */
 ECS_CHAMP_STATUT_E         statut            /* --> Nom  rechercher         */
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui supprime un noeud (c-a-d la rfrence au champ donn)
 *   dans une liste chane de champs dont la tte est donne
 *
 *  La tte de la liste n'est pas modifie
 *   sauf si le noeud  supprimer est aussi la tte de la liste !
 *
 *  Attention! Le champ dont la rfrence est supprime de la liste,
 *              n'est pas dtruit et
 *              son lien sur un champ suivant est r-initialis  `NULL'
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__supprime
(
 ecs_champ_t * *const this_champ_noeud ,  /* --> Adresse du champ  supprimer */
 ecs_champ_t * *const champ_tete          /* --> Tte de la liste contenant   */
                                          /*      le noeud  supprimer        */
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui ajoute  la fin d'une liste chane de champs rceptrice
  *   une liste chane de champs  concatner.
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__ajoute
(
 ecs_champ_t * *const this_champ_tete   ,
 ecs_champ_t   *const champ_concat_tete
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui ajoute  la fin d'une liste chane de champs rceptrice
 *   une liste chane de champs  concatner.
 *  Si des champs de mme nom existent dans la liste chane, ils sont
 *   supprims avant l'ajout des nouveaux champs.
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__remplace
(
 ecs_champ_t * *const this_champ_tete   ,
 ecs_champ_t   *const champ_concat_tete
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui concatne les champs identiques
 *   de deux listes chanes de champs auxiliaires
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__concatene
(
 ecs_champ_t * * this_champ_tete    ,  /* <-> Tte de la chane rceptrice    */
 ecs_champ_t * * concat_champ_tete  ,  /* <-> Tte de la chane contenant     */
                                       /*         les champs  concatner     */
 ecs_int_t       nbr_elt_init_recep ,  /* --> Nbr d'elts de l'entit rcept.  */
 ecs_int_t       nbr_elt_ent_concat    /* --> Nbr d'elts ent.  concatner.   */
) ;


/*----------------------------------------------------------------------------
 *  Fonction appliquant un vecteur de transformation
 *    une liste chane de champs donne
 *----------------------------------------------------------------------------*/

typedef void (* ECS_PTR_FCT_TRANSF_CHAMP)
(
       ecs_champ_t         *const this_champ ,
 const ecs_int_t            nbr_elt_new,
 const ecs_tab_int_t        vect_transf
) ;


void ecs_champ_chaine__transforme
(
       ecs_champ_t         * this_champ_chaine_tete ,
 const ecs_int_t             nbr_elt_new            ,
 const ecs_tab_int_t         vect_transf            ,
 ECS_PTR_FCT_TRANSF_CHAMP    fct_transformation
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit les champs de la chane de champs des sous-lments
 *                 partir des champs de la chane de champs des lments
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__herite
(
 ecs_champ_t *champ_elt_tete,
 ecs_champ_t *champ_elt_def_sselt,
 ecs_champ_t *champ_def_sselt
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui renumrote les labels et les rfrences des champs
 *  attributs de type ECS_CHAMP_STATUT_REF_ELT
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__renum
(
       ecs_champ_t    *champ_elt_tete,
       ecs_champ_t    *champ_elt_old_new,
 const ecs_int_t       nbr_elt_new,
 const ecs_int_t       num_algo_renum
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui modifie le champ de connectivit inverse "faces -> cellules"
 *  dans le cas d'une ou plusieurs priodicit.
 *  On retourne un champ marquant les cellules fantmes pour chaque
 *  priodicit.
 *----------------------------------------------------------------------------*/

ecs_int_t ecs_champ_chaine__connect_perio
(
 ecs_champ_t  * champ_att_tete,
 ecs_champ_t  * champ_connect_inverse_fac_cel
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit la liste des faces priodiques correspondant
 *   une ou plusieurs priodicit.
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__cree_liste_fac_perio
(
       ecs_champ_t     *const champ_att_tete,
       ecs_int_t     * *const liste_fac_perio,
 const ecs_int_t              nbr_fac_interne,
       ecs_int_t       *const liste_fac_interne
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit la liste du sens de la priodicit  appliquer
 *  pour obtenir les faces priodiques  partir de leur correspondant.
 *   -->  1 correspond  la priodicit normale
 *       -1 correspond  la priodicit inverse
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__sens_perio
(
 ecs_champ_t    *const champ_att_tete,
 ecs_int_t    * *const dir_per_fac_perio,
 ecs_int_t    * *const liste_fac_perio,
 ecs_int_t      *const liste_fac_interne
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit une liste de toutes les faces priodiques,
 *  tries dans l'ordre croissant avec le numro et le sens de la priodicit
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_chaine__liste_fac_perio_glob
(
 ecs_champ_t    * champ_att_tete
) ;


#endif /* _ECS_CHAMP_CHAINE_H_ */
