#ifndef _ECS_CHAMP_GRD_H_
#define _ECS_CHAMP_GRD_H_

/*============================================================================
 *  Prototypes des fonctions
 *   associees a la structure `ecs_champ_t' decrivant un champ
 *   et propres aux calculs de type "grandeur"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  des paquetages visibles
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_publ.h"


/*============================================================================
 *                       Prototypes de fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les coordonnes minimales et maximales
 *----------------------------------------------------------------------------*/

void ecs_champ_grd__calc_coo_ext
(
 ecs_champ_t   *const champ_som_def
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les distorsions de faces que l'on peut filtrer
 *  via une liste. On renvoie galement une liste des faces dont la
 *  distorsion est suprieure  un angle 'max_warp_fac' pour un ventuel
 *  affichage et/ou redcoupage de ces faces en triangles.
 *
 *  On calcule l'angle entre chacune des artes d'une face et la normale
 *  de la face (calcule prcdemment). L'angle maximal obtenu sera
 *  pris comme tant la distorsion de la face.
 *----------------------------------------------------------------------------*/

void  ecs_champ_grd__verif_fac_distor
(
       ecs_champ_t    *const champ_som_def     ,
       ecs_champ_t    *const champ_are_def     ,
       ecs_champ_t    *const champ_fac_def     ,
 const ecs_tab_int_t  *const liste_filtre      ,
       ecs_tab_int_t  *const liste_fac_warp    ,
       ecs_tab_real_t *const tab_val_warp      ,
 const ecs_real_t            max_warp_fac
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui calcule le maximum associ  une cellule d'un tableau de
 *   valeurs associes aux faces internes.
 *----------------------------------------------------------------------------*/

ecs_tab_real_t  ecs_champ_grd__max_fac_int
(
 const ecs_int_t               nbr_cel               ,
       ecs_champ_t      *const champ_connect_fac_cel ,
 const ecs_tab_int_t    *const liste_fac_interne     ,
 const ecs_tab_real_t   *const val_fac_interne       ,
 const ecs_real_t              val_min
) ;


#endif /* _ECS_CHAMP_GRD_H_ */
