/*============================================================================
 *  Dfinitions des fonctions de base
 *   associes a la structure `ecs_renum_t' dcrivant une renumrotation
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

#include "ecs_config.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>  /* memcmp() */
#include <string.h>  /* memcmp() */

#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_param_perio_glob.h"
#include "ecs_vec_int.h"
#include "ecs_vec_grd.h"

/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#if defined(HAVE_METIS)

#ifdef __cplusplus
extern "C" {
#endif

/*
  Attention :
  -----------
  Les fichiers inclus par <metis.h> posent problme  minima sous Linux du 
  des redclarations de fonctions de <stdlib.h>. On recopie dans ce cas le
  minimum ncessaire issu de <metis.h> (correspondant  la METIS 4.0)
*/

typedef int idxtype;
void METIS_PartGraphRecursive(int *, idxtype *, idxtype *, idxtype *, idxtype *,
                              int *, int *, int *, int *, int *, idxtype *);
void METIS_PartGraphKway(int *, idxtype *, idxtype *, idxtype *, idxtype *,
                         int *, int *, int *, int *, int *, idxtype *);

#ifdef __cplusplus
}
#endif

#endif /* defined(HAVE_METIS) */


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_renum.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_renum_priv.h"



/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des faces pour un seul domaine
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_fac_1
(
       ecs_renum_t       *       renum               , /* <-> Struct. de renum*/
       ecs_bool_t                bool_voiset_cel_cel , /* --> Voisinage tendu
                                                                   ou non     */
 const ecs_param_perio_t * const liste_param_perio   , /* --> Param priodicit*/
 const ecs_real_t        * const coord_som_maillage    /* --> Coord. sommets  */
) ;


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation locale des sommets.
 *    Les sommets sont utiliss pour des calculs gomtriques, mais ne
 *    portent pas de variables  ce jour. On ne cre donc pas de liste de
 *    correspondance entre sommets de domaines voisins, et on se contente
 *    de crer la connectivit locale  chaque sous-domaine.
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_som
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
) ;


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des sommets pour un seul domaine
 *----------------------------------------------------------------------------*/

static void ecs_loc_renum__connect_som_1
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
) ;


/*----------------------------------------------------------------------------
 *    Fonction qui retourne un voisinage tendu cellule -> cellules
 *
 *    On dtermine, pour chaque cellule, le voisinage tendu, i.e. l'ensemble
 *      des cellules qui partagent un noeud avec la cellule considre.
 *    Pour une cellule donne, les cellules voisines vues au travers des
 *      faces ne sont pas incluses dans le voisinage tendu.
 *----------------------------------------------------------------------------*/

static void ecs_loc_renum__voiset_cel_cel
(
       ecs_renum_t       *       renum               ,
                                    /* --> Structure de renumrotation        */
 const ecs_param_perio_t * const liste_param_perio   ,
                                    /* --> Paramtres de priodicit          */
 const ecs_real_t        * const coord_som_maillage  ,
                                    /* --> Coordonnes des sommets            */
       ecs_int_t         *      *ppos_connect_voiset_cel_cel ,
                                    /* <-- Adresse du pointeur qui donne,
                                           pour chaque cellule, position dans
                                           val_connect_voiset_cel_cel de la
                                           1ere cellule du voisinage tendu
                                           pos_connect_voiset_cel_cel > 0 */
       ecs_int_t         *      *pval_connect_voiset_cel_cel ,
                                    /* <-- Adresse du pointeur qui donne,
                                           numro global des cellules du
                                           du voisinage tendu
                                           val_connect_voiset_cel_cel > 0     */
       ecs_int_t         *      *pind_per_dir_voiset_cel_cel
                                    /* <-- Adresse du pointeur qui donne,
                                           indique la priodicit ventuelle
                                           au travers de laquelle les cellules
                                           du voisinage tendu sont vues :
                                           = 0 : non priodique
                                           > 0 : n de priodicit, directe
                                           < 0 : n de priodicit, inverse   */
) ;


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction qui cre une structure de renumrotation
 *----------------------------------------------------------------------------*/

ecs_renum_t * ecs_renum__cree
(
 void
)
{

  ecs_renum_t   * renum ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  BFT_MALLOC(renum, 1, ecs_renum_t);


  renum->nbr_dom = 1 ;

  renum->nbr_per     = 0 ;
  renum->nbr_per_rot = 0 ;

  renum->nbr_cel = 0 ;
  renum->nbr_cel_loc = NULL ;
  renum->nbr_cel_par_loc = NULL ;
  renum->nbr_cel_par_loc_voiset = NULL ;

  renum->nbr_fac = 0 ;
  renum->nbr_fac_interne = 0 ;
  renum->nbr_fac_de_bord = 0 ;
  renum->nbr_fac_perio       = NULL ;
  renum->nbr_fac_interne_loc = NULL ;
  renum->nbr_fac_de_bord_loc = NULL ;

  renum->nbr_som = 0 ;
  renum->nbr_som_loc = NULL ;

  renum->val_connect_fac_interne_cel = NULL ;
  renum->val_connect_fac_de_bord_cel = NULL ;
  renum->val_connect_fac_interne_cel_loc = NULL ;
  renum->val_connect_fac_de_bord_cel_loc = NULL ;

  renum->liste_fac_interne = NULL ;
  renum->liste_fac_de_bord = NULL ;
  renum->liste_fac_perio   = NULL ;
  renum->dir_per_fac_perio = NULL ;

  renum->pos_connect_fac_de_bord_som = NULL ;
  renum->pos_connect_fac_interne_som = NULL ;
  renum->val_connect_fac_de_bord_som = NULL ;
  renum->val_connect_fac_interne_som = NULL ;
  renum->pos_connect_fac_de_bord_som_loc = NULL ;
  renum->pos_connect_fac_interne_som_loc = NULL ;
  renum->val_connect_fac_de_bord_som_loc = NULL ;
  renum->val_connect_fac_interne_som_loc = NULL ;
  renum->pos_connect_voiset_cel_cel_loc = NULL ;
  renum->val_connect_voiset_cel_cel_loc = NULL ;

  renum->num_glob_cel = NULL ;
  renum->num_glob_fac = NULL ;
  renum->num_glob_fbr = NULL ;
  renum->num_glob_som = NULL ;

  renum->nbr_dom_cel_ext = NULL ;
  renum->num_loc_cel_ext = NULL ;
  renum->num_dom_cel_ext = NULL ;
  renum->pos_dom_cel_ext = NULL ;
  renum->pos_per_cel_ext = NULL ;

  renum->nbr_dom_cel_ext_voiset = NULL ;
  renum->num_dom_cel_ext_voiset = NULL ;
  renum->pos_dom_cel_ext_voiset = NULL ;
  renum->pos_per_cel_ext_voiset = NULL ;

  renum->num_dom_cel = NULL ;

  renum->bool_voiset_separe = ECS_FALSE ;

  return renum ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui dtruit une structure de renumrotation
 *----------------------------------------------------------------------------*/

ecs_renum_t * ecs_renum__detruit
(
 ecs_renum_t *renum
)
{

  ecs_int_t   ind_dom ;
  ecs_int_t   ind_per ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  if (renum->nbr_dom > 1) {

    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

      if (renum->val_connect_fac_interne_cel_loc[ind_dom] !=
          renum->val_connect_fac_interne_cel)
        BFT_FREE(renum->val_connect_fac_interne_cel_loc[ind_dom]) ;

      if (renum->val_connect_fac_de_bord_cel_loc[ind_dom] !=
          renum->val_connect_fac_de_bord_cel)
        BFT_FREE(renum->val_connect_fac_de_bord_cel_loc[ind_dom]) ;

      if (renum->nbr_som > 0) {

        if ((renum->pos_connect_fac_interne_som_loc[ind_dom] != NULL) &&
            (renum->pos_connect_fac_interne_som_loc[ind_dom] !=
             renum->pos_connect_fac_interne_som)) {
          BFT_FREE(renum->val_connect_fac_interne_som_loc[ind_dom]) ;
          BFT_FREE(renum->pos_connect_fac_interne_som_loc[ind_dom]) ;
        }

        if ((renum->pos_connect_fac_de_bord_som_loc[ind_dom] != NULL) &&
            (renum->pos_connect_fac_de_bord_som_loc[ind_dom] !=
             renum->pos_connect_fac_de_bord_som)) {
          BFT_FREE(renum->val_connect_fac_de_bord_som_loc[ind_dom]) ;
          BFT_FREE(renum->pos_connect_fac_de_bord_som_loc[ind_dom]) ;
        }

      }

      if (renum->val_connect_voiset_cel_cel_loc          != NULL &&
          renum->val_connect_voiset_cel_cel_loc[ind_dom] != NULL)
        BFT_FREE(renum->val_connect_voiset_cel_cel_loc[ind_dom]) ;
      if (renum->pos_connect_voiset_cel_cel_loc          != NULL &&
          renum->pos_connect_voiset_cel_cel_loc[ind_dom] != NULL)
        BFT_FREE(renum->pos_connect_voiset_cel_cel_loc[ind_dom]) ;

      if (renum->num_glob_cel != NULL)
        BFT_FREE(renum->num_glob_cel[ind_dom]) ;

      if (renum->num_glob_fac != NULL)
        BFT_FREE(renum->num_glob_fac[ind_dom]) ;

      if (renum->num_glob_fbr != NULL)
        BFT_FREE(renum->num_glob_fbr[ind_dom]) ;

      if (renum->num_glob_som != NULL)
        BFT_FREE(renum->num_glob_som[ind_dom]) ;

      BFT_FREE(renum->pos_dom_cel_ext[ind_dom]) ;
      BFT_FREE(renum->num_loc_cel_ext[ind_dom]) ;
      BFT_FREE(renum->num_dom_cel_ext[ind_dom]) ;

      if (renum->nbr_dom_cel_ext_voiset != NULL &&
          renum->pos_dom_cel_ext_voiset[ind_dom] != NULL)
          BFT_FREE(renum->pos_dom_cel_ext_voiset[ind_dom]) ;
      if (renum->num_dom_cel_ext_voiset != NULL &&
          renum->num_dom_cel_ext_voiset[ind_dom] != NULL)
          BFT_FREE(renum->num_dom_cel_ext_voiset[ind_dom]) ;

    }


    BFT_FREE(renum->num_dom_cel) ;

    BFT_FREE(renum->num_glob_cel) ;

    if (renum->num_glob_fac != NULL)
      BFT_FREE(renum->num_glob_fac) ;

    if (renum->num_glob_fbr != NULL)
      BFT_FREE(renum->num_glob_fbr) ;

    if (renum->num_glob_som != NULL)
      BFT_FREE(renum->num_glob_som);

    BFT_FREE(renum->nbr_cel_par_loc) ;
    BFT_FREE(renum->num_loc_cel_ext) ;

    if (renum->nbr_cel_par_loc_voiset != NULL)
      BFT_FREE(renum->nbr_cel_par_loc_voiset) ;

    BFT_FREE(renum->nbr_dom_cel_ext) ;
    BFT_FREE(renum->num_dom_cel_ext) ;
    BFT_FREE(renum->num_loc_cel_ext) ;
    BFT_FREE(renum->pos_dom_cel_ext) ;

    if (renum->nbr_dom_cel_ext_voiset != NULL)
      BFT_FREE(renum->nbr_dom_cel_ext_voiset) ;
    if (renum->num_dom_cel_ext_voiset != NULL)
      BFT_FREE(renum->num_dom_cel_ext_voiset) ;
    if (renum->pos_dom_cel_ext_voiset != NULL)
      BFT_FREE(renum->pos_dom_cel_ext_voiset) ;

  }


  if (renum->nbr_per > 0) {

    /* On a besoin d'une structure identique au parallelisme pour
       la gestion des faces fantomes periodiques (meme dans le cas
       d'un calcul sequentiel) */

    /* Tableaux propres a la periodicite */

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {
      BFT_FREE(renum->liste_fac_perio[ind_per]) ;
      BFT_FREE(renum->dir_per_fac_perio[ind_per]) ;
    }

    BFT_FREE(renum->liste_fac_perio) ;
    BFT_FREE(renum->dir_per_fac_perio) ;
    BFT_FREE(renum->nbr_fac_perio) ;


    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++)
      BFT_FREE(renum->pos_per_cel_ext[ind_dom]) ;

    BFT_FREE(renum->pos_per_cel_ext) ;

    if (renum->pos_per_cel_ext_voiset != NULL) {
      for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
        if (renum->pos_per_cel_ext_voiset[ind_dom] != NULL)
          BFT_FREE(renum->pos_per_cel_ext_voiset[ind_dom]) ;
      }
      BFT_FREE(renum->pos_per_cel_ext_voiset) ;
    }

    /* Tableaux deja liberes dans le cas du parallelisme */

    if (renum->nbr_dom == 1) {

      BFT_FREE(renum->nbr_dom_cel_ext) ;
      BFT_FREE(renum->nbr_cel_par_loc) ;

      if (renum->nbr_cel_par_loc_voiset != NULL)
        BFT_FREE(renum->nbr_cel_par_loc_voiset) ;

      BFT_FREE(renum->num_loc_cel_ext[0]) ;
      BFT_FREE(renum->num_loc_cel_ext) ;

    }

  }


  if (renum->nbr_som > 0) {

    if (renum->pos_connect_fac_interne_som_loc != NULL) {
      BFT_FREE(renum->val_connect_fac_interne_som_loc) ;
      BFT_FREE(renum->pos_connect_fac_interne_som_loc) ;
    }

    if (renum->pos_connect_fac_de_bord_som_loc != NULL) {
      BFT_FREE(renum->val_connect_fac_de_bord_som_loc) ;
      BFT_FREE(renum->pos_connect_fac_de_bord_som_loc) ;
    }

  }

  if (renum->nbr_dom == 1) {
    if (renum->val_connect_voiset_cel_cel_loc    != NULL &&
        renum->val_connect_voiset_cel_cel_loc[0] != NULL)
        BFT_FREE(renum->val_connect_voiset_cel_cel_loc[0]);
    if (renum->pos_connect_voiset_cel_cel_loc    != NULL &&
        renum->pos_connect_voiset_cel_cel_loc[0] != NULL)
      BFT_FREE(renum->pos_connect_voiset_cel_cel_loc[0]);
  }

  if (renum->pos_connect_voiset_cel_cel_loc != NULL) {
    BFT_FREE(renum->val_connect_voiset_cel_cel_loc) ;
    BFT_FREE(renum->pos_connect_voiset_cel_cel_loc) ;
  }

  BFT_FREE(renum->nbr_cel_loc);
  BFT_FREE(renum->nbr_fac_interne_loc) ;
  BFT_FREE(renum->nbr_fac_de_bord_loc) ;
  BFT_FREE(renum->nbr_som_loc);

  BFT_FREE(renum->val_connect_fac_interne_cel_loc) ;
  BFT_FREE(renum->val_connect_fac_de_bord_cel_loc) ;

  BFT_FREE(renum->liste_fac_interne);
  BFT_FREE(renum->liste_fac_de_bord);

  BFT_FREE(renum->val_connect_fac_interne_cel) ;
  BFT_FREE(renum->val_connect_fac_de_bord_cel) ;


  if (renum->pos_connect_fac_de_bord_som != NULL) {
    BFT_FREE(renum->val_connect_fac_de_bord_som) ;
    BFT_FREE(renum->pos_connect_fac_de_bord_som) ;
  }
  if (renum->pos_connect_fac_interne_som != NULL) {
    BFT_FREE(renum->val_connect_fac_interne_som) ;
    BFT_FREE(renum->pos_connect_fac_interne_som) ;
  }


  BFT_FREE(renum);


  return NULL ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie une structure ecs_tab_int_t correspondant aux
 *  numros de sous-domaine des cellules
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_renum__ret_num_domaine
(
 ecs_renum_t  * renum               /* --> Structure de renumerotation        */
)
{

  ecs_tab_int_t   num_domaine ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  num_domaine.nbr = renum->nbr_cel ;
  num_domaine.val = renum->num_dom_cel ;


  return num_domaine ;

}


/*----------------------------------------------------------------------------
 *    Fonction de dcoupage de maillage
 *----------------------------------------------------------------------------*/

void ecs_renum__decoup
(
 ecs_renum_t  * renum     ,         /* --> Structure de renumrotation        */
 ecs_int_t      nbr_dom             /* --> Nombre de blocs aprs dcoupage    */
)
{

  ecs_int_t * num_dom_cel ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  BFT_MALLOC(num_dom_cel, renum->nbr_cel, ecs_int_t);


#if defined(HAVE_METIS)

  {

    ecs_int_t   cpt_cel_ident ;

    int    nbr_cel      = (int)renum->nbr_cel ;
    int    nparts       = (int)nbr_dom ;

    int    wgtflag    = 0 ; /* Pas de pondration pour les faces ou cellules */
    int    numflag    = 1 ; /* Numrotation de 1  n (type Fortran) */
    int    options[5] = {0, 3, 1, 1, 0} ; /* Par dfaut si options[0] = 0 */
    int    edgecut    = 0 ; /* <-- nombre de faces sur la partition */
    int    ind ;
    int    ind_cel_1 ;
    int    ind_cel_2 ;

    idxtype  * nbr_connect ;
    idxtype  * pos_connect ;
    idxtype  * val_connect ;


    assert(sizeof(idxtype) == sizeof(ecs_int_t)) ; /* Sinon, rinstaller
                                                      METIS avec ixdtype
                                                      modifi dans
                                                      Lib/struct.h */

    /* On doit crer une connectivit cellules <-> cellules voisines */

    BFT_MALLOC(nbr_connect, renum->nbr_cel,             idxtype);
    BFT_MALLOC(pos_connect, renum->nbr_cel + 1,         idxtype);
    BFT_MALLOC(val_connect, renum->nbr_fac_interne * 2, idxtype);

    for (ind = 0 ; ind < renum->nbr_cel ; ind++)
      nbr_connect[ind] = 0 ;

    cpt_cel_ident = 0 ;


    for (ind = 0 ; ind < renum->nbr_fac_interne ; ind++) {

      ind_cel_1 = renum->val_connect_fac_interne_cel[ind * 2    ] - 1 ;
      ind_cel_2 = renum->val_connect_fac_interne_cel[ind * 2 + 1] - 1 ;

      if (ind_cel_1 != ind_cel_2) {
        nbr_connect[ind_cel_1] += 1 ;
        nbr_connect[ind_cel_2] += 1 ;
      }
      else
        cpt_cel_ident += 1 ;

    }


    BFT_REALLOC(val_connect,
                (renum->nbr_fac_interne - cpt_cel_ident)* 2,
                ecs_int_t) ;



    pos_connect[0] = 1 ;

    for (ind = 0 ; ind < renum->nbr_cel ; ind++)
      pos_connect[ind + 1] = pos_connect[ind] + nbr_connect[ind] ;


    for (ind = 0 ; ind < renum->nbr_cel ; ind++)
      nbr_connect[ind] = 0 ;


    for (ind = 0 ; ind < renum->nbr_fac_interne ; ind++) {

      ind_cel_1 = renum->val_connect_fac_interne_cel[ind * 2    ] - 1 ;
      ind_cel_2 = renum->val_connect_fac_interne_cel[ind * 2 + 1] - 1 ;

      /*
         Test de sortie si ind_cel_1 == ind_cel_2.
         Cas possible dans certaines configuration de periodicite :
            - rotation de 180 degres avec axe coupant une face
            - maillage trop mince (une cellule d'epaisseur)
            - rotation avec un coin 'non decoupe'
      */

      if (ind_cel_1 != ind_cel_2) {

        val_connect[pos_connect[ind_cel_1] + nbr_connect[ind_cel_1] - 1]
          = ind_cel_2 + 1 ;
        val_connect[pos_connect[ind_cel_2] + nbr_connect[ind_cel_2] - 1]
          = ind_cel_1 + 1;
        nbr_connect[ind_cel_1] += 1 ;
        nbr_connect[ind_cel_2] += 1 ;

      }

    }

    BFT_FREE(nbr_connect);


    if (nparts < 8)

      METIS_PartGraphRecursive(&nbr_cel,
                               pos_connect,
                               val_connect,
                               NULL,       /* vwgt   : poids des cellules */
                               NULL,       /* adjwgt : poids des faces    */
                               &wgtflag,
                               &numflag,
                               &nparts,
                               options,
                               &edgecut,
                               num_dom_cel) ;

    else

      METIS_PartGraphKway(&nbr_cel,
                          pos_connect,
                          val_connect,
                          NULL,       /* vwgt   : poids des cellules */
                          NULL,       /* adjwgt : poids des faces    */
                          &wgtflag,
                          &numflag,
                          &nparts,
                          options,
                          &edgecut,
                          num_dom_cel) ;

    BFT_FREE(pos_connect);
    BFT_FREE(val_connect);

  }

#else

  {
    ecs_int_t   ind_cel  ;
    ecs_int_t   diviseur ;

    diviseur = renum->nbr_cel / nbr_dom ;

    if (nbr_dom * diviseur < renum->nbr_cel)
      diviseur += 1;


    for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++)

      num_dom_cel[ind_cel] = (ind_cel / diviseur) + 1 ;

  }

#endif

  renum->nbr_dom     = nbr_dom   ;
  renum->num_dom_cel = num_dom_cel ;

}


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des faces
 *----------------------------------------------------------------------------*/

void ecs_renum__connect_fac
(
       ecs_renum_t       *       renum               , /* <-> Struct. renum.  */
       ecs_bool_t                bool_voiset_cel_cel , /* --> Vois. tendu ?  */
 const ecs_param_perio_t * const liste_param_perio   , /* --> Param. priod.  */
 const ecs_real_t        * const coord_som_maillage    /* --> Coord. sommets  */
)
{

  ecs_int_t       cpt_dom ;
  ecs_int_t       ind_bcl_cel ;
  ecs_int_t       ind_cel ;
  ecs_int_t       ind_cel_loc ;
  ecs_int_t       ind_cel_loc_voiset ;
  ecs_int_t       ind_cel_par_loc ;
  ecs_int_t       ind_cel_voiset ;
  ecs_int_t       ind_cel_vis ;
  ecs_int_t       ind_dom ;
  ecs_int_t       ind_dom_vis ;
  ecs_int_t       ind_dom_voiset ;
  ecs_int_t       ind_fac ;
  ecs_int_t       ind_iter ;
  ecs_int_t       ind_loc ;
  ecs_int_t       ind_per ;
  ecs_int_t       nbr_cel_par_loc_avec_voiset ;
  ecs_int_t       num_per_voiset ;
  ecs_int_t       val_connect ;

  ecs_int_t     * cpt_cel_ext_dom ;
  ecs_int_t     * cpt_cel_ext_per ;
  ecs_int_t     * ind_fac_perio ;
  ecs_int_t     * ind_per_dir_voiset_cel_cel ;

  ecs_int_t     * num_loc_cel ;
  ecs_int_t     * nbr_cel_loc ;
  ecs_int_t     * nbr_cel_par_loc ;
  ecs_int_t     * nbr_cel_par_loc_dim ;
  ecs_int_t     * nbr_cel_par_loc_sans_voiset ;
  ecs_int_t     * nbr_fac_interne_loc ;
  ecs_int_t     * nbr_fac_de_bord_loc ;

  ecs_int_t     * pos_connect_voiset_cel_cel ;
  ecs_int_t     * val_connect_voiset_cel_cel ;

  ecs_int_t   * * num_glob_cel ;
  ecs_int_t   * * num_glob_fac ;
  ecs_int_t   * * num_glob_fbr ;
  ecs_int_t   * * num_dom_cel_ext ;
  ecs_int_t   * * num_per_cel_ext ;

  ecs_int_t   * * val_connect_fac_interne_cel_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_cel_loc ;
  ecs_int_t   * * pos_connect_voiset_cel_cel_loc ;
  ecs_int_t   * * val_connect_voiset_cel_cel_loc ;

  ecs_bool_t      bool_fac_est_perio ;
  ecs_bool_t      bool_ind_cel_voiset_vu ;

  ecs_tab_int_t   tab_num_dom ;
  ecs_tab_int_t   tab_transf ;
  ecs_tab_int_t   tab_temp ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  /* Traitement distincts en parallle et en squentiel */

  if (renum->nbr_dom == 1) {

    ecs_loc_renum__connect_fac_1(renum,
                                 bool_voiset_cel_cel,
                                 liste_param_perio,
                                 coord_som_maillage) ;

    return ;

  }


  /* Numrotation locale des cellules */
  /* -------------------------------- */

  /* A partir de num_dom_cel (numro de domaine de chaque cellule)
     fourni par le dcoupage antrieur, on calcule

        num_loc_cel     : numro local des cellules par domaine
        nbr_cel_loc     : nombre de cellules par domaine
        nbr_cel_par_loc : nombre de cellules fantmes par domaine
  */


  /* Allocation des pointeurs pour
            num_loc_cel     : numrotation locale des cellules
            nbr_cel_loc     : nombre de cellules par domaine
            nbr_cel_par_loc : nombre de cellules fantmes par domaine
  */

  BFT_MALLOC(num_loc_cel, renum->nbr_cel, ecs_int_t) ;


  BFT_MALLOC(nbr_cel_loc, renum->nbr_dom, ecs_int_t) ;
  BFT_MALLOC(nbr_cel_par_loc, renum->nbr_dom, ecs_int_t) ;

  renum->nbr_cel_loc     = nbr_cel_loc ;


  /* Initialisation du nombre de cellules locales et fantmes */

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       ind_dom++) {

    nbr_cel_loc[ind_dom] = 0 ;
    nbr_cel_par_loc[ind_dom] = 0 ;

  }


  /* Comptage */

  /*   Traitement de
         num_loc_cel     : numro local des cellules par domaine
         nbr_cel_loc     : nombre de cellules par domaine
  */

  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;

    num_loc_cel[ind_cel] = ++(nbr_cel_loc[ind_dom]) ;

  }

  /*   Traitement de
         nbr_cel_par_loc : nombre de cellules fantmes par domaine
  */

  /*     S'il y a de la priodicit, on alloue un tableau de travail de
           reprage
         ind_fac_perio : porte un numero de face pour chaque priodicit
  */

  if (renum->nbr_per > 0) {

    BFT_MALLOC(ind_fac_perio, renum->nbr_per, ecs_int_t) ;
    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      ind_fac_perio[ind_per] = 0 ;

  }

  /*     On compte les cellules fantmes  ajouter

         On compte d'abord les paralleles (eventuellement priodiques),
           ensuite les periodiques non paralleles.
  */

  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

    /* On compte ici les cellules fantmes  ajouter pour le paralllisme

       On en ajoute une pour chaque face parallle (ventuellement parallle
         et priodique). Les faces parallles sont les faces internes dont les
         deux cellules voisines sont dans des domaines diffrents.
       Si la face est priodique, le second lement de la connectivite
         face cellule pointe vers la cellule correspondante par la periodicite
         associe  la face.
    */

    ind_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2]     - 1 ;
    ind_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] - 1 ;

    if (renum->num_dom_cel[ind_cel] != renum->num_dom_cel[ind_cel_vis]) {

      ind_dom = renum->num_dom_cel[ind_cel] - 1 ;
      nbr_cel_par_loc[ind_dom] += 1 ;

      ind_dom_vis = renum->num_dom_cel[ind_cel_vis] - 1 ;
      nbr_cel_par_loc[ind_dom_vis] += 1 ;

    }

    /* On compte ici les cellules fantomes  ajouter pour
         les faces internes priodiques mais non parallles

       Pour cela, on teste s'il existe une priodicit
         (ind_per avec le for(ind_per++))
         dont la liste de faces priodiques associe (liste_fac_perio)
         contient le numro de la face courante (ind_fac)
       Dans le cas ou la face est periodique et qu'elle n'est pas parallele
         il faut ajouter une cellule fantme (nbr_cel_par_loc+=1)
    */

    bool_fac_est_perio = ECS_FALSE ;

    for (ind_per = 0 ;
         ind_per < renum->nbr_per && bool_fac_est_perio == ECS_FALSE ;
         ind_per++)

      while (ind_fac_perio[ind_per] < renum->nbr_fac_perio[ind_per] &&
             renum->liste_fac_perio[ind_per]
                                   [ind_fac_perio[ind_per]] <= ind_fac &&
             bool_fac_est_perio == ECS_FALSE)

        if (ind_fac == renum->liste_fac_perio[ind_per]
                                             [ind_fac_perio[ind_per]++])
          bool_fac_est_perio = ECS_TRUE ;


    if (renum->num_dom_cel[ind_cel] == renum->num_dom_cel[ind_cel_vis] &&
        bool_fac_est_perio == ECS_TRUE)
      nbr_cel_par_loc[renum->num_dom_cel[ind_cel] - 1] += 1 ;

  }
  /* Fin de la boucle sur les faces internes  */

  /* Allocations d'entits par domaine
        num_glob_cel    : numro global des cellules
        num_loc_cel_ext : numro local des cellules fantmes
        num_dom_cel_ext : numro de domaine dont provient
                                   l'info des cellules fantmes
        num_per_cel_ext : numro de priodicit associ aux cellules fantmes
   */

  BFT_MALLOC(num_glob_cel, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(num_dom_cel_ext, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(num_per_cel_ext, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(renum->num_loc_cel_ext, renum->nbr_dom, ecs_int_t *) ;

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       ind_dom++) {

    BFT_MALLOC(num_glob_cel[ind_dom], nbr_cel_loc[ind_dom], ecs_int_t) ;
    BFT_MALLOC(num_dom_cel_ext[ind_dom], nbr_cel_par_loc[ind_dom], ecs_int_t) ;
    BFT_MALLOC(num_per_cel_ext[ind_dom], nbr_cel_par_loc[ind_dom], ecs_int_t) ;
    BFT_MALLOC(renum->num_loc_cel_ext[ind_dom], nbr_cel_par_loc[ind_dom],
               ecs_int_t) ;

  }

  renum->num_glob_cel    = num_glob_cel ;


  /* Remplissage du numro global de cellule (par domaine) */

  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;

    renum->num_glob_cel[ind_dom][num_loc_cel[ind_cel] - 1] = ind_cel + 1 ;

  }

  /* On a ici calcul
       num_glob_cel    : numro global des cellules  par domaine
       num_loc_cel     : numro local des cellules   par domaine
       nbr_cel_loc     : nombre de cellules          par domaine
       nbr_cel_par_loc : nombre de cellules fantmes par domaine

     mais pas encore
       num_loc_cel_ext : numro local des cellules fantmes
       num_dom_cel_ext : numro de domaine dont provient l'information
                         a verser dans les cellules fantmes
       num_per_cel_ext : numro de priodicit associ aux cellules fantmes
  */


  /* Traitement des faces */
  /* -------------------- */

  /* Comptage des faces par domaine
       nbr_fac_interne_loc : nombre de faces internes par domaine
       nbr_fac_de_bord_loc : nombre de faces de bord  par domaine
  */

  BFT_MALLOC(nbr_fac_interne_loc, renum->nbr_dom, ecs_int_t) ;
  BFT_MALLOC(nbr_fac_de_bord_loc, renum->nbr_dom, ecs_int_t) ;

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       ind_dom++) {

    nbr_fac_interne_loc[ind_dom] = 0 ;
    nbr_fac_de_bord_loc[ind_dom] = 0 ;

  }


  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

    ind_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2]     - 1 ;
    ind_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] - 1 ;

    ind_dom     = renum->num_dom_cel[ind_cel]     - 1 ;
    ind_dom_vis = renum->num_dom_cel[ind_cel_vis] - 1 ;

    nbr_fac_interne_loc[ind_dom] += 1 ;

    if (ind_dom != ind_dom_vis)
      nbr_fac_interne_loc[ind_dom_vis] += 1 ;

  }


  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_de_bord ; ind_fac++) {

    ind_cel = renum->val_connect_fac_de_bord_cel[ind_fac] - 1 ;
    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;
    nbr_fac_de_bord_loc[ind_dom] += 1 ;

  }


  renum->nbr_fac_interne_loc = nbr_fac_interne_loc ;
  renum->nbr_fac_de_bord_loc = nbr_fac_de_bord_loc ;


  /* Allocations par domaine : numro de faces, connectivite, communication

       num_glob_fac : numro global des faces internes
       num_glob_fbr : numro global des faces de bord

       val_connect_fac_interne_cel_loc :
                      connectivit locale faces internes -> cellules
       val_connect_fac_de_bord_cel_loc :
                      connectivit locale faces de bord  -> cellules

       nbr_dom_cel_ext : nombre de domaines voisins
       num_dom_cel_ext : numro des domaines voisins
       pos_dom_cel_ext : position des blocs de cellules fantmes  destination
                         des autres domaines (de tous les autres, pas seulement
                         des voisins)
  */

  BFT_MALLOC(num_glob_fac, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(num_glob_fbr, renum->nbr_dom, ecs_int_t *) ;

  BFT_MALLOC(val_connect_fac_interne_cel_loc, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_cel_loc, renum->nbr_dom, ecs_int_t *) ;

  BFT_MALLOC(renum->nbr_dom_cel_ext, renum->nbr_dom, ecs_int_t) ;
  BFT_MALLOC(renum->num_dom_cel_ext, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(renum->pos_dom_cel_ext, renum->nbr_dom, ecs_int_t *) ;

  if (renum->nbr_per > 0)
    BFT_MALLOC(renum->pos_per_cel_ext, renum->nbr_dom, ecs_int_t *) ;

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       ind_dom++) {

    BFT_MALLOC(num_glob_fac[ind_dom],
               nbr_fac_interne_loc[ind_dom], ecs_int_t) ;
    BFT_MALLOC(num_glob_fbr[ind_dom],
               nbr_fac_de_bord_loc[ind_dom], ecs_int_t) ;

    BFT_MALLOC(val_connect_fac_interne_cel_loc[ind_dom],
               nbr_fac_interne_loc[ind_dom] * 2, ecs_int_t) ;
    BFT_MALLOC(val_connect_fac_de_bord_cel_loc[ind_dom],
               nbr_fac_de_bord_loc[ind_dom], ecs_int_t) ;

  }

  renum->num_glob_fac = num_glob_fac ;
  renum->num_glob_fbr = num_glob_fbr ;

  renum->val_connect_fac_interne_cel_loc = val_connect_fac_interne_cel_loc ;
  renum->val_connect_fac_de_bord_cel_loc = val_connect_fac_de_bord_cel_loc ;


  /* Traitement des faces internes parallles */

  if (renum->nbr_per > 0)
    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      ind_fac_perio[ind_per] = 0 ;

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

    nbr_fac_interne_loc[ind_dom] = 0 ;
    nbr_cel_par_loc[ind_dom] = 0 ;

  }


  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

    ind_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2]     - 1 ;
    ind_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] - 1 ;

    ind_dom     = renum->num_dom_cel[ind_cel]     - 1 ;
    ind_dom_vis = renum->num_dom_cel[ind_cel_vis] - 1 ;


    /* On vrifie si la face interne est une face priodique ou non
         et si elle est priodique, on dtermine la priodicit ind_per
    */

    bool_fac_est_perio = ECS_FALSE ;

    for (ind_per = 0 ;
         ind_per < renum->nbr_per && bool_fac_est_perio == ECS_FALSE ;
         ind_per++)

      while (ind_fac_perio[ind_per] < renum->nbr_fac_perio[ind_per] &&
             renum->liste_fac_perio[ind_per]
                                   [ind_fac_perio[ind_per]] <= ind_fac &&
             bool_fac_est_perio == ECS_FALSE)

        if (ind_fac == renum->liste_fac_perio[ind_per]
                                             [ind_fac_perio[ind_per]++])
          bool_fac_est_perio = ECS_TRUE ;


    /* On calcule :
         val_connect_fac_interne_cel_loc
                             : connectivit locale face -> cellule
         num_glob_fac        : numro global des faces internes
         nbr_fac_interne_loc : nbre de faces internes local (recalcul)
       et on ajoute ventuellement des cellules fantmes
         num_dom_cel_ext     : numro du domaine voisin
         num_loc_cel_ext     : numro local des cellules fantmes
         nbr_cel_par_loc     : nombre de cellules fantmes locales (recalcul)
         num_per_cel_ext     : numro de la priodicit associe  la face
                                 (0 : non priodique, +/- selon le sens)
    */

    /* Si la face n'est pas priodique
       et si les cellules voisines sont dans le mme domaine
         on traite en mme temps les deux cellules voisines
     */

    if (ind_dom == ind_dom_vis && bool_fac_est_perio == ECS_FALSE) {

      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2]
        = num_loc_cel[ind_cel] ;
      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2 + 1]
        = num_loc_cel[ind_cel_vis] ;

      num_glob_fac[ind_dom][nbr_fac_interne_loc[ind_dom]] = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom] += 1 ;

    }

    /* Si l'on est sur une frontire parallle mais non priodique
       on traite en mme temps les deux cellules voisines
    */

    else if (ind_dom != ind_dom_vis && bool_fac_est_perio == ECS_FALSE) {

      /* On ajoute des cellules tendues ("cellules fantmes") */

      renum->num_loc_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
        = num_loc_cel[ind_cel_vis];
      num_dom_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
        = ind_dom_vis + 1 ;

      nbr_cel_par_loc[ind_dom] += 1 ;

      renum->num_loc_cel_ext[ind_dom_vis][nbr_cel_par_loc[ind_dom_vis]]
        = num_loc_cel[ind_cel] ;
      num_dom_cel_ext[ind_dom_vis][nbr_cel_par_loc[ind_dom_vis]]
        = ind_dom + 1 ;

      nbr_cel_par_loc[ind_dom_vis] += 1 ;

      num_per_cel_ext[ind_dom    ][nbr_cel_par_loc[ind_dom    ] - 1] = 0 ;
      num_per_cel_ext[ind_dom_vis][nbr_cel_par_loc[ind_dom_vis] - 1] = 0 ;

      /* La connectivit se fait par rapport aux cellules tendues,
         qui s'ajoutent en fin de liste des cellules locales */

      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2]
        = num_loc_cel[ind_cel] ;
      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2 + 1]
        = nbr_cel_loc[ind_dom] + nbr_cel_par_loc[ind_dom] ;

      val_connect_fac_interne_cel_loc[ind_dom_vis]
                                     [nbr_fac_interne_loc[ind_dom_vis] * 2]
        = nbr_cel_loc[ind_dom_vis] + nbr_cel_par_loc[ind_dom_vis] ;
      val_connect_fac_interne_cel_loc[ind_dom_vis]
                                     [nbr_fac_interne_loc[ind_dom_vis] * 2 + 1]
        = num_loc_cel[ind_cel_vis] ;


      /* Correspondance numro local <-> numro global */

      num_glob_fac[ind_dom][nbr_fac_interne_loc[ind_dom]]
        = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom] += 1 ;

      num_glob_fac[ind_dom_vis][nbr_fac_interne_loc[ind_dom_vis]]
        = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom_vis] += 1 ;

    }

    /* Si la face "interne" considre est priodique (parallle ou non)
       on ne traite qu'une cellule (la cellule correspondante par priodicit
       sera vue par une autre face)
    */

    else {

      renum->num_loc_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
        = num_loc_cel[ind_cel_vis];
      num_dom_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
        = ind_dom_vis + 1 ;

      nbr_cel_par_loc[ind_dom] += 1 ;

      if (renum->dir_per_fac_perio[ind_per - 1]
                                  [ind_fac_perio[ind_per - 1] - 1] > 0)
        num_per_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom] - 1] =  ind_per ;
      else
        num_per_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom] - 1] = -ind_per ;

      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2]
        = num_loc_cel[ind_cel] ;
      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2 + 1]
        = nbr_cel_loc[ind_dom] + nbr_cel_par_loc[ind_dom] ;

      num_glob_fac[ind_dom][nbr_fac_interne_loc[ind_dom]] = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom] += 1 ;


    }

  }
  /* Fin de la boucle sur les faces internes */

  if (renum->nbr_per > 0)
    BFT_FREE(ind_fac_perio) ;


  /* On a ici calcul :
       num_glob_cel    : numro global des cellules  par domaine
       num_loc_cel     : numro local des cellules   par domaine
       nbr_cel_loc     : nombre de cellules          par domaine
       nbr_cel_par_loc : nombre de cellules fantmes par domaine

     et aussi :
       num_loc_cel_ext : numro local des cellules fantmes
       num_dom_cel_ext : numro de domaine dont provient l'information
                          verser dans les cellules fantmes
       num_per_cel_ext : numro de priodicit associ aux cellules fantmes

     et enfin :
       num_glob_fac : numro global des faces internes
       val_connect_fac_interne_cel_loc :
                      connectivit locale faces internes -> cellules

     mais pas encore :
       num_glob_fbr : numro global des faces de bord
       val_connect_fac_de_bord_cel_loc :
                      connectivit locale faces de bord  -> cellules
       nbr_dom_cel_ext : nombre de domaines voisins
       pos_dom_cel_ext : position des blocs de cellules fantmes  destination
                         des autres domaines (de tous les autres,
                         pas seulement des voisins)
  */




  /*
     Voisinage tendu :


     On dtermine, pour chaque cellule, le voisinage tendu, i.e. l'ensemble
       des cellules qui partagent un noeud avec la cellule considre.
     Pour une cellule donne, les cellules voisines vues au travers des
       faces ne sont pas incluses dans le voisinage tendu.
     Il faut prendre en compte les cellules situes sur des domaines voisins
       et les cellules correspondantes en priodicit.

     Dans un premier temps et de toute facons pour la priodicit de rotation,
       on tend simplement la liste des cellules fantmes.
     Dans un second temps, sans priodicit de rotation,
       on pourra stocker sparement les cellules fantmes supplmentaires
       ncessaires au voisinage tendu.
  */


  /* On spare le voisinage tendu pour allger les changes en parallle :
     pas de priodicit de rotation */
  renum->bool_voiset_separe
    = ((bool_voiset_cel_cel == ECS_TRUE) &&
       (renum->nbr_per_rot == 0)) ? ECS_TRUE : ECS_FALSE ;


  if (bool_voiset_cel_cel == ECS_TRUE) {


    /*
       On construit la connectivit allge cellule -> cellules
     (i.e. voisinage tendu, sans les cellules vues au travers des faces
    */

    ecs_loc_renum__voiset_cel_cel(renum,
                                  liste_param_perio,
                                  coord_som_maillage,
                                  &pos_connect_voiset_cel_cel,
                                  &val_connect_voiset_cel_cel,
                                  &ind_per_dir_voiset_cel_cel) ;


    /*
       On construit les cellules fantmes supplmentaires pour la
       connectivit voisinage tendu cellules cellules.

       On construit galement la connectivite locale
       voisinage tendu cellule cellule.
    */


    /* Allocation */

    /* Allocation des pointeurs de pointeur */
    BFT_MALLOC(val_connect_voiset_cel_cel_loc, renum->nbr_dom, ecs_int_t *) ;
    BFT_MALLOC(pos_connect_voiset_cel_cel_loc, renum->nbr_dom, ecs_int_t *) ;

    renum->val_connect_voiset_cel_cel_loc = val_connect_voiset_cel_cel_loc ;
    renum->pos_connect_voiset_cel_cel_loc = pos_connect_voiset_cel_cel_loc ;


    /* Allocation des pointeurs pos_* par domaine  */
    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++)
      BFT_MALLOC(pos_connect_voiset_cel_cel_loc[ind_dom],
                 nbr_cel_loc[ind_dom]+1, ecs_int_t) ;

    /*
       Pour allouer les val_*, il suffirait de calculer la taille+1 dans le
       dernier lment des pos_*, mais on prfre remplir des  prsent tous
       les pos_*, puisqu'il faudra de toute facon le faire.

       On stocke d'abord le nombre de voisins de chaque cellule ind_cel_loc
       dans pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc+1]
       puis on somme sur les ind_cel_loc pour obtenir les bonnes valeurs de
       pos_connect_voiset_cel_cel_loc[ind_dom][]
       la dernire valeur de pos_connect_voiset_cel_cel_loc[ind_dom][] est la
       taille+1 des val_*
    */

    for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {
      ind_dom = renum->num_dom_cel[ind_cel] - 1 ;
      ind_cel_loc = num_loc_cel[ind_cel] - 1 ;
      pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc+1]
        = pos_connect_voiset_cel_cel[ind_cel+1]
        - pos_connect_voiset_cel_cel[ind_cel] ;
    }

    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

      pos_connect_voiset_cel_cel_loc[ind_dom][0] = 1 ;
      for (ind_cel_loc = 0 ; ind_cel_loc < nbr_cel_loc[ind_dom] ; ind_cel_loc++)
        pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc+1]
          += pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc] ;

      BFT_MALLOC(val_connect_voiset_cel_cel_loc[ind_dom],
                 pos_connect_voiset_cel_cel_loc[ind_dom]
                                               [nbr_cel_loc[ind_dom]]-1,
                 ecs_int_t) ;
    }


    /*
       On conserve le nombre de cellules fantmes sans voisinage tendu
       et on rserve un tableau pour allonger la taille des tableaux
    */
    BFT_MALLOC(nbr_cel_par_loc_sans_voiset, renum->nbr_dom, ecs_int_t) ;
    BFT_MALLOC(nbr_cel_par_loc_dim,         renum->nbr_dom, ecs_int_t) ;
    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
      nbr_cel_par_loc_sans_voiset[ind_dom] = nbr_cel_par_loc[ind_dom] ;
      nbr_cel_par_loc_dim[ind_dom]         = nbr_cel_par_loc[ind_dom] ;
    }


    /* Construction */

    /* On parcourt toutes les cellules globales */
    for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {
      ind_dom = renum->num_dom_cel[ind_cel] - 1 ;
      ind_cel_loc = num_loc_cel[ind_cel] - 1 ;

      /* On parcourt les cellules du voisinage tendu de la cellule courante */
      for (ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel]   - 1,
             ind_iter = 0 ;
           ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel+1] - 1 ;
           ind_bcl_cel++,
             ind_iter++) {

        ind_cel_voiset = val_connect_voiset_cel_cel[ind_bcl_cel] - 1 ;
        ind_dom_voiset = renum->num_dom_cel[ind_cel_voiset] - 1 ;
        ind_cel_loc_voiset = num_loc_cel[ind_cel_voiset] - 1 ;
        num_per_voiset = ind_per_dir_voiset_cel_cel[ind_bcl_cel] ;

        /* si ncessaire :
           ind_per_voiset = ECS_ABS(ind_per_dir)-1 ;
           dir_per_voiset = (ind_per_dir > 0) ? 1 : -1 ; */


        /*
           Si la cellule et sa voisine tendue correspondent par priodicit
           il faut convertir la connectivit en connectivit locale
           et ajouter une cellule fantme tendue si ce n'est pas dj fait.
        */

        if (num_per_voiset != 0 ) {

          /* On regarde si on a dj ajout la cellule fantme */
          /* On suppose que des priodicits de numro diffrent sont
             effectivement diffrentes. Ce n'est pas obligatoirement le cas */
          bool_ind_cel_voiset_vu = ECS_FALSE ;

          /* faux : ind_cel_par_loc = nbr_cel_par_loc_sans_voiset[ind_dom] ; */

          ind_cel_par_loc = 0 ;

          while (   bool_ind_cel_voiset_vu == ECS_FALSE
                 && ind_cel_par_loc < nbr_cel_par_loc[ind_dom]) {

            if (   (   renum->num_loc_cel_ext[ind_dom][ind_cel_par_loc]
                    == ind_cel_loc_voiset + 1)
                && (   num_dom_cel_ext[ind_dom][ind_cel_par_loc]
                    == ind_dom_voiset + 1)
                && (   num_per_cel_ext[ind_dom][ind_cel_par_loc]
                    == num_per_voiset))
              bool_ind_cel_voiset_vu = ECS_TRUE ;
            else
              ind_cel_par_loc++;
          }

          /* Si on n'a pas encore ajout la cellule fantme :
             on le fait et on traduit la connectivit */

          if (bool_ind_cel_voiset_vu == ECS_FALSE) {

            if (nbr_cel_par_loc[ind_dom]+1 > nbr_cel_par_loc_dim[ind_dom]) {
              nbr_cel_par_loc_dim[ind_dom] = 2*nbr_cel_par_loc_dim[ind_dom] ;
              BFT_REALLOC(renum->num_loc_cel_ext[ind_dom],
                          nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
              BFT_REALLOC(num_dom_cel_ext[ind_dom],
                          nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
              BFT_REALLOC(num_per_cel_ext[ind_dom],
                          nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
            }

            renum->num_loc_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
              = ind_cel_loc_voiset + 1 ;
            num_dom_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
              = ind_dom_voiset + 1 ;
            num_per_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
              = num_per_voiset     ;

            nbr_cel_par_loc[ind_dom]++ ;

            val_connect_voiset_cel_cel_loc
              [ind_dom]
              [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc]
               + ind_iter - 1]
              = nbr_cel_loc[ind_dom] + nbr_cel_par_loc[ind_dom] ;

          }

          /* Si on a dj ajout la cellule fantme,
             on se contente de traduire la connectivit */

          else

            val_connect_voiset_cel_cel_loc
              [ind_dom]
              [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc]
               +ind_iter - 1]
              = nbr_cel_loc[ind_dom] + ind_cel_par_loc + 1 ;

        }

        /*
           Si la cellule et sa voisine tendue ne correspondent pas par
           priodicit et sont en outre dans le mme domaine,
           il suffit de convertir la connectivit en connectivit locale
        */

        else if (ind_dom == ind_dom_voiset) {
          val_connect_voiset_cel_cel_loc
            [ind_dom]
            [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc] + ind_iter - 1]
            = ind_cel_loc_voiset + 1 ;
        }

        /*
           Si la cellule et sa voisine tendue ne correspondent pas par
           priodicit mais ne sont pas dans le mme domaine,
           il faut convertir la connectivit en connectivit locale
           et ajouter une cellule fantme si ce n'est pas dj fait
        */

        else {

          /* On regarde si on a dj ajout la cellule fantme */
          /* On suppose que des priodicits de numro diffrent sont
             effectivement diffrentes. Ce n'est pas obligatoirement le cas */

          bool_ind_cel_voiset_vu = ECS_FALSE ;

          /* faux : ind_cel_par_loc = nbr_cel_par_loc_sans_voiset[ind_dom] ; */

          ind_cel_par_loc = 0 ;

          while (   bool_ind_cel_voiset_vu == ECS_FALSE
                 && ind_cel_par_loc < nbr_cel_par_loc[ind_dom]) {

            if (   (   renum->num_loc_cel_ext[ind_dom][ind_cel_par_loc]
                    == ind_cel_loc_voiset + 1)
                && (   num_dom_cel_ext[ind_dom][ind_cel_par_loc]
                    == ind_dom_voiset + 1)
                && (   num_per_cel_ext[ind_dom][ind_cel_par_loc]
                    == 0))
              bool_ind_cel_voiset_vu = ECS_TRUE ;
            else
              ind_cel_par_loc++;

          }

          /* Si on n'a pas encore ajout la cellule fantme :
             on le fait et on traduit la connectivit */

          if (bool_ind_cel_voiset_vu == ECS_FALSE) {

            if (nbr_cel_par_loc[ind_dom]+1 > nbr_cel_par_loc_dim[ind_dom]) {
              nbr_cel_par_loc_dim[ind_dom] = 2*nbr_cel_par_loc_dim[ind_dom] ;
              BFT_REALLOC(renum->num_loc_cel_ext[ind_dom],
                          nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
              BFT_REALLOC(num_dom_cel_ext[ind_dom],
                          nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
              BFT_REALLOC(num_per_cel_ext[ind_dom],
                          nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
            }

            renum->num_loc_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
              = ind_cel_loc_voiset + 1 ;
            num_dom_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
              = ind_dom_voiset + 1 ;
            num_per_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]] = 0 ;

            nbr_cel_par_loc[ind_dom]++ ;

            val_connect_voiset_cel_cel_loc[ind_dom]
              [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc]+ind_iter-1]
              = nbr_cel_loc[ind_dom] + nbr_cel_par_loc[ind_dom] ;

          }

          /* Si on a dj ajout la cellule fantme,
             on se contente de traduire la connectivit */
          else
            val_connect_voiset_cel_cel_loc
              [ind_dom]
              [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc]
               + ind_iter - 1]
              = nbr_cel_loc[ind_dom] + ind_cel_par_loc + 1 ;

        } /* Fin si priodique ou non, mme domaine ou non */

      } /* Fin du parcours du voisinage tendu de la cellule courante */

    } /* Fin de la boucle sur les cellules */


    /* On ralloue les tableaux relatifs aux fantmes  la taille exacte */

    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
      BFT_REALLOC(renum->num_loc_cel_ext[ind_dom],
                  nbr_cel_par_loc[ind_dom], ecs_int_t) ;
      BFT_REALLOC(num_dom_cel_ext[ind_dom],
                  nbr_cel_par_loc[ind_dom], ecs_int_t) ;
      BFT_REALLOC(num_per_cel_ext[ind_dom],
                  nbr_cel_par_loc[ind_dom], ecs_int_t) ;
    }


    /*
      On libere la connectivit voisinage tendu cellule -> cellules
      On conserve la connectivit locale voisinage tendu cellule -> cellules
    */
    BFT_FREE(pos_connect_voiset_cel_cel) ;
    BFT_FREE(val_connect_voiset_cel_cel) ;
    BFT_FREE(ind_per_dir_voiset_cel_cel) ;


    /*
       Si ncessaire, on conserve le nombre de cellules voisinage tendu,
       afin de pouvoir les traiter sparment
    */

    if (renum->bool_voiset_separe == ECS_TRUE) {

      BFT_MALLOC(renum->nbr_cel_par_loc_voiset, renum->nbr_dom, ecs_int_t) ;

      for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
        renum->nbr_cel_par_loc_voiset[ind_dom]
          = nbr_cel_par_loc[ind_dom] - nbr_cel_par_loc_sans_voiset[ind_dom] ;
        nbr_cel_par_loc[ind_dom] = nbr_cel_par_loc_sans_voiset[ind_dom] ;
      }

    }

    /* On libre les tableaux temporaires de nombre de cellules fantmes */
    BFT_FREE(nbr_cel_par_loc_sans_voiset) ;
    BFT_FREE(nbr_cel_par_loc_dim) ;

  } /* Fin si voisinage tendu */


  /*
    Renumrotation des cellules fantmes :

     On trie les cellules fantmes comme suit, par domaine "i" puis :
     - si le domaine "i" est le domaine courant :
     cel perio1 // cel perio2 // cel perio3 (cellules uniquement periodiques)
     - si le domaine "i" n'est pas le domaine courant :
     cel uniquement paralleles // cel perio1 // cel perio2 // cel perio3

     De plus, elles sont triees en 2 groupes par periodicite :
     - sens normal, d'abord
     - sens inverse, ensuite
     Il n'y a pas de periodicite 'bizarre' ou une face se voit elle-meme, donc
     il y a autant de faces dans un groupe que dans l'autre.

     Et on retouche donc :
       num_loc_cel_ext : numro local des cellules fantmes
       num_dom_cel_ext : numro de domaine dont provient l'information
                          verser dans les cellules fantmes
       num_per_cel_ext : numro de priodicit associ aux cellules fantmes
       val_connect_fac_interne_cel_loc :
                         connectivit locale faces internes -> cellules
       pos_connect_voiset_cel_cel_loc et
       val_connect_voiset_cel_cel_loc
                       : connectivit voisinage tendu cellule -> cellules

     Et on calcule :
       nbr_dom_cel_ext : nombre de domaines voisins
       pos_dom_cel_ext : position des blocs de cellules fantmes  destination
                         des autres domaines (de tous les autres,
                         pas seulement des voisins)
       pos_per_cel_ext : position des blocs de cellules fantmes priodiques
                                                                  destination
                         des autres domaines (de tous les autres,
                         pas seulement des voisins)
       num_dom_cel_ext : numro de domaine dont provient l'information
                          verser dans les cellules fantmes (dans renum->)

    Si l'on veut que le voisinage tendu soit stock  part, on le met  la fin
      de chaque domaine, dans le meme ordre que les cellules fantmes classiques
      (parallle pur puis priodicits).

  */

  if (renum->bool_voiset_separe == ECS_TRUE) {
    BFT_MALLOC(renum->nbr_dom_cel_ext_voiset, renum->nbr_dom, ecs_int_t) ;
    BFT_MALLOC(renum->num_dom_cel_ext_voiset, renum->nbr_dom, ecs_int_t *) ;
    BFT_MALLOC(renum->pos_dom_cel_ext_voiset, renum->nbr_dom, ecs_int_t *) ;
    if (renum->nbr_per > 0)
      BFT_MALLOC(renum->pos_per_cel_ext_voiset, renum->nbr_dom, ecs_int_t *) ;
  }


  renum->nbr_cel_par_loc = nbr_cel_par_loc ;

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       ind_dom++) {

    nbr_cel_par_loc_avec_voiset = renum->nbr_cel_par_loc[ind_dom] ;

    if (renum->bool_voiset_separe == ECS_TRUE)
      nbr_cel_par_loc_avec_voiset += renum->nbr_cel_par_loc_voiset[ind_dom] ;

    tab_num_dom.nbr = nbr_cel_par_loc_avec_voiset ;
    BFT_MALLOC(tab_num_dom.val, tab_num_dom.nbr, ecs_int_t) ;

    /*
       Determination de l'ordre, on trie selon tab_num_dom.val,
       dans l'ordre croissant
    */

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < renum->nbr_cel_par_loc[ind_dom] ;
         ind_cel_par_loc++) {

      if (num_per_cel_ext[ind_dom][ind_cel_par_loc] > 0)
        tab_num_dom.val[ind_cel_par_loc]
          =   2*( num_per_cel_ext[ind_dom][ind_cel_par_loc])
            + (2*renum->nbr_per + 1)
              * num_dom_cel_ext[ind_dom][ind_cel_par_loc] ;
      else
        tab_num_dom.val[ind_cel_par_loc]
          =   2*(-num_per_cel_ext[ind_dom][ind_cel_par_loc]) + 1
            + (2*renum->nbr_per + 1)
              * num_dom_cel_ext[ind_dom][ind_cel_par_loc] ;

    }

    if (renum->bool_voiset_separe == ECS_TRUE) {

      for (ind_cel_par_loc = renum->nbr_cel_par_loc[ind_dom] ;
           ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
           ind_cel_par_loc++) {

        if (num_per_cel_ext[ind_dom][ind_cel_par_loc] > 0) {
          tab_num_dom.val[ind_cel_par_loc]
            =    2*(num_per_cel_ext[ind_dom][ind_cel_par_loc])
              + (2*renum->nbr_per + 1) * num_dom_cel_ext[ind_dom]
                                                        [ind_cel_par_loc]
              + (2*renum->nbr_per + 1) * (renum->nbr_dom + 1) ;
        }
        else {
          tab_num_dom.val[ind_cel_par_loc]
            =   2*(-num_per_cel_ext[ind_dom][ind_cel_par_loc]) + 1
              + (2*renum->nbr_per + 1) * num_dom_cel_ext[ind_dom]
                                                        [ind_cel_par_loc]
              + (2*renum->nbr_per + 1) * (renum->nbr_dom + 1) ;
        }

      }

    }

    tab_transf.nbr = nbr_cel_par_loc_avec_voiset ;
    BFT_MALLOC(tab_transf.val, tab_transf.nbr, ecs_int_t) ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < tab_transf.nbr ;
         ind_cel_par_loc++)
      tab_transf.val[ind_cel_par_loc] = ind_cel_par_loc ;

    ecs_tab_int__trie(tab_num_dom, tab_transf) ;
    BFT_FREE(tab_num_dom.val) ;

    if (tab_transf.nbr > 0)
      ecs_tab_int__inverse(&tab_transf) ;

    /* On traite la connectivit face -> cellule */

    for (ind_fac = 0 ; ind_fac < nbr_fac_interne_loc[ind_dom] ; ind_fac++) {

      val_connect =   val_connect_fac_interne_cel_loc[ind_dom]
                                                     [ind_fac * 2]
                    - nbr_cel_loc[ind_dom] - 1 ;

      if (val_connect > -1)
        val_connect_fac_interne_cel_loc[ind_dom][ind_fac * 2]
          = tab_transf.val[val_connect] + nbr_cel_loc[ind_dom] + 1 ;

      val_connect =   val_connect_fac_interne_cel_loc[ind_dom]
                                                     [ind_fac * 2 + 1]
                    - nbr_cel_loc[ind_dom] - 1 ;

      if (val_connect > -1)
        val_connect_fac_interne_cel_loc[ind_dom][ind_fac * 2 + 1]
          = tab_transf.val[val_connect] + nbr_cel_loc[ind_dom] + 1 ;

    }

    /* On traite la connectivit voisinage tendu cellule -> cellule */

    if (bool_voiset_cel_cel == ECS_TRUE) {

      for (ind_cel_loc = 0 ;
           ind_cel_loc < nbr_cel_loc[ind_dom] ;
           ind_cel_loc++) {

        for (ind_cel_loc_voiset
               = pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc  ]-1 ;
             ind_cel_loc_voiset
               < pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc+1]-1 ;
             ind_cel_loc_voiset++) {

          /* On soustrait nbr_cel_loc[ind_dom]+1 pour etre >= 0 si fantme
             on rtablit ensuite */

          val_connect
            =   val_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc_voiset]
              - nbr_cel_loc[ind_dom] - 1 ;

          if (val_connect > -1)
            val_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc_voiset]
              = tab_transf.val[val_connect] + nbr_cel_loc[ind_dom] + 1 ;

        }

      }

    }


  /*
       On rordonne successivement :
       numro local des cellules fantmes
       numro de domaine correspondant
       numro de priodicit associ
    */

    if (tab_transf.nbr > 0)
      ecs_tab_int__inverse(&tab_transf);

    tab_temp.nbr = nbr_cel_par_loc_avec_voiset ;
    BFT_MALLOC(tab_temp.val, tab_temp.nbr, ecs_int_t) ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
         ind_cel_par_loc++)
      tab_temp.val[ind_cel_par_loc]
        = renum->num_loc_cel_ext[ind_dom][ind_cel_par_loc] ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
         ind_cel_par_loc++)
      renum->num_loc_cel_ext[ind_dom][ind_cel_par_loc]
        = tab_temp.val[tab_transf.val[ind_cel_par_loc]] ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
         ind_cel_par_loc++)
      tab_temp.val[ind_cel_par_loc]
        = num_dom_cel_ext[ind_dom][ind_cel_par_loc] ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
         ind_cel_par_loc++)
      num_dom_cel_ext[ind_dom][ind_cel_par_loc]
        = tab_temp.val[tab_transf.val[ind_cel_par_loc]] ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
         ind_cel_par_loc++)
      tab_temp.val[ind_cel_par_loc]
        = num_per_cel_ext[ind_dom][ind_cel_par_loc] ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
         ind_cel_par_loc++)
      num_per_cel_ext[ind_dom][ind_cel_par_loc]
        = tab_temp.val[tab_transf.val[ind_cel_par_loc]] ;

    BFT_FREE(tab_temp.val) ;
    BFT_FREE(tab_transf.val) ;


    /*
       Identification des domaines auxquels appartiennent les cellules fantme :
                 renum->nbr_dom_cel_ext,
                 renum->num_dom_cel_ext,
                 renum->pos_dom_cel_ext,
       et, si le voisinage tendu est spar :
                 renum->nbr_dom_cel_ext_voiset,
                 renum->num_dom_cel_ext_voiset,
                 renum->pos_dom_cel_ext_voiset.
    */

    BFT_MALLOC(cpt_cel_ext_dom, renum->nbr_dom, ecs_int_t) ;

    for (ind_loc = 0 ; ind_loc < renum->nbr_dom ; ind_loc++)
      cpt_cel_ext_dom[ind_loc] = 0 ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < renum->nbr_cel_par_loc[ind_dom] ;
         ind_cel_par_loc++)
      cpt_cel_ext_dom[num_dom_cel_ext[ind_dom][ind_cel_par_loc] - 1] += 1 ;

    renum->nbr_dom_cel_ext[ind_dom] = 0 ;

    for (ind_loc = 0 ; ind_loc < renum->nbr_dom ; ind_loc++) {
      if (cpt_cel_ext_dom[ind_loc] > 0)
        renum->nbr_dom_cel_ext[ind_dom] += 1 ;
    }

    BFT_MALLOC(renum->num_dom_cel_ext[ind_dom],
               renum->nbr_dom_cel_ext[ind_dom], ecs_int_t) ;
    BFT_MALLOC(renum->pos_dom_cel_ext[ind_dom],
               renum->nbr_dom_cel_ext[ind_dom] + 1, ecs_int_t) ;

    renum->pos_dom_cel_ext[ind_dom][0] = 1 ;

    cpt_dom = 0 ;

    for (ind_loc = 0 ; ind_loc < renum->nbr_dom ; ind_loc++) {
      if (cpt_cel_ext_dom[ind_loc] > 0) {
        renum->num_dom_cel_ext[ind_dom][cpt_dom] = ind_loc + 1 ;
        renum->pos_dom_cel_ext[ind_dom][cpt_dom + 1]
          =   renum->pos_dom_cel_ext[ind_dom][cpt_dom]
            + cpt_cel_ext_dom[ind_loc] ;
        cpt_dom += 1 ;
      }
    }

    if (renum->bool_voiset_separe == ECS_TRUE) {

      for (ind_loc = 0 ; ind_loc < renum->nbr_dom ; ind_loc++)
        cpt_cel_ext_dom[ind_loc] = 0 ;

      for (ind_cel_par_loc = renum->nbr_cel_par_loc[ind_dom] ;
           ind_cel_par_loc < nbr_cel_par_loc_avec_voiset ;
           ind_cel_par_loc++)
        cpt_cel_ext_dom[num_dom_cel_ext[ind_dom][ind_cel_par_loc] - 1] += 1 ;

      renum->nbr_dom_cel_ext_voiset[ind_dom] = 0 ;

      for (ind_loc = 0 ; ind_loc < renum->nbr_dom ; ind_loc++)
        if (cpt_cel_ext_dom[ind_loc] > 0)
          renum->nbr_dom_cel_ext_voiset[ind_dom] += 1 ;

      BFT_MALLOC(renum->num_dom_cel_ext_voiset[ind_dom],
                 renum->nbr_dom_cel_ext_voiset[ind_dom], ecs_int_t) ;
      BFT_MALLOC(renum->pos_dom_cel_ext_voiset[ind_dom],
                 renum->nbr_dom_cel_ext_voiset[ind_dom] + 1, ecs_int_t) ;

      renum->pos_dom_cel_ext_voiset[ind_dom][0] = 1 ;

      cpt_dom = 0 ;

      for (ind_loc = 0 ; ind_loc < renum->nbr_dom ; ind_loc++) {
        if (cpt_cel_ext_dom[ind_loc] > 0) {
          renum->num_dom_cel_ext_voiset[ind_dom][cpt_dom] = ind_loc + 1 ;
          renum->pos_dom_cel_ext_voiset[ind_dom][cpt_dom + 1]
            =   renum->pos_dom_cel_ext_voiset[ind_dom][cpt_dom]
            + cpt_cel_ext_dom[ind_loc] ;
          cpt_dom += 1 ;
        }
      }

    }


    BFT_FREE(num_dom_cel_ext[ind_dom]) ;
    BFT_FREE(cpt_cel_ext_dom) ;


    /* Comptage du nombre de cellules periodiques dans chaque sous domaine    */
    /* Il ne faut pas oublier non plus la distinction entre perio norm/inv    */

    if (renum->nbr_per > 0) {

      BFT_MALLOC(cpt_cel_ext_per, 2*renum->nbr_per + 1, ecs_int_t) ;

      BFT_MALLOC(renum->pos_per_cel_ext[ind_dom],
                 (2*renum->nbr_per + 1)*renum->nbr_dom_cel_ext[ind_dom]
                 + 1,
                 ecs_int_t) ;

      renum->pos_per_cel_ext[ind_dom][0] = 1 ;

      for (ind_loc = 0 ;
           ind_loc < renum->nbr_dom_cel_ext[ind_dom] ;
           ind_loc++) {

        for (ind_per = 0 ; ind_per < 2*renum->nbr_per + 1 ; ind_per++)
          cpt_cel_ext_per[ind_per] = 0 ;

        for (ind_cel_par_loc = renum->pos_dom_cel_ext[ind_dom][ind_loc] - 1 ;
             ind_cel_par_loc < renum->pos_dom_cel_ext[ind_dom][ind_loc + 1] - 1;
             ind_cel_par_loc++)
          if (num_per_cel_ext[ind_dom][ind_cel_par_loc] > 0)
            cpt_cel_ext_per
              [2*( num_per_cel_ext[ind_dom][ind_cel_par_loc]) - 1] += 1 ;
          else
            cpt_cel_ext_per
              [2*(-num_per_cel_ext[ind_dom][ind_cel_par_loc])    ] += 1 ;


        for (ind_per = 0 ; ind_per < 2*renum->nbr_per + 1 ; ind_per++)
          renum->pos_per_cel_ext[ind_dom]
            [(2*renum->nbr_per + 1)*ind_loc + ind_per + 1] =
          renum->pos_per_cel_ext[ind_dom]
            [(2*renum->nbr_per + 1)*ind_loc + ind_per    ]
          + cpt_cel_ext_per[ind_per] ;

      }


      if (renum->bool_voiset_separe == ECS_TRUE) {

        BFT_MALLOC(renum->pos_per_cel_ext_voiset[ind_dom],
                   (2*renum->nbr_per + 1)*renum->nbr_dom_cel_ext_voiset[ind_dom]
                   + 1,
                   ecs_int_t) ;

        renum->pos_per_cel_ext_voiset[ind_dom][0] = 1 ;

        for (ind_loc = 0 ;
             ind_loc < renum->nbr_dom_cel_ext_voiset[ind_dom] ;
             ind_loc++) {

          for (ind_per = 0 ; ind_per < 2*renum->nbr_per + 1 ; ind_per++)
            cpt_cel_ext_per[ind_per] = 0 ;

          for (ind_cel_par_loc
                 = renum->pos_dom_cel_ext_voiset[ind_dom][ind_loc] - 1
                 + renum->nbr_cel_par_loc[ind_dom] ;
               ind_cel_par_loc
                 < renum->pos_dom_cel_ext_voiset[ind_dom][ind_loc + 1] - 1
                 + renum->nbr_cel_par_loc[ind_dom] ;
               ind_cel_par_loc++)
            if (num_per_cel_ext[ind_dom][ind_cel_par_loc] > 0)
              cpt_cel_ext_per
                [2*( num_per_cel_ext[ind_dom][ind_cel_par_loc]) - 1] += 1 ;
            else
              cpt_cel_ext_per
                [2*(-num_per_cel_ext[ind_dom][ind_cel_par_loc])    ] += 1 ;


          for (ind_per = 0 ; ind_per < 2*renum->nbr_per + 1 ; ind_per++)
            renum->pos_per_cel_ext_voiset[ind_dom]
              [(2*renum->nbr_per + 1)*ind_loc + ind_per + 1] =
              renum->pos_per_cel_ext_voiset[ind_dom]
              [(2*renum->nbr_per + 1)*ind_loc + ind_per    ]
              + cpt_cel_ext_per[ind_per] ;
        }
      }
      /* Fin du test sur voisinage tendu spar */

      BFT_FREE(cpt_cel_ext_per) ;

    }
    /* Fin du test sur s'il y a des priodicits */

    BFT_FREE(num_per_cel_ext[ind_dom]) ;

  }
  /* Fin du parcours des domaines */


  BFT_FREE(num_dom_cel_ext) ;
  BFT_FREE(num_per_cel_ext) ;


  /* Traitement des faces de bord
     (connectivit, nombre local et numro global
  */

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       nbr_fac_de_bord_loc[ind_dom++] = 0) ;


  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_de_bord ; ind_fac++) {

    ind_cel = renum->val_connect_fac_de_bord_cel[ind_fac] - 1 ;
    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;

    val_connect_fac_de_bord_cel_loc[ind_dom][nbr_fac_de_bord_loc[ind_dom]]
      = num_loc_cel[ind_cel] ;

    num_glob_fbr[ind_dom][nbr_fac_de_bord_loc[ind_dom]] = ind_fac + 1 ;
    nbr_fac_de_bord_loc[ind_dom] += 1 ;

  }

  BFT_FREE(num_loc_cel) ;


  /* Traitement de la connectivit faces <-> sommets */

  ecs_loc_renum__connect_som(renum) ;


}


/*----------------------------------------------------------------------------
 *    Fonction d'affichage des infos sur le domaine principal
 *----------------------------------------------------------------------------*/

void ecs_renum__print_info
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;

  bft_printf(_("\n\nMain mesh properties\n"
               "--------------------\n\n")) ;

  bft_printf(_("  Number of cells:                            %10d\n"
               "  Number of internal faces:                   %10d\n"),
             renum->nbr_cel,
             renum->nbr_fac_interne) ;


  if (renum->nbr_per > 0) {

    ecs_int_t  ind_per ;

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      bft_printf(_("    including periodicity %1d:                  %10d\n"),
                 ind_per + 1, renum->nbr_fac_perio[ind_per]) ;

  }


  bft_printf(_("  Number of boundary faces:                   %10d\n"),
             renum->nbr_fac_de_bord) ;


  bft_printf(_("  Number of vertices:                         %10d\n"),
             renum->nbr_som) ;


}


/*----------------------------------------------------------------------------
 *    Fonction d'affichage des infos sur les sous-domaines
 *----------------------------------------------------------------------------*/

void ecs_renum__print_info_loc
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_int_t       ind_dom ;
  ecs_int_t       ind_loc ;
  ecs_int_t       ind_per ;

  ecs_int_t       ipos ;

  ecs_int_t       nbr_fac_dir_crt ;
  ecs_int_t       nbr_fac_inv_crt ;
  ecs_int_t       nbr_fac_dir_par ;
  ecs_int_t       nbr_fac_inv_par ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;

  if (renum->nbr_dom == 1)

    return ;


  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {


    bft_printf(_("  Sub-domain %d\n\n"
                 "    Number of true cells:                     %10d\n"
                 "    Number of ghost cells:                    %10d\n"
                 "    Number of internal faces:                 %10d\n"
                 "    Number of boundary faces:                 %10d\n\n"),
               ind_dom + 1,
               renum->nbr_cel_loc[ind_dom],
               renum->nbr_cel_par_loc[ind_dom],
               renum->nbr_fac_interne_loc[ind_dom],
               renum->nbr_fac_de_bord_loc[ind_dom]) ;


    /* Impressions relatives a la priodicit */


    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {

      nbr_fac_dir_crt = 0 ;
      nbr_fac_inv_crt = 0 ;
      nbr_fac_dir_par = 0 ;
      nbr_fac_inv_par = 0 ;


      for (ind_loc = 0 ; ind_loc < renum->nbr_dom_cel_ext[ind_dom] ; ind_loc++)

        /* Domaines courant */

        if (renum->num_dom_cel_ext[ind_dom][ind_loc] == ind_dom + 1) {

          ipos = (2*renum->nbr_per + 1)*ind_loc + 2*ind_per ;

          nbr_fac_dir_crt =   renum->pos_per_cel_ext[ind_dom][ipos + 2]
                            - renum->pos_per_cel_ext[ind_dom][ipos + 1] ;
          nbr_fac_inv_crt =   renum->pos_per_cel_ext[ind_dom][ipos + 3]
                            - renum->pos_per_cel_ext[ind_dom][ipos + 2] ;

        }


        /* Faces priodiques 'partages' entre domaines */

        else {

          ipos = (2*renum->nbr_per + 1)*ind_loc + 2*ind_per ;

          nbr_fac_dir_par +=   renum->pos_per_cel_ext[ind_dom][ipos + 2]
                             - renum->pos_per_cel_ext[ind_dom][ipos + 1] ;
          nbr_fac_inv_par +=   renum->pos_per_cel_ext[ind_dom][ipos + 3]
                             - renum->pos_per_cel_ext[ind_dom][ipos + 2] ;

        }


      bft_printf(_("    Periodicity %1d :\n"), ind_per + 1) ;
      bft_printf(_("      Number of faces of sub-domain\n"
                   "        - direct:        %10d\n"
                   "        - inverse:       %10d\n"),
                 nbr_fac_dir_crt, nbr_fac_inv_crt) ;

      bft_printf(_("      Number of faces whose periodic correspondant\n"
                   "      belongs to another subdomain\n"
                   "        - direct:        %10d\n"
                   "        - inverse:       %10d\n\n"),
                 nbr_fac_dir_par, nbr_fac_inv_par) ;

    }

  }


}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des faces pour un seul domaine
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_fac_1
(
       ecs_renum_t       *       renum               , /* <-> Struct. de renum*/
       ecs_bool_t                bool_voiset_cel_cel , /* --> Voisinage tendu
                                                                   ou non     */
 const ecs_param_perio_t * const liste_param_perio   , /* --> Param priodicit*/
 const ecs_real_t        * const coord_som_maillage    /* --> Coord. sommets  */
)
{

  ecs_int_t       ind_per ;
  ecs_int_t       ind_bcl_cel ;
  ecs_int_t       ind_cel ;
  ecs_int_t       ind_cel_loc ;
  ecs_int_t       ind_cel_par_loc ;
  ecs_int_t       ind_cel_vis ;
  ecs_int_t       ind_cel_loc_voiset ;
  ecs_int_t       ind_cel_voiset ;
  ecs_int_t       ind_dom ;
  ecs_int_t       ind_fac ;
  /*  ecs_int_t       ind_iter ;*/
  ecs_int_t       ind_loc ;
  ecs_int_t       num_per_voiset ;

  ecs_int_t       val_connect ;

  ecs_int_t     * nbr_cel_loc ;
  ecs_int_t     * nbr_cel_par_loc ;
  ecs_int_t     * nbr_cel_par_loc_dim ;
  ecs_int_t     * nbr_cel_par_loc_sans_voiset ;
  ecs_int_t     * nbr_fac_interne_loc ;
  ecs_int_t     * nbr_fac_de_bord_loc ;

  ecs_int_t     * num_per_cel_ext ;
  ecs_int_t     * cpt_cel_ext_per ;

  ecs_int_t     * ind_per_dir_voiset_cel_cel ;
  ecs_int_t     * pos_connect_voiset_cel_cel ;
  ecs_int_t     * val_connect_voiset_cel_cel ;

  ecs_int_t   * * val_connect_fac_interne_cel_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_cel_loc ;
  ecs_int_t   * * pos_connect_voiset_cel_cel_loc ;
  ecs_int_t   * * val_connect_voiset_cel_cel_loc ;

  ecs_bool_t      bool_ind_cel_voiset_vu ;

  ecs_tab_int_t   tab_temp ;
  ecs_tab_int_t   tab_transf ;
  ecs_tab_int_t   tab_num_per ;



  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  BFT_MALLOC(nbr_cel_loc, 1, ecs_int_t) ;

  nbr_cel_loc[0]  = renum->nbr_cel ;

  renum->nbr_cel_loc     = nbr_cel_loc ;


  BFT_MALLOC(nbr_fac_interne_loc, 1, ecs_int_t) ;
  BFT_MALLOC(nbr_fac_de_bord_loc, 1, ecs_int_t) ;

  nbr_fac_interne_loc[0] = renum->nbr_fac_interne ;
  nbr_fac_de_bord_loc[0] = renum->nbr_fac_de_bord ;

  renum->nbr_fac_interne_loc = nbr_fac_interne_loc ;
  renum->nbr_fac_de_bord_loc = nbr_fac_de_bord_loc ;


  /* Tableaux */

  BFT_MALLOC(val_connect_fac_interne_cel_loc, 1, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_cel_loc, 1, ecs_int_t *) ;

  val_connect_fac_interne_cel_loc[0] = renum->val_connect_fac_interne_cel ;
  val_connect_fac_de_bord_cel_loc[0] = renum->val_connect_fac_de_bord_cel ;


  /* Allocations dans le cas priodique */

  if (renum->nbr_per > 0) {

    BFT_MALLOC(renum->nbr_dom_cel_ext, 1 ,ecs_int_t) ;
    renum->nbr_dom_cel_ext[0] = 1 ;

    BFT_MALLOC(nbr_cel_par_loc, 1, ecs_int_t) ;
    nbr_cel_par_loc[0] = 0 ;

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      nbr_cel_par_loc[0] += renum->nbr_fac_perio[ind_per] ;

    renum->nbr_cel_par_loc = nbr_cel_par_loc ;


    BFT_MALLOC(num_per_cel_ext, nbr_cel_par_loc[0], ecs_int_t) ;

    BFT_MALLOC(renum->num_loc_cel_ext, 1, ecs_int_t *) ;
    BFT_MALLOC(renum->pos_per_cel_ext, 1, ecs_int_t *) ;

    BFT_MALLOC(renum->num_loc_cel_ext[0], nbr_cel_par_loc[0], ecs_int_t) ;
    BFT_MALLOC(renum->pos_per_cel_ext[0],
               (2*renum->nbr_per + 1) + 1,
               ecs_int_t) ;

  }


  /* Traitement des faces internes priodiques */
  /*-------------------------------------------*/

  if (renum->nbr_per > 0 && renum->nbr_cel_par_loc[0] > 0) {   /* faces prio */

    /* On ajoute des cellules tendues ("cellules fantmes") */

    nbr_cel_par_loc[0] = 0 ;

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)

      for (ind_loc = 0 ; ind_loc < renum->nbr_fac_perio[ind_per] ; ind_loc++) {

        ind_fac = renum->liste_fac_perio[ind_per][ind_loc] ;

        ind_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2]     - 1 ;
        ind_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] - 1 ;


        if (renum->dir_per_fac_perio[ind_per][ind_loc] > 0)
          num_per_cel_ext[nbr_cel_par_loc[0]] =   ind_per + 1 ;
        else
          num_per_cel_ext[nbr_cel_par_loc[0]] = -(ind_per + 1) ;


        renum->num_loc_cel_ext[0][nbr_cel_par_loc[0]++] = ind_cel_vis + 1 ;

        val_connect_fac_interne_cel_loc[0][2*ind_fac    ] = ind_cel + 1 ;
        val_connect_fac_interne_cel_loc[0][2*ind_fac + 1] =
          nbr_cel_loc[0] + nbr_cel_par_loc[0] ;

      }
  }

  /*
     Voisinage tendu :


     On dtermine, pour chaque cellule, le voisinage tendu, i.e. l'ensemble
       des cellules qui partagent un noeud avec la cellule considre.
     Pour une cellule donne, les cellules voisines vues au travers des
       faces ne sont pas incluses dans le voisinage tendu.
     Il faut prendre en compte
          les cellules correspondantes en priodicit.

     Dans un premier temps et de toute facons pour la priodicit de rotation,
       on tend simplement la liste des cellules fantmes.
     Dans un second temps, sans priodicit de rotation,
       on pourra stocker sparement les cellules fantmes supplmentaires
       ncessaires au voisinage tendu.
       Ceci sera utile surtout pour minimiser les changes en parallle,
       mais peut tre fait galement par cohrence dans le cas d'un seul domaine

  */


  if (bool_voiset_cel_cel == ECS_TRUE) { /* Voisinage tendu */


    /* On construit la connectivit allge cellule -> cellules
       (i.e. voisinage tendu, sans les cellules vues au travers des faces) */

    ecs_loc_renum__voiset_cel_cel(renum,
                                  liste_param_perio,
                                  coord_som_maillage,
                                  &pos_connect_voiset_cel_cel,
                                  &val_connect_voiset_cel_cel,
                                  &ind_per_dir_voiset_cel_cel) ;


    /* La connectivit voisinage tendu cellule cellule est transmise
        la structure "locale" (la mme puisque l'on a un seul domaine). */

    /* Allocation des pointeurs de pointeur */
    BFT_MALLOC(val_connect_voiset_cel_cel_loc, 1, ecs_int_t *) ;
    BFT_MALLOC(pos_connect_voiset_cel_cel_loc, 1, ecs_int_t *) ;

    /* Reprage des adresses */
    val_connect_voiset_cel_cel_loc[0] = val_connect_voiset_cel_cel ;
    pos_connect_voiset_cel_cel_loc[0] = pos_connect_voiset_cel_cel ;


    /* On conserve le nombre de cellules fantmes sans voisinage tendu
       et on rserve un tableau pour allonger la taille des tableaux */

    /* En priodicit uniquement */
    if (renum->nbr_per > 0) {
      BFT_MALLOC(nbr_cel_par_loc_sans_voiset, 1, ecs_int_t) ;
      BFT_MALLOC(nbr_cel_par_loc_dim,         1, ecs_int_t) ;
      nbr_cel_par_loc_sans_voiset[0] = nbr_cel_par_loc[0] ;
      nbr_cel_par_loc_dim[0]         = nbr_cel_par_loc[0] ;
    }

    /* S'il y a de la priodicit,
       on ajoute les cellules fantmes et on adapte la connectivit
       cellule -> cellules en consquence. */

    if (renum->nbr_per > 0) { /* ve et prio */

    /* Numro de domaine = 0, pour la lisibilit ci-dessous */
      ind_dom = 0 ;

      /* On parcourt toutes les cellules globales */
      for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

        /* On parcourt les cellules du vois. tendu de la cellule courante */
        /* Il est possible de simplifier la boucle originale, issue du cas
           avec plusieurs domaines :
           for (ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel]   - 1,
                  ind_iter = 0 ;
                ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel+1] - 1 ;
                ind_bcl_cel++,
                   ind_iter++) {
        */
        for (ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel]   - 1 ;
             ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel+1] - 1 ;
             ind_bcl_cel++) {

          num_per_voiset = ind_per_dir_voiset_cel_cel[ind_bcl_cel] ;
          /*      si ncessaire :
                  ind_per_voiset = ECS_ABS(ind_per_dir)-1 ;
                  dir_per_voiset = (ind_per_dir > 0) ? 1 : -1 ; */


          /*
             Si la cellule et sa voisine tendue correspondent par priodicit
             il faut convertir la connectivit en connectivit locale
             et ajouter une cellule fantme tendue si ce n'est pas dj fait.
          */
          if (num_per_voiset != 0 ) {

            ind_cel_voiset     = val_connect_voiset_cel_cel[ind_bcl_cel] - 1 ;
            ind_cel_loc_voiset = ind_cel_voiset ;
            ind_cel_loc        = ind_cel ;

            /* On regarde si on a dj ajout la cellule fantme */
            /* On suppose que des priodicits de numro diffrent sont
               effectivement diffrentes. Ce n'est pas obligatoirement le cas */

            bool_ind_cel_voiset_vu = ECS_FALSE ;
            ind_cel_par_loc = 0 ;

            while (   bool_ind_cel_voiset_vu == ECS_FALSE
                   && ind_cel_par_loc < nbr_cel_par_loc[ind_dom]) {

              if (   (   renum->num_loc_cel_ext[ind_dom][ind_cel_par_loc]
                      == ind_cel_loc_voiset + 1)
                  && (   num_per_cel_ext[ind_cel_par_loc]
                      == num_per_voiset))
                bool_ind_cel_voiset_vu = ECS_TRUE ;
              else
                ind_cel_par_loc++;

            }

            /* Si on n'a pas encore ajout la cellule fantme :
               on le fait et on traduit la connectivit */

            if (bool_ind_cel_voiset_vu == ECS_FALSE) {

              if (nbr_cel_par_loc[ind_dom]+1 > nbr_cel_par_loc_dim[ind_dom]) {
                nbr_cel_par_loc_dim[ind_dom] = 2*nbr_cel_par_loc_dim[ind_dom] ;
                BFT_REALLOC(renum->num_loc_cel_ext[ind_dom],
                            nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
                BFT_REALLOC(num_per_cel_ext,
                            nbr_cel_par_loc_dim[ind_dom], ecs_int_t) ;
              }

              renum->num_loc_cel_ext[ind_dom][nbr_cel_par_loc[ind_dom]]
                = ind_cel_loc_voiset + 1 ;
              num_per_cel_ext[nbr_cel_par_loc[ind_dom]]
                = num_per_voiset ;

              nbr_cel_par_loc[ind_dom]++ ;

              /* Il est possible de simplifier la boucle originale,
                 issue du cas avec plusieurs domaines :
              val_connect_voiset_cel_cel_loc[ind_dom]
                [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc]
                +ind_iter-1]
                = nbr_cel_loc[ind_dom] + nbr_cel_par_loc[ind_dom] ;
              */
              val_connect_voiset_cel_cel_loc[ind_dom][ind_bcl_cel]
                = nbr_cel_loc[ind_dom] + nbr_cel_par_loc[ind_dom] ;

            }

            /* Si on a dj ajout la cellule fantme,
               on se contente de traduire la connectivit */

            else

              /* Il est possible de simplifier la boucle originale,
                 issue du cas avec plusieurs domaines :
              val_connect_voiset_cel_cel_loc[ind_dom]
                [pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc]
                +ind_iter-1]
                = nbr_cel_loc[ind_dom] + ind_cel_par_loc + 1 ;
              */
              val_connect_voiset_cel_cel_loc[ind_dom][ind_bcl_cel]
                = nbr_cel_loc[ind_dom] + ind_cel_par_loc + 1 ;

          }

          /* Si la cellule et sa voisine tendue ne correspondent pas par
             priodicit, il n'y a rien a faire */

        } /* Fin du parcours du voisinage tendu de la cellule courante*/

      } /* Fin de la boucle sur les cellules */

      /* On ralloue les tableaux relatifs aux fantmes  la taille exacte */
      BFT_REALLOC(renum->num_loc_cel_ext[ind_dom],
                  nbr_cel_par_loc[ind_dom], ecs_int_t) ;
      BFT_REALLOC(num_per_cel_ext,
                  nbr_cel_par_loc[ind_dom], ecs_int_t) ;

    } /* Fin du test sur priodicit et voisinage tendu */

    /* On libere l'indicateur de priodicit pour la connectivit
         voisinage tendu cellule -> cellules
       On conserve pos et val, car il sont galement la connectivit locale
         voisinage tendu cellule -> cellules */
    BFT_FREE(ind_per_dir_voiset_cel_cel) ;


    /* On libre les tableaux temporaires de nombre de cellules fantmes
       On devra peut tre conserver nbr_cel_par_loc_sans_voiset pour les
       distinguer */

    /* En priodicit uniquement */
    if (renum->nbr_per > 0) {
      BFT_FREE(nbr_cel_par_loc_sans_voiset) ;
      BFT_FREE(nbr_cel_par_loc_dim) ;
    }

  } /* Fin si voisinage tendu */


  /* Renumrotation des cellules fantmes, i.e. en priodicit uniquement */

  if (renum->nbr_per > 0 && renum->nbr_cel_par_loc[0] > 0) { /* Priodicit */

    /*
       Renumrotation des cellules fantmes :

       Elles sont triees en 2 groupes par periodicite :
       - sens normal, d'abord
       - sens inverse, ensuite
       Il n'y a pas de periodicite 'bizarre' ou une face se voit elle-meme,
       donc il y a autant de faces dans un groupe que dans l'autre.
    */

    tab_num_per.nbr = renum->nbr_cel_par_loc[0] ;
    BFT_MALLOC(tab_num_per.val, tab_num_per.nbr, ecs_int_t) ;

    /*
       Determination de l'ordre, on trie selon tab_num_dom.val,
       dans l'ordre croissant
    */
    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < renum->nbr_cel_par_loc[0] ; ind_cel_par_loc++)
      if (num_per_cel_ext[ind_cel_par_loc] > 0)
        tab_num_per.val[ind_cel_par_loc]
          = 2*( num_per_cel_ext[ind_cel_par_loc]) ;
      else
        tab_num_per.val[ind_cel_par_loc]
          = 2*(-num_per_cel_ext[ind_cel_par_loc]) + 1 ;

    tab_transf.nbr = renum->nbr_cel_par_loc[0] ;
    BFT_MALLOC(tab_transf.val, tab_transf.nbr, ecs_int_t) ;
    for (ind_cel = 0 ; ind_cel < tab_transf.nbr ; ind_cel++)
      tab_transf.val[ind_cel] = ind_cel ;

    ecs_tab_int__trie(tab_num_per, tab_transf) ;
    BFT_FREE(tab_num_per.val) ;
    ecs_tab_int__inverse(&tab_transf) ;

    /* On traite la connectivit face -> cellule */

    for (ind_fac = 0 ; ind_fac < nbr_fac_interne_loc[0] ; ind_fac++) {

      val_connect =   val_connect_fac_interne_cel_loc[0][ind_fac * 2]
                    - nbr_cel_loc[0] - 1 ;

      if (val_connect > -1)
        val_connect_fac_interne_cel_loc[0][ind_fac * 2]
          = tab_transf.val[val_connect] + nbr_cel_loc[0] + 1 ;

      val_connect =   val_connect_fac_interne_cel_loc[0][ind_fac * 2 + 1]
                    - nbr_cel_loc[0] - 1 ;

      if (val_connect > -1)
        val_connect_fac_interne_cel_loc[0][ind_fac * 2 + 1]
          = tab_transf.val[val_connect] + nbr_cel_loc[0] + 1 ;

    }

    /* On traite la connectivit voisinage tendu cellule -> cellule */

    if (bool_voiset_cel_cel == ECS_TRUE) {

      ind_dom = 0 ;

      for (ind_cel_loc = 0 ; ind_cel_loc < nbr_cel_loc[ind_dom] ;
           ind_cel_loc++) {

        for (ind_cel_loc_voiset
               = pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc  ]-1;
             ind_cel_loc_voiset
               < pos_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc+1]-1;
             ind_cel_loc_voiset++) {

          /* On soustrait nbr_cel_loc[ind_dom]+1 pour etre >= 0 si fantme
             on rtablit ensuite */

          val_connect
            = val_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc_voiset]
            - nbr_cel_loc[ind_dom] - 1 ;

          if (val_connect > -1)
            val_connect_voiset_cel_cel_loc[ind_dom][ind_cel_loc_voiset]
              = tab_transf.val[val_connect] + nbr_cel_loc[ind_dom] + 1 ;
        }
      }
    }

    /* On rordonne le numro local des cellules fantmes */

    ecs_tab_int__inverse(&tab_transf);

    tab_temp.nbr = renum->nbr_cel_par_loc[0] ;
    BFT_MALLOC(tab_temp.val, tab_temp.nbr, ecs_int_t) ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < renum->nbr_cel_par_loc[0] ; ind_cel_par_loc++)
      tab_temp.val[ind_cel_par_loc]
        = renum->num_loc_cel_ext[0][ind_cel_par_loc] ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < renum->nbr_cel_par_loc[0] ; ind_cel_par_loc++)
      renum->num_loc_cel_ext[0][ind_cel_par_loc]
        = tab_temp.val[tab_transf.val[ind_cel_par_loc]] ;

    /* Inutile de renumeroter num_per_cel_ext vu qu'il ne sert qu'a compter
       et qu'on a qu'un seul sous-domaine (pas de parallelisme) */

    BFT_FREE(tab_temp.val)    ;
    BFT_FREE(tab_transf.val)  ;


    /* Comptage du nombre de cellules periodiques (au plus 3 periodicites) */
    /* Il ne faut pas oublier non plus la distinction entre perio norm/inv */

    BFT_MALLOC(cpt_cel_ext_per, 2*renum->nbr_per + 1, ecs_int_t) ;

    for (ind_per = 0 ; ind_per < 2*renum->nbr_per + 1 ; ind_per++)
      cpt_cel_ext_per[ind_per] = 0 ;

    for (ind_cel_par_loc = 0 ;
         ind_cel_par_loc < renum->nbr_cel_par_loc[0] ; ind_cel_par_loc++)
      if (num_per_cel_ext[ind_cel_par_loc] > 0)
        cpt_cel_ext_per[2*( num_per_cel_ext[ind_cel_par_loc]) - 1] += 1 ;
      else
        cpt_cel_ext_per[2*(-num_per_cel_ext[ind_cel_par_loc])    ] += 1 ;

    BFT_FREE(num_per_cel_ext) ;


    renum->pos_per_cel_ext[0][0] = 1 ;

    for (ind_per = 0 ; ind_per < 2*renum->nbr_per + 1 ; ind_per++)
      renum->pos_per_cel_ext[0][ind_per + 1] =
        renum->pos_per_cel_ext[0][ind_per] + cpt_cel_ext_per[ind_per] ;

    BFT_FREE(cpt_cel_ext_per) ;

  }
  /* Fin s'il y a des faces priodiques */

  /*
    Les connectivits sont transmises  la structure de renumrotation.
  */

  /* face -> cellule : simple reprage de l'adresse */
  renum->val_connect_fac_interne_cel_loc = val_connect_fac_interne_cel_loc ;
  renum->val_connect_fac_de_bord_cel_loc = val_connect_fac_de_bord_cel_loc ;

  /* voisinage tendu cellule -> cellule : allocation de pointeurs
     et reprage de l'adresse */

  if (bool_voiset_cel_cel == ECS_TRUE) {

    /* Allocation des pointeurs de pointeur */
    BFT_MALLOC(renum->val_connect_voiset_cel_cel_loc, 1, ecs_int_t *) ;
    BFT_MALLOC(renum->pos_connect_voiset_cel_cel_loc, 1, ecs_int_t *) ;

    /* Reprage des adresses */
    renum->val_connect_voiset_cel_cel_loc[0]
      = val_connect_voiset_cel_cel_loc[0] ;
    renum->pos_connect_voiset_cel_cel_loc[0]
      = pos_connect_voiset_cel_cel_loc[0] ;

    BFT_FREE(val_connect_voiset_cel_cel_loc);
    BFT_FREE(pos_connect_voiset_cel_cel_loc);

  }


  /* Traitement de la connectivit faces <-> sommets */

  ecs_loc_renum__connect_som_1(renum) ;


}


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation locale des sommets.
 *    Les sommets sont utiliss pour des calculs gomtriques, mais ne
 *    portent pas de variables  ce jour. On ne cre donc pas de liste de
 *    correspondance entre sommets de domaines voisins, et on se contente
 *    de crer la connectivit locale  chaque sous-domaine.
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_som
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_int_t       ind_dom ;
  ecs_int_t       ind_som ;
  ecs_int_t       ind_fac ;
  ecs_int_t       ind_fac_loc ;

  ecs_int_t       iloc ;
  ecs_int_t       ipos ;
  ecs_int_t       ipos_loc ;

  ecs_int_t       nbr_som_fac ;
  ecs_int_t       nbr_val_connect_fac_interne_loc ;
  ecs_int_t       nbr_val_connect_fac_de_bord_loc ;

  ecs_int_t     * num_loc_som ;
  ecs_int_t     * cur_dom_som ;

  ecs_int_t     * nbr_som_loc ;

  ecs_int_t   * * num_glob_som ;

  ecs_int_t   * * pos_connect_fac_interne_som_loc ;
  ecs_int_t   * * pos_connect_fac_de_bord_som_loc ;
  ecs_int_t   * * val_connect_fac_interne_som_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_som_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;

  BFT_MALLOC(nbr_som_loc, renum->nbr_dom, ecs_int_t) ;

  renum->nbr_som_loc = nbr_som_loc ;

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; nbr_som_loc[ind_dom++] = 0) ;


  if (renum->nbr_som == 0)
    return ;


  /* Tableaux locaux */

  BFT_MALLOC(num_loc_som, renum->nbr_som, ecs_int_t) ;
  BFT_MALLOC(cur_dom_som, renum->nbr_som, ecs_int_t) ;

  for (ind_som = 0 ; ind_som < renum->nbr_som ; num_loc_som[ind_som++] = -1) ;
  for (ind_som = 0 ; ind_som < renum->nbr_som ; cur_dom_som[ind_som++] = -1) ;


  /* Tableaux pour la structure renum */

  BFT_MALLOC(pos_connect_fac_de_bord_som_loc, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_som_loc, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(pos_connect_fac_interne_som_loc, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_interne_som_loc, renum->nbr_dom, ecs_int_t *) ;


  renum->pos_connect_fac_de_bord_som_loc = pos_connect_fac_de_bord_som_loc ;
  renum->pos_connect_fac_interne_som_loc = pos_connect_fac_interne_som_loc ;
  renum->val_connect_fac_de_bord_som_loc = val_connect_fac_de_bord_som_loc ;
  renum->val_connect_fac_interne_som_loc = val_connect_fac_interne_som_loc ;


  BFT_MALLOC(num_glob_som, renum->nbr_dom, ecs_int_t *) ;

  renum->num_glob_som = num_glob_som ;


  /* Boucle sur les sous-domaines */

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {


    /* Prparation pour la numrotation locale des sommets */


    nbr_val_connect_fac_interne_loc = 0 ;
    nbr_val_connect_fac_de_bord_loc = 0 ;


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_interne_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fac[ind_dom][ind_fac_loc] - 1 ;

      for (ipos = renum->pos_connect_fac_interne_som[ind_fac    ] - 1 ;
           ipos < renum->pos_connect_fac_interne_som[ind_fac + 1] - 1 ;
           ipos++) {

        ind_som = renum->val_connect_fac_interne_som[ipos] - 1;

        nbr_val_connect_fac_interne_loc += 1 ;

        if (cur_dom_som[ind_som] < ind_dom) {

          nbr_som_loc[ind_dom] += 1 ;
          num_loc_som[ind_som] = nbr_som_loc[ind_dom] ;
          cur_dom_som[ind_som] = ind_dom ;

        }

      }

    }


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_de_bord_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fbr[ind_dom][ind_fac_loc] - 1 ;

      for (ipos = renum->pos_connect_fac_de_bord_som[ind_fac    ] - 1 ;
           ipos < renum->pos_connect_fac_de_bord_som[ind_fac + 1] - 1 ;
           ipos++) {

        ind_som = renum->val_connect_fac_de_bord_som[ipos] - 1 ;

        nbr_val_connect_fac_de_bord_loc += 1 ;

        if (cur_dom_som[ind_som] < ind_dom) {

          nbr_som_loc[ind_dom] += 1 ;
          num_loc_som[ind_som] = nbr_som_loc[ind_dom] ;
          cur_dom_som[ind_som] = ind_dom ;

        }

      }

    }


    BFT_MALLOC(num_glob_som[ind_dom], nbr_som_loc[ind_dom], ecs_int_t) ;


    /* Connectivit des faces internes locales avec les sommets locaux */


    BFT_MALLOC(pos_connect_fac_interne_som_loc[ind_dom],
               renum->nbr_fac_interne_loc[ind_dom] + 1,
               ecs_int_t) ;
    BFT_MALLOC(val_connect_fac_interne_som_loc[ind_dom],
               nbr_val_connect_fac_interne_loc,
               ecs_int_t) ;


    pos_connect_fac_interne_som_loc[ind_dom][0] = 1 ;


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_interne_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fac[ind_dom][ind_fac_loc] - 1 ;


      nbr_som_fac =   renum->pos_connect_fac_interne_som[ind_fac + 1]
                    - renum->pos_connect_fac_interne_som[ind_fac    ] ;

      pos_connect_fac_interne_som_loc[ind_dom][ind_fac_loc + 1]
        = pos_connect_fac_interne_som_loc[ind_dom][ind_fac_loc]
          + nbr_som_fac ;


      for (iloc = 0 ; iloc < nbr_som_fac ; iloc++) {

        ipos     =   renum->pos_connect_fac_interne_som[ind_fac] - 1 + iloc ;
        ipos_loc =   pos_connect_fac_interne_som_loc[ind_dom][ind_fac_loc]
                   - 1 + iloc ;

        ind_som = renum->val_connect_fac_interne_som[ipos] - 1 ;

        val_connect_fac_interne_som_loc[ind_dom][ipos_loc]
          = num_loc_som[ind_som] ;

        num_glob_som[ind_dom][num_loc_som[ind_som] - 1] = ind_som + 1 ;

      }

    }


    /* Connectivit des faces de bord locales avec les sommets locaux */


    BFT_MALLOC(pos_connect_fac_de_bord_som_loc[ind_dom],
               renum->nbr_fac_de_bord_loc[ind_dom] + 1,
               ecs_int_t) ;
    BFT_MALLOC(val_connect_fac_de_bord_som_loc[ind_dom],
               nbr_val_connect_fac_de_bord_loc,
               ecs_int_t) ;


    pos_connect_fac_de_bord_som_loc[ind_dom][0] = 1 ;


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_de_bord_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fbr[ind_dom][ind_fac_loc] - 1 ;


      nbr_som_fac =   renum->pos_connect_fac_de_bord_som[ind_fac + 1]
                    - renum->pos_connect_fac_de_bord_som[ind_fac    ] ;

      pos_connect_fac_de_bord_som_loc[ind_dom][ind_fac_loc + 1]
        = pos_connect_fac_de_bord_som_loc[ind_dom][ind_fac_loc]
          + nbr_som_fac ;


      for (iloc = 0 ; iloc < nbr_som_fac ; iloc++) {

        ipos     =   renum->pos_connect_fac_de_bord_som[ind_fac] - 1 + iloc ;
        ipos_loc =   pos_connect_fac_de_bord_som_loc[ind_dom][ind_fac_loc]
                   - 1 + iloc ;

        ind_som = renum->val_connect_fac_de_bord_som[ipos] - 1 ;

        val_connect_fac_de_bord_som_loc[ind_dom][ipos_loc]
          = num_loc_som[ind_som] ;

        num_glob_som[ind_dom][num_loc_som[ind_som] - 1] = ind_som + 1 ;

      }

    }

  }


  /* Libration des tableaux locaux */

  BFT_FREE(num_loc_som) ;
  BFT_FREE(cur_dom_som) ;


}


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des sommets pour un seul domaine
 *----------------------------------------------------------------------------*/

static void ecs_loc_renum__connect_som_1
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_int_t     * nbr_som_loc ;

  ecs_int_t   * * pos_connect_fac_interne_som_loc ;
  ecs_int_t   * * pos_connect_fac_de_bord_som_loc ;
  ecs_int_t   * * val_connect_fac_interne_som_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_som_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  BFT_MALLOC(nbr_som_loc, 1, ecs_int_t) ;

  nbr_som_loc[0] = renum->nbr_som ;

  renum->nbr_som_loc = nbr_som_loc ;


  if (renum->nbr_som == 0)
    return ;


  /* Tableaux ventuels */


  BFT_MALLOC(pos_connect_fac_interne_som_loc, 1, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_interne_som_loc, 1, ecs_int_t *) ;
  BFT_MALLOC(pos_connect_fac_de_bord_som_loc, 1, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_som_loc, 1, ecs_int_t *) ;

  pos_connect_fac_interne_som_loc[0] = renum->pos_connect_fac_interne_som ;
  val_connect_fac_interne_som_loc[0] = renum->val_connect_fac_interne_som ;
  pos_connect_fac_de_bord_som_loc[0] = renum->pos_connect_fac_de_bord_som ;
  val_connect_fac_de_bord_som_loc[0] = renum->val_connect_fac_de_bord_som ;

  renum->pos_connect_fac_interne_som_loc = pos_connect_fac_interne_som_loc ;
  renum->val_connect_fac_interne_som_loc = val_connect_fac_interne_som_loc ;
  renum->pos_connect_fac_de_bord_som_loc = pos_connect_fac_de_bord_som_loc ;
  renum->val_connect_fac_de_bord_som_loc = val_connect_fac_de_bord_som_loc ;


}


/*----------------------------------------------------------------------------
 *    Fonction qui retourne un voisinage tendu cellule -> cellules
 *
 *    On dtermine, pour chaque cellule, le voisinage tendu, i.e. l'ensemble
 *      des cellules qui partagent un noeud avec la cellule considre.
 *    Pour une cellule donne, les cellules voisines vues au travers des
 *      faces ne sont pas incluses dans le voisinage tendu.
 *----------------------------------------------------------------------------*/

static void ecs_loc_renum__voiset_cel_cel
(
       ecs_renum_t       *       renum               ,
                                    /* --> Structure de renumrotation        */
 const ecs_param_perio_t * const liste_param_perio   ,
                                    /* --> Paramtres de priodicit          */
 const ecs_real_t        * const coord_som_maillage  ,
                                    /* --> Coordonnes des sommets            */
                                    /* --> Dimension de l'espace              */
       ecs_int_t         *      *ppos_connect_voiset_cel_cel ,
                                    /* <-- Adresse du pointeur qui donne,
                                           pour chaque cellule, position dans
                                           val_connect_voiset_cel_cel de la
                                           1ere cellule du voisinage tendu
                                           pos_connect_voiset_cel_cel > 0 */
       ecs_int_t         *      *pval_connect_voiset_cel_cel ,
                                    /* <-- Adresse du pointeur qui donne,
                                           numro global des cellules du
                                           du voisinage tendu
                                           val_connect_voiset_cel_cel > 0     */
       ecs_int_t         *      *pind_per_dir_voiset_cel_cel
                                    /* <-- Adresse du pointeur qui donne,
                                           indique la priodicit ventuelle
                                           au travers de laquelle les cellules
                                           du voisinage tendu sont vues :
                                           = 0 : non priodique
                                           > 0 : n de priodicit, directe
                                           < 0 : n de priodicit, inverse   */
)
{

  ecs_int_t       dir_per_som ;
  ecs_int_t       icoo ;
  ecs_int_t       icoo_inf ;
  ecs_int_t       ind_bcl_cel ;
  ecs_int_t       ind_bcl_cel_1 ;
  ecs_int_t       ind_bcl_som ;
  ecs_int_t       ind_bcl_som_fac ;
  ecs_int_t       ind_bcl_som_inf ;
  ecs_int_t       ind_bcl_som_per ;
  ecs_int_t       ind_bcl_som_sup ;
  ecs_int_t       ind_cel ;
  ecs_int_t       ind_cel_1 ;
  ecs_int_t       ind_cel_2 ;
  ecs_int_t       ind_cel_per ;
  ecs_int_t       ind_cel_per_vis ;
  ecs_int_t       ind_cel_vis ;
  ecs_int_t       ind_fac ;
  ecs_int_t       ind_fac_per ;
  ecs_int_t       ind_fac_per_vis ;
  ecs_int_t       ind_fac_vis ;
  ecs_int_t       ind_per ;
  ecs_int_t       ind_per_dir_som ;
  ecs_int_t       ind_pos_last ;
  ecs_int_t       ind_som ;
  ecs_int_t       ind_som_vis ;
  ecs_int_t       ind_val_last ;
  ecs_int_t       isom ;
  ecs_int_t       isom_dep ;
  ecs_int_t       isom_min ;

  ecs_int_t       nbr_cel_elimine ;
  ecs_int_t       nbr_triplet_per ;
  ecs_int_t       nbr_triplet_per_max ;
  ecs_int_t       nbr_val_estime_connect_som_cel ;
  ecs_int_t       nbr_val_connect_som_cel ;
  ecs_int_t       nbr_val_estime_voiset_cel_cel ;
  ecs_int_t       nbr_val_connect_voiset_cel_cel ;

  ecs_int_t       nsmx ;
  ecs_int_t       nsom ;
  ecs_int_t       nsom_max ;
  ecs_int_t       nsom_max_old ;
  ecs_int_t       num_cel ;
  ecs_int_t       num_cel_vis ;

  ecs_int_t     * ind_dir_per_fac_perio_tmp ;

  ecs_int_t     * pos_connect_voiset_cel_cel ;
  ecs_int_t     * val_connect_voiset_cel_cel ;
  ecs_int_t     * ind_per_dir_voiset_cel_cel ;

  ecs_int_t     * ind_per_dir_som_per ;
  ecs_int_t     * ind_som_per ;
  ecs_int_t     * ind_som_vis_per ;

  ecs_int_t     * pos_connect_som_fac_interne ;
  ecs_int_t     * val_connect_som_fac_interne ;
  ecs_int_t     * pos_connect_som_cel ;
  ecs_int_t     * val_connect_som_cel ;
  ecs_int_t     * pos_connect_cel_som ;
  ecs_int_t     * val_connect_cel_som ;

  ecs_real_t      ecart2 ;
  ecs_real_t      ecart2_min ;

  ecs_bool_t      bool_ind_cel_1_vu ;
  ecs_bool_t      bool_ind_cel_vis_vu ;
  ecs_bool_t      bool_ind_cel_vu ;
  ecs_bool_t      bool_ind_fac_vu ;
  ecs_bool_t      bool_inv_perio ;
  ecs_bool_t      bool_num_cel_vis_vu ;
  ecs_bool_t      bool_num_cel_vu ;

  ecs_bool_t    * bool_som_vu ;
  ecs_bool_t    * bool_som_per ;

  ecs_vec_int_t * vec_int_connect_som_fac_interne ;
  ecs_vec_int_t * vec_int_connect_fac_interne_som ;
  ecs_vec_int_t * vec_int_connect_som_cel ;
  ecs_vec_int_t * vec_int_connect_cel_som ;

  ecs_point_t     coord_som ;

  ecs_point_t   * coord_som_per ;
  ecs_point_t   * coord_som_vis ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  /*
    On souhaite trouver les cellules partageant un noeud en parcourant
    les sommets. Il faut donc une connectivit sommets -> cellules.

    Remarque : la connectivit nodale cellules -> sommets n'est pas conserve
               aprs construction de la connectivit descendante
               cellule->faces->sommets (dans ecs_maillage__connect_descend).
               C'est dommage, il aurait suffit de l'inverser.

    On construit la connectivit sommets -> faces par inversion de
    faces -> sommets puis on construit sommets -> cellules.

    Il suffit de s'interesser aux faces internes car une cellule de
    voisinage tendu ne peut pas tre attache  un noeud qui n'est vu
    que par des faces de bord.
  */

  /* La prsence de sommets est indispensable */

  assert(renum->nbr_som != 0) ;


  /* Pour l'inversion de la connectivit fac_interne->sommets, on utilise
     une fonction d'inversion d'un vecteur index : ecs_vec_int__inverse
     Noter que *vec_int_connect_som_fac_interne->val_tab > 0 */

  /* On construit le vecteur index */
  assert(renum->pos_connect_fac_interne_som != NULL) ;
  vec_int_connect_fac_interne_som
    = ecs_vec_int__initialise(renum->nbr_fac_interne + 1,
                              renum->pos_connect_fac_interne_som,
                              renum->val_connect_fac_interne_som) ;
  /* On l'inverse */
  vec_int_connect_som_fac_interne =
    ecs_vec_int__inverse(vec_int_connect_fac_interne_som, renum->nbr_som) ;

  ecs_vec_int__libere(vec_int_connect_fac_interne_som);

  /* On cre des aliases */
  pos_connect_som_fac_interne
    = ecs_vec_int__ret_pos_tab(vec_int_connect_som_fac_interne) ;
  val_connect_som_fac_interne
    = ecs_vec_int__ret_val_tab(vec_int_connect_som_fac_interne) ;

  /*
     On construit la connectivit sommets cellules :
     pos_connect_som_cel et val_connect_som_cel
     ces deux tableaux sont calqus sur les membres d'un vecteur index
     et en particulier pos[] a nbr_som+1 lments et pos[0] = 1

     pour tous les sommets,
       on parcourt les fac_interne
         on prend les cellules voisines
           on les retient si on ne les a pas dj vues
  */


  /* Pour ne pas les inclure, on marque les cellules vues par priodicit
     On dmarquera aprs */

  /* Boucle sur les priodicits */

  for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {

    /* Parcours des faces priodiques */

    for (ind_fac_per = 0 ;
         ind_fac_per < renum->nbr_fac_perio[ind_per] ;
         ind_fac_per++) {

      ind_fac = renum->liste_fac_perio[ind_per][ind_fac_per] ;

      /* Marquage en ngatif */
      renum->val_connect_fac_interne_cel[ind_fac * 2 + 1]
        = -renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] ;

    }

  }


  /* On cherche les cellules s'appuyant sur un sommet */
  /*--------------------------------------------------*/

  /* Allocation estime */
  nbr_val_estime_connect_som_cel=8*renum->nbr_som + 1 ;
  BFT_MALLOC(val_connect_som_cel, nbr_val_estime_connect_som_cel, ecs_int_t) ;
  BFT_MALLOC(pos_connect_som_cel, renum->nbr_som + 1, ecs_int_t) ;

  /* Initialisation */
  pos_connect_som_cel[0] = 1 ;
  nbr_val_connect_som_cel = 0 ;


  for (ind_som = 0 ; ind_som < renum->nbr_som  ; ind_som++) {

    /* Reprage des fac_interne */

    for (ind_bcl_som_fac = pos_connect_som_fac_interne[ind_som]   - 1 ;
         ind_bcl_som_fac < pos_connect_som_fac_interne[ind_som+1] - 1 ;
         ind_bcl_som_fac++) {

      ind_fac = val_connect_som_fac_interne[ind_bcl_som_fac] - 1 ;

      /* Reprage des cellules voisines (num_cel != 0)*/
      num_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2]     ;
      num_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] ;

      /* A priori, elles n'ont pas encore t vues */
      bool_num_cel_vu     = ECS_FALSE ;
      bool_num_cel_vis_vu = ECS_FALSE ;

      /* Test si elles sont vues par priodicit,
         auquel cas on ne les traitera pas */
      if (num_cel     < 0) bool_num_cel_vu     = ECS_TRUE ;
      if (num_cel_vis < 0) bool_num_cel_vis_vu = ECS_TRUE ;

      /* Test si on les a dj rencontres pour le sommet courant
         auquel cas on ne les traitera pas */
      ind_bcl_som = pos_connect_som_cel[ind_som] - 1 ;

      while (  (    bool_num_cel_vu     == ECS_FALSE
                ||  bool_num_cel_vis_vu == ECS_FALSE)
             && ind_bcl_som < nbr_val_connect_som_cel) {

        if (   bool_num_cel_vu == ECS_FALSE
            && num_cel         == val_connect_som_cel[ind_bcl_som])
          bool_num_cel_vu     = ECS_TRUE ;

        if (   bool_num_cel_vis_vu == ECS_FALSE
            && num_cel_vis         == val_connect_som_cel[ind_bcl_som])
          bool_num_cel_vis_vu = ECS_TRUE ;

        ind_bcl_som++;

      }

      /* Si on ne les a pas dj vues, on les intgre  la connectivit */

      if (bool_num_cel_vu == ECS_FALSE) {

        if (nbr_val_connect_som_cel + 1 > nbr_val_estime_connect_som_cel) {
          nbr_val_estime_connect_som_cel *=2 ;
          BFT_REALLOC(val_connect_som_cel, nbr_val_estime_connect_som_cel,
                      ecs_int_t) ;
        }
        val_connect_som_cel[nbr_val_connect_som_cel] = num_cel     ;
        nbr_val_connect_som_cel++;

      }
      if (bool_num_cel_vis_vu == ECS_FALSE) {

        if (nbr_val_connect_som_cel + 1 > nbr_val_estime_connect_som_cel) {
          nbr_val_estime_connect_som_cel *=2 ;
          BFT_REALLOC(val_connect_som_cel, nbr_val_estime_connect_som_cel,
                      ecs_int_t) ;
        }
        val_connect_som_cel[nbr_val_connect_som_cel] = num_cel_vis ;
        nbr_val_connect_som_cel++;

      }

    }

    /* Reprage de la fin+1 de la liste des cellules pour le sommet courant */

    pos_connect_som_cel[ind_som+1] = nbr_val_connect_som_cel+1 ;

  }

  /* On libre le vecteur index de connectivit sommets fac_interne */

  ecs_vec_int__detruit(vec_int_connect_som_fac_interne) ;

  /* Rallocation avec la dimension exacte */

  BFT_REALLOC(val_connect_som_cel, nbr_val_connect_som_cel, ecs_int_t) ;

  /* On dmarque les cellules vues par priodicit */

  /* Boucle sur les priodicits */
  for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {

    /* Parcours des faces priodiques */
    for (ind_fac_per = 0 ;
         ind_fac_per < renum->nbr_fac_perio[ind_per] ;
         ind_fac_per++) {
      ind_fac = renum->liste_fac_perio[ind_per][ind_fac_per] ;
      renum->val_connect_fac_interne_cel[ind_fac * 2 + 1]
        = -renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] ;
    }
  }

  /* On dispose ici de la connectivit sommets cellules */
  /*----------------------------------------------------*/

#if 0 && defined(DEBUG) && !defined(NDEBUG)
  printf("Vrification de la connectivit sommets cellules\n");
  for (ind_som = 0 ; ind_som < renum->nbr_som ; ind_som++) {
    printf("Sommet %3d %14.5e %14.5e %14.5e :: Cellules :",ind_som+1,
           coord_som_maillage[3*ind_som +0],
           coord_som_maillage[3*ind_som +1],
           coord_som_maillage[3*ind_som +2]);
    for (ind_bcl_som = pos_connect_som_cel[ind_som  ]-1 ;
         ind_bcl_som < pos_connect_som_cel[ind_som+1]-1 ; ind_bcl_som++) {
      ind_cel = val_connect_som_cel[ind_bcl_som] - 1 ;
      printf(" %3d",ind_cel+1);
    }
    printf("\n");
  }
  printf("\n");
#endif

  /* On construit la connectivit cellules sommets
     en inversant la connectivit sommets cellules
     au moyen de ecs_vec_int__inverse, oprant sur vecteur index */

  /* On construit le vecteur index */
  vec_int_connect_som_cel
    = ecs_vec_int__initialise(renum->nbr_som + 1,
                              pos_connect_som_cel,
                              val_connect_som_cel) ;
  /* On inverse */
  vec_int_connect_cel_som
    = ecs_vec_int__inverse(vec_int_connect_som_cel, renum->nbr_cel) ;

  ecs_vec_int__libere(vec_int_connect_som_cel);

  /* On cre des aliases (on rappelle val != 0) */
  pos_connect_cel_som
    = ecs_vec_int__ret_pos_tab(vec_int_connect_cel_som) ;
  val_connect_cel_som
    = ecs_vec_int__ret_val_tab(vec_int_connect_cel_som) ;


  /* On dispose ici de la connectivit cellules sommets */
  /*----------------------------------------------------*/

#if 0 && defined(DEBUG) && !defined(NDEBUG)
  printf("Vrification de la connectivit cellules sommets\n");
  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {
    printf("Cellule %3d :: Sommets :",ind_cel+1);
    for (ind_bcl_cel = pos_connect_cel_som[ind_cel  ]-1 ;
         ind_bcl_cel < pos_connect_cel_som[ind_cel+1]-1 ; ind_bcl_cel++) {
      ind_som = val_connect_cel_som[ind_bcl_cel] - 1 ;
      printf(" %3d",ind_som+1);
    }
    printf("\n");
  }
  printf("\n");
#endif

  /*
     Pour intgrer les priodicits  la connectivit cellule -> cellule,
     on recherche d'abord les sommets priodiques, leurs correspondants et
     les priodicits associes.

     Plus tard, on pourra construire les priodicits composes
     supplmentaires, si on constate que cela peut servir a quelque chose.
     Dans l'immdiat c'est une usine a gaz et les applications sont rares
     (d'autant que les seuls cas de multipriodicit que je connaisse sont
     maills en briques, et que le voisinage tendu n'y sera donc pas utile.
  */

  /* S'il y a des priodicits */

  if (renum->nbr_per > 0) {

    /* Allocation des tableaux pour rprage des sommets priodiques */

    BFT_MALLOC(bool_som_vu        , renum->nbr_som     , ecs_bool_t) ;

    /* Supposons que le nombre de sommets d'une face soit < nsom_max */
    nsom_max = 20 ;
    BFT_MALLOC(coord_som_per, nsom_max, ecs_point_t) ;
    BFT_MALLOC(coord_som_vis, nsom_max, ecs_point_t) ;
    BFT_MALLOC(bool_som_per , nsom_max, ecs_bool_t ) ;

    /*
      Supposons que le nombre de triplets
      (sommet priodique, sommet priodique correspondant, priodicit)
      soit < nbr_triplet_per_max
      Par exemple, en supposant qu'il y a trois fois plus de sommets
      priodiques que de faces priodiques
    */

    nbr_triplet_per     = 0 ;
    nbr_triplet_per_max = 1 ;
    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      nbr_triplet_per_max += renum->nbr_fac_perio[ind_per] ;
    nbr_triplet_per_max *= 3 ;
    BFT_MALLOC(ind_som_per        , nbr_triplet_per_max, ecs_int_t) ;
    BFT_MALLOC(ind_som_vis_per    , nbr_triplet_per_max, ecs_int_t) ;
    BFT_MALLOC(ind_per_dir_som_per, nbr_triplet_per_max, ecs_int_t) ;


    /* Recherche des cellules priodiques  ajouter  sommet -> cellules */
    /*-------------------------------------------------------------------*/

    /* Boucle sur les priodicits */

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {

      /* Pour indiquer que l'on n'a pas trouv le correspondant du sommet */
      for (ind_som = 0 ; ind_som < renum->nbr_som ; ind_som++)
        bool_som_vu[ind_som] = ECS_FALSE ;

      /* Parcours des faces priodiques */
      for (ind_fac_per = 0 ; ind_fac_per < renum->nbr_fac_perio[ind_per] ;
           ind_fac_per++) {

        /* Numro */
        ind_fac = renum->liste_fac_perio[ind_per][ind_fac_per] ;

        /* Type de priodicit (attention bool_inv est l'inverse)*/
        dir_per_som = renum->dir_per_fac_perio[ind_per][ind_fac_per] ;
        bool_inv_perio  = (dir_per_som < 0) ? ECS_FALSE : ECS_TRUE ;

        /* Recherche de la face correspondante :
           obtenue en parcourant les faces periodiques  la recherche
           d'une face de mmes voisins, mais dans l'ordre inverse.
           Il y en a obligatoirement une. */

        ind_cel
          = renum->val_connect_fac_interne_cel[ind_fac * 2]     - 1 ;

        /* Attention, pour un seul sous-domaine,
           on a cras renum->val_connect_fac_interne_cel
           il contient le numro de cellule halo,
           pas le numro de cellule correspondante par priodicit */

        if (renum->nbr_dom > 1)
          ind_cel_vis
            = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] - 1 ;
        else
          ind_cel_vis =
            renum->num_loc_cel_ext[0]
            [renum->val_connect_fac_interne_cel[ind_fac * 2 + 1]
             - renum->nbr_cel - 1] - 1 ;

        ind_fac_per_vis = 0 ;
        bool_ind_fac_vu = ECS_FALSE ;

        while (bool_ind_fac_vu == ECS_FALSE &&
               ind_fac_per_vis < renum->nbr_fac_perio[ind_per]) {

          ind_fac_vis = renum->liste_fac_perio[ind_per][ind_fac_per_vis] ;
          ind_cel_per
            = renum->val_connect_fac_interne_cel[ind_fac_vis * 2] - 1 ;

          /* Attention, pour un seul sous-domaine,
             on a cras renum->val_connect_fac_interne_cel
             il contient le numro de cellule halo,
             pas le numro de cellule correspondante par priodicit */

          if (renum->nbr_dom > 1)
            ind_cel_per_vis
              = renum->val_connect_fac_interne_cel[ind_fac_vis * 2 + 1] - 1 ;
          else
            ind_cel_per_vis =
              renum->num_loc_cel_ext[0]
              [renum->val_connect_fac_interne_cel[ind_fac_vis * 2 + 1]
               - renum->nbr_cel - 1] - 1 ;

          if (   ind_cel == ind_cel_per_vis && ind_cel_vis == ind_cel_per
              && ind_fac_vis != ind_fac)
            bool_ind_fac_vu = ECS_TRUE ;
          else
            ind_fac_per_vis++;

        }

        /* Si bool_ind_fac_vu == ECS_TRUE
           on a trouv une face correspondante distincte de la
           face initiale et on a fini
           Sinon, la face recherche est peut tre confondue avec la
           face initiale, dans le cas d'une face se voyant elle-mme,
           et on se laisse donc une deuxime chance.
           Le fait de chercher d'abord des faces correspondantes
           distinctes permet de trouver la face correspondante (et non
           pas la face intiale) lorsqu'une cellule se voit elle mme 
           travers la face initiale (cas d'une priodicit de rotation
           pour un calcul 2D axi). */

        if (bool_ind_fac_vu == ECS_FALSE) {

          ind_fac_per_vis = 0 ;
          bool_ind_fac_vu = ECS_FALSE ;

          while (bool_ind_fac_vu == ECS_FALSE &&
                 ind_fac_per_vis < renum->nbr_fac_perio[ind_per]) {

            ind_fac_vis = renum->liste_fac_perio[ind_per][ind_fac_per_vis] ;
            ind_cel_per
              = renum->val_connect_fac_interne_cel[ind_fac_vis * 2]     - 1 ;

            /* Attention, pour un seul sous-domaine,
               on a cras renum->val_connect_fac_interne_cel
               il contient le numro de cellule halo,
               pas le numro de cellule correspondante par priodicit */

            if (renum->nbr_dom > 1)
              ind_cel_per_vis
                = renum->val_connect_fac_interne_cel[ind_fac_vis * 2 + 1] - 1 ;
            else
              ind_cel_per_vis =
                renum->num_loc_cel_ext[0]
                [renum->val_connect_fac_interne_cel[ind_fac_vis * 2 + 1]
                 - renum->nbr_cel - 1] - 1 ;

            if (ind_cel == ind_cel_per_vis && ind_cel_vis == ind_cel_per)
              bool_ind_fac_vu = ECS_TRUE ;
            else
              ind_fac_per_vis++;

          }

        }

        /* On a obligatoirement trouv une face priodique. */

        assert(bool_ind_fac_vu == ECS_TRUE) ;

        /*
           Parcours des sommets de la face priodique de dpart,
           rcupration des coordonnees des sommets qu'elle porte,
           calcul des coordonnes transformes dans le but de trouver
           les sommets priodiques correspondants
        */

        /*
           On vrifie que l'on pourra stocker les coordonnes transformes
           de tous les sommets
           (normalement, il y en a autant sur la face priodique et sur sa
           correspondante, mais on vrifie quand mme...)
        */

        nsom_max_old = nsom_max ;
        ind_bcl_som_inf
          = renum->pos_connect_fac_interne_som[ind_fac_vis  ] - 1 ;
        ind_bcl_som_sup
          = renum->pos_connect_fac_interne_som[ind_fac_vis+1] - 1 ;
        nsom = ind_bcl_som_sup - ind_bcl_som_inf ;
        nsmx = ECS_MAX(nsom, nsom_max) ;
        nsom_max = nsmx ;
        ind_bcl_som_inf
          = renum->pos_connect_fac_interne_som[ind_fac  ] - 1 ;
        ind_bcl_som_sup
          = renum->pos_connect_fac_interne_som[ind_fac+1] - 1 ;
        nsom = ind_bcl_som_sup - ind_bcl_som_inf ;
        nsmx = ECS_MAX(nsom, nsom_max) ;
        nsom_max = nsmx ;

        if (nsom_max > nsom_max_old) {
          BFT_REALLOC(coord_som_per, nsom_max, ecs_point_t) ;
          BFT_REALLOC(coord_som_vis, nsom_max, ecs_point_t) ;
          BFT_REALLOC(bool_som_per , nsom_max, ecs_bool_t ) ;
        }

        /* Calcul des coordonnes transformes des sommets  */

        ind_bcl_som_inf = renum->pos_connect_fac_interne_som[ind_fac  ] - 1 ;
        ind_bcl_som_sup = renum->pos_connect_fac_interne_som[ind_fac+1] - 1 ;

        for (ind_bcl_som = ind_bcl_som_inf, nsom = 0 ;
             ind_bcl_som < ind_bcl_som_sup ; ind_bcl_som++, nsom++) {

          /* Numro du sommet */
          ind_som = renum->val_connect_fac_interne_som[ind_bcl_som] - 1 ;

          /* Si on n'a pas encore trouv le correspondant du sommet */
          if (bool_som_vu[ind_som] == ECS_FALSE) {

            /* Coordonnes du sommet */
            icoo_inf = ECS_DIM_3 * ind_som ;
            for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
              coord_som[icoo] = coord_som_maillage[icoo_inf + icoo] ;

            /*
               Coordonnes transformes

               Attention, pour trouver les coordonnes du correspondant, il
               faut utiliser la priodicit inverse puisque la priodicit
               associe  une face est celle qu'il faut appliquer au
               correspondant priodique pour obtenir la face.
            */

            ecs_vec_grd__point_new_coor_perio
              (coord_som,
               coord_som_per[nsom],
               liste_param_perio[ind_per].translation,
               liste_param_perio[ind_per].point_inv,
               liste_param_perio[ind_per].matrice,
               bool_inv_perio) ;

          }

        }


        /* Identification des sommets transforms et des sommets
           de la face priodique correspondante */

        /* Coordonnes des sommets de la face priodique correspondante*/
        ind_bcl_som_inf
          = renum->pos_connect_fac_interne_som[ind_fac_vis  ] - 1 ;
        ind_bcl_som_sup
          = renum->pos_connect_fac_interne_som[ind_fac_vis+1] - 1 ;

        for (ind_bcl_som = ind_bcl_som_inf, nsom = 0 ;
             ind_bcl_som < ind_bcl_som_sup ;
             ind_bcl_som++, nsom++) {

          /* Numro du sommet */
          ind_som_vis = renum->val_connect_fac_interne_som[ind_bcl_som] - 1 ;

          /* Coordonnes du sommet */

          icoo_inf = ECS_DIM_3 * ind_som_vis ;

          for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
            coord_som_vis[nsom][icoo] = coord_som_maillage[icoo_inf + icoo] ;

        }

        /* nsom est ici
           le nombre de sommets de la face priodique correspondante */


        /* Les sommets priodiques correspondants coor_som_vis n'ont pas
           encore t apparis avec des sommets transforms */

        for (isom = 0 ; isom < nsom ; isom ++)
          bool_som_per[isom] = ECS_FALSE ;

        /* On boucle sur les sommets non vus de la face priodique de dpart*/

        ind_bcl_som_inf = renum->pos_connect_fac_interne_som[ind_fac  ] - 1 ;
        ind_bcl_som_sup = renum->pos_connect_fac_interne_som[ind_fac+1] - 1 ;

        for (ind_bcl_som = ind_bcl_som_inf, isom_dep = 0 ;
             ind_bcl_som < ind_bcl_som_sup ;
             ind_bcl_som++, isom_dep++) {

          ind_som = renum->val_connect_fac_interne_som[ind_bcl_som] - 1 ;

          /* Si on n'a pas encore trouv un correspondant au sommet */

          if (bool_som_vu[ind_som] == ECS_FALSE) {

            /* Ecart aux sommets priodiques correspondants qui restent */

            ecart2_min = HUGE_VAL ;
            isom_min   = nsom ;

            for (isom = 0 ; isom < nsom ; isom ++) {

              if (bool_som_per[isom] == ECS_FALSE) {
                ecart2
                  =   (  (coord_som_per[isom_dep][0]-coord_som_vis[isom][0])
                       * (coord_som_per[isom_dep][0]-coord_som_vis[isom][0]))
                    + (  (coord_som_per[isom_dep][1]-coord_som_vis[isom][1])
                       * (coord_som_per[isom_dep][1]-coord_som_vis[isom][1]))
                    + (  (coord_som_per[isom_dep][2]-coord_som_vis[isom][2])
                       * (coord_som_per[isom_dep][2]-coord_som_vis[isom][2])) ;
                if (ecart2 < ecart2_min) {
                  ecart2_min = ecart2 ;
                  isom_min = isom ;
                }
              }

            }

            assert(isom_min < nsom) ;

            /* On a appari deux sommets */
            bool_som_per[isom_min] = ECS_TRUE ;
            bool_som_vu[ind_som]   = ECS_TRUE ;
            ind_som_vis
              = renum->val_connect_fac_interne_som
              [renum->pos_connect_fac_interne_som[ind_fac_vis]+isom_min-1]-1 ;

            /* On ajoute l'info  la connectivit
               ind_som est le transform de ind_som_vis
               par la priodicit ind_per dans le sens dir_per_som */

            if (nbr_triplet_per+1 > nbr_triplet_per_max) {
              nbr_triplet_per_max = 2*nbr_triplet_per_max ;
              BFT_REALLOC(ind_som_per        ,
                          nbr_triplet_per_max, ecs_int_t) ;
              BFT_REALLOC(ind_som_vis_per    ,
                          nbr_triplet_per_max, ecs_int_t) ;
              BFT_REALLOC(ind_per_dir_som_per,
                          nbr_triplet_per_max, ecs_int_t) ;
            }

            ind_som_per        [nbr_triplet_per] = ind_som ;
            ind_som_vis_per    [nbr_triplet_per] = ind_som_vis ;
            ind_per_dir_som_per[nbr_triplet_per] = (ind_per+1)*dir_per_som ;
            nbr_triplet_per++;

            /* Si ind_som == ind_som_vis (axe de rotation par exemple)
               on ajoute un lment "symtrique" */

            if (ind_som == ind_som_vis) {

              if (nbr_triplet_per+1 > nbr_triplet_per_max) {
                nbr_triplet_per_max = 2*nbr_triplet_per_max ;
                BFT_REALLOC(ind_som_per        ,
                            nbr_triplet_per_max, ecs_int_t) ;
                BFT_REALLOC(ind_som_vis_per    ,
                            nbr_triplet_per_max, ecs_int_t) ;
                BFT_REALLOC(ind_per_dir_som_per,
                            nbr_triplet_per_max, ecs_int_t) ;
              }

              ind_som_per        [nbr_triplet_per] = ind_som_vis ;
              ind_som_vis_per    [nbr_triplet_per] = ind_som     ;
              ind_per_dir_som_per[nbr_triplet_per] = -(ind_per+1)*dir_per_som;
              nbr_triplet_per++;

            }

          } /* Fin si on n'a pas encore trouv un correspondant au sommet */

        } /* Fin de la boucle sur les sommets non vus de la face priodique
             de dpart ; on passe  la face priodique suivante */

      } /* Fin de la boucle sur les faces priodiques ;
           on passe  la priodicit suivante */

    } /* Fin de la boucle sur les priodicits */

    /* Libration */
    BFT_FREE(coord_som_per) ;
    BFT_FREE(coord_som_vis) ;
    BFT_FREE(bool_som_per ) ;
    BFT_FREE(bool_som_vu);

  } /* Fin du traitement des priodicits */

  /* On dispose ici de l'identification des sommets priodiques */
  /*------------------------------------------------------------*/

#if 0 && defined(DEBUG) && !defined(NDEBUG)
  printf("Vrification de la correspondance des sommets priodiques\n");
  if (renum->nbr_per > 0)
    for (ind_bcl_som_per = 0 ; ind_bcl_som_per < nbr_triplet_per ;
         ind_bcl_som_per++)
      printf("Sommet %3d   Correspondant %3d  Priodicit %3d\n",
             ind_som_per[ind_bcl_som_per]+1,
             ind_som_vis_per[ind_bcl_som_per]+1,
             ind_per_dir_som_per[ind_bcl_som_per]) ;
#endif

  /*
     On construit la connectivit voisinage tendu cellules cellules
     en utilisant les connectivits cellules sommets et sommets cellules
     pos_connect_voiset_cel_cel et val_connect_voiset_cel_cel
     ces deux tableaux sont calqus sur les membres d'un vecteur index
     et en particulier pos[] a nbr_cel+1 lments et pos[0] = 1

       pour toutes les cellules,
         on parcourt les sommets,
           on parcourt les cellules attaches,
             on les retient si on ne les a pas dj vues
  */

  /* Allocation type hexa potentiellement sous-dimensionne */

  nbr_val_estime_voiset_cel_cel = 26*renum->nbr_cel + 1 ;

  /* Connectivit locale voisinage tendu */

  BFT_MALLOC(pos_connect_voiset_cel_cel, renum->nbr_cel + 1, ecs_int_t) ;
  BFT_MALLOC(val_connect_voiset_cel_cel,
             nbr_val_estime_voiset_cel_cel, ecs_int_t) ;

  /* Indicateur de priodicit numro et signe : (ind_per+1)*dir_per
      appliquer  une cellule pour obtenir la cellule fantme dans
     laquelle decharger ses donnes (0 : non priodique) */

  BFT_MALLOC(ind_per_dir_voiset_cel_cel,
             nbr_val_estime_voiset_cel_cel, ecs_int_t) ;

  /* Initialisation */

  nbr_val_connect_voiset_cel_cel = 0 ;
  pos_connect_voiset_cel_cel[0]  = 1 ;

  /* Boucle sur les cellules */
  /*-------------------------*/

  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

    /* Reprage des sommets */
    /*----------------------*/

    for (ind_bcl_cel = pos_connect_cel_som[ind_cel]   - 1 ;
         ind_bcl_cel < pos_connect_cel_som[ind_cel+1] - 1 ;
         ind_bcl_cel++) {

      ind_som = val_connect_cel_som[ind_bcl_cel] - 1 ;

      /* Pour un sommet priodique,
         ajout des cellules associees au sommet correspondant  */

      /* Recherche pour dterminer si le sommet est priodique */

      if (renum->nbr_per > 0) {

        for (ind_bcl_som_per = 0 ;
             ind_bcl_som_per < nbr_triplet_per ;
             ind_bcl_som_per++) {

          /* Si le sommet est priodique on repre ses caractristiques */

          if (ind_som_per[ind_bcl_som_per] == ind_som ) {

            /* Priodicit (numro et signe) */
            ind_per_dir_som = ind_per_dir_som_per[ind_bcl_som_per] ;

            /* Reprage correspondants priodiques (antcdent de ind_som) */
            ind_som_vis = ind_som_vis_per[ind_bcl_som_per] ;

            /* Reprage des cellules attaches au correspondant ind_som_vis */

            for (ind_bcl_som = pos_connect_som_cel[ind_som_vis]   - 1 ;
                 ind_bcl_som < pos_connect_som_cel[ind_som_vis+1] - 1 ;
                 ind_bcl_som++) {

              ind_cel_1 = val_connect_som_cel[ind_bcl_som] - 1 ;

              /* Test si la cellule a dj t incluse dans le voisinage
                 tendu de la cellule courante ind_cel (par un autre sommet)*/

              bool_ind_cel_1_vu = ECS_FALSE ;
              ind_bcl_cel_1 = pos_connect_voiset_cel_cel[ind_cel] - 1 ;

              while (   bool_ind_cel_1_vu == ECS_FALSE
                     && ind_bcl_cel_1 < nbr_val_connect_voiset_cel_cel) {
                ind_cel_2 = val_connect_voiset_cel_cel[ind_bcl_cel_1] - 1 ;
                if (   (ind_cel_1 == ind_cel_2)
                    && (   ind_per_dir_som
                        == ind_per_dir_voiset_cel_cel[ind_bcl_cel_1]) )
                  bool_ind_cel_1_vu = ECS_TRUE ;
                ind_bcl_cel_1++;
              }

              /* On retient la cellule si on ne l'a pas dj vue */

              if (bool_ind_cel_1_vu == ECS_FALSE) {

                if (  nbr_val_connect_voiset_cel_cel+1
                    > nbr_val_estime_voiset_cel_cel) {
                  nbr_val_estime_voiset_cel_cel *= 2 ;
                  BFT_REALLOC(val_connect_voiset_cel_cel,
                              nbr_val_estime_voiset_cel_cel, ecs_int_t) ;
                  BFT_REALLOC(ind_per_dir_voiset_cel_cel,
                              nbr_val_estime_voiset_cel_cel, ecs_int_t) ;
                }
                val_connect_voiset_cel_cel
                  [nbr_val_connect_voiset_cel_cel] = ind_cel_1 + 1 ;
                ind_per_dir_voiset_cel_cel
                  [nbr_val_connect_voiset_cel_cel] = ind_per_dir_som ;
                nbr_val_connect_voiset_cel_cel++ ;

              }

            } /* Fin du parcours des cellules attaches au correspondant */

          } /* Fin si le sommet est priodique */

        } /* Fin du parcours de la liste des sommets priodiques */

      } /* Fin s'il y a des priodicits */

      /* Que le sommet soit priodique ou non,
         traitement des cellules directement rattaches
         (non vues  travers une priodicit) */

      /* Reprage des cellules attaches non priodiques */

      for (ind_bcl_som = pos_connect_som_cel[ind_som]   - 1 ;
           ind_bcl_som < pos_connect_som_cel[ind_som+1] - 1 ;
           ind_bcl_som++) {

        ind_cel_1 = val_connect_som_cel[ind_bcl_som] - 1 ;

        /* Test si la cellule n'est pas la cellule courante */

        if (ind_cel_1 != ind_cel) { /* Test si la cellule a dj t vue */

          bool_ind_cel_1_vu = ECS_FALSE ;
          ind_bcl_cel_1 = pos_connect_voiset_cel_cel[ind_cel] - 1 ;

          while (   bool_ind_cel_1_vu == ECS_FALSE
                 && ind_bcl_cel_1 < nbr_val_connect_voiset_cel_cel) {

            ind_cel_2 = val_connect_voiset_cel_cel[ind_bcl_cel_1] - 1 ;

            if (   (ind_cel_1 == ind_cel_2)
                && (ind_per_dir_voiset_cel_cel[ind_bcl_cel_1] == 0))
              bool_ind_cel_1_vu = ECS_TRUE ;

            ind_bcl_cel_1++;

          }

          /* On retient la cellule si on ne l'a pas dj vue
             (il faut completer ind_per_dir_voiset_cel_cel,
             car on ne l'a pas initialis, tant donn qu'il peut tre
             ncessaire de l'allonger par realloc) */

          if (bool_ind_cel_1_vu == ECS_FALSE) {

            if (  nbr_val_connect_voiset_cel_cel+1
                > nbr_val_estime_voiset_cel_cel) {
              nbr_val_estime_voiset_cel_cel *= 2 ;
              BFT_REALLOC(val_connect_voiset_cel_cel,
                          nbr_val_estime_voiset_cel_cel, ecs_int_t) ;
              BFT_REALLOC(ind_per_dir_voiset_cel_cel,
                          nbr_val_estime_voiset_cel_cel, ecs_int_t) ;
            }

            val_connect_voiset_cel_cel
              [nbr_val_connect_voiset_cel_cel] = ind_cel_1 + 1 ;
            ind_per_dir_voiset_cel_cel
              [nbr_val_connect_voiset_cel_cel] = 0 ;
            nbr_val_connect_voiset_cel_cel++ ;

          }

        } /* Fin si la cellule n'est pas la cellule courante */

      } /* Fin du parcours des cellules attaches non priodiques */

    }

    /* Fin+1 de la liste des cellules associes  la cellule courante  */
    pos_connect_voiset_cel_cel[ind_cel+1] = nbr_val_connect_voiset_cel_cel+1 ;

  } /* Fin de la boucle sur les cellules */


  /* On libre la  connectivit sommets cellules */
  BFT_FREE(val_connect_som_cel) ;
  BFT_FREE(pos_connect_som_cel) ;

  /* On libre le vecteur index de connectivit cellule sommet */
  ecs_vec_int__detruit(vec_int_connect_cel_som) ;

  /* On libre les vecteurs temporaires priodiques */
  if (renum->nbr_per > 0) {
    BFT_FREE(ind_som_per        ) ;
    BFT_FREE(ind_som_vis_per    ) ;
    BFT_FREE(ind_per_dir_som_per) ;
  }

  /* Rallocation avec la dimension exacte */
  BFT_REALLOC(val_connect_voiset_cel_cel,
              nbr_val_connect_voiset_cel_cel, ecs_int_t) ;
  BFT_REALLOC(ind_per_dir_voiset_cel_cel,
              nbr_val_connect_voiset_cel_cel, ecs_int_t) ;

#if 0 && defined(DEBUG) && !defined(NDEBUG)
  printf("Vrification de la connectivit cellules cellule non allge\n");
  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {
    printf("Cellule %3d :: Cellules :",ind_cel+1);
    for (ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel  ]-1 ;
         ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel+1]-1 ;
         ind_bcl_cel++) {
      ind_cel_1 = val_connect_voiset_cel_cel[ind_bcl_cel] - 1 ;
      printf(" %3d",ind_cel_1+1);
    }
    printf("\n");
  }
  printf("\n");
#endif

  /*
     On allge la connectivit voisinage tendu cellules cellules
     en liminant, pour chaque cellule, les cellules vues au travers
     des faces.

     Pour le calcul des gradients tendus, ce choix est satisfaisant
     si l'on adopte (comme c'est le cas pour le moment) une formulation
     qui fait toujours intervenir les voisins vu au travers des faces
     (dans ce cas, le calcul du gradient sur support tendu se traduit
     par l'ajout d'une boucle  l'algorithme sur support non tendu
     pour traiter les cellules du voisinage tendu qui ne sont pas vues
     au travers des faces : on passe donc aisment d'un algo  l'autre).

     En outre, cela permet d'allger les boucles suivantes ainsi que
     les messages envoys au noyau.

     Par ailleurs, si on souhaite disposer de la connectivit voisinage
     tendu complte, rajouter les cellules vues au travers des faces
     est relativement ais (boucle sur les faces, on ajoute les voisins
      la connectivit de chacun d'eux, avec un test au cas o deux faces
     distinctes auraient les mmes voisins (pour des faces redcoupes
     par exemple ?)).

     Si l'on ne fait pas le choix d'allger la connectivit, il sera
     nanmoins ncessaire de marquer les cellules fantmes vues au
     travers des faces (en prvision du fait que, s'il n'y a pas de
     priodicit de rotation, seules ces cellules interviennent lors
     de la plupart des communications du noyau).
  */

  /* Marquage en ngatif des cellules voisines vues au travers de faces */

  /* Attention,
     une cellule n peut tre vue par une cellule p  la fois de deux faons
     diffrentes (au moins) : directement et par priodicit. Son numro
     apparat alors plusieurs fois dans le voisinage tendu associ  p, mais
     dsigne des reprsentations distinctes de la mme cellule, associes  des
     proprits de priodicit diffrentes.
     Ainsi, si p voit n  travers une face dont les proprits de priodicit
     sont donnes (ind_dir_per_fac_perio_tmp), on liminera dans le
     voisinage tendu de p les instances de la cellule n qui y sont reprsentes
     avec les mmes proprits de priodicit (et uniquement celles-l).  */

  /* Reprage */
  BFT_MALLOC(ind_dir_per_fac_perio_tmp, renum->nbr_fac_interne, ecs_int_t) ;
  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++)
    ind_dir_per_fac_perio_tmp[ind_fac] = 0 ;
  for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
    for (ind_fac_per = 0 ; ind_fac_per < renum->nbr_fac_perio[ind_per] ;
         ind_fac_per++) {
      ind_fac = renum->liste_fac_perio[ind_per][ind_fac_per] ;
      ind_dir_per_fac_perio_tmp[ind_fac]
        = (ind_per+1)*renum->dir_per_fac_perio[ind_per][ind_fac_per] ;
    }

  /* Marquage des cellules */

  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

    ind_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2]     - 1 ;
    ind_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] - 1 ;

    /* Attention, pour un seul sous-domaine,
       on a cras renum->val_connect_fac_interne_cel
       il contient le numro de cellule halo,
       pas le numro de cellule correspondante par priodicit */

    if ((renum->nbr_dom == 1) && (ind_cel_vis + 1 > renum->nbr_cel))
      ind_cel_vis =
        renum->num_loc_cel_ext[0]
        [renum->val_connect_fac_interne_cel[ind_fac * 2 + 1]
         - renum->nbr_cel - 1] - 1 ;

    bool_ind_cel_vis_vu = ECS_FALSE ;
    ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel      ] - 1 ;

    while (   bool_ind_cel_vis_vu == ECS_FALSE
           && ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel    +1] - 1) {

      if (   (ind_cel_vis == val_connect_voiset_cel_cel[ind_bcl_cel] - 1)
          && (   ind_dir_per_fac_perio_tmp[ind_fac]
              == ind_per_dir_voiset_cel_cel[ind_bcl_cel]) ) {
        bool_ind_cel_vis_vu = ECS_TRUE ;
        val_connect_voiset_cel_cel[ind_bcl_cel]
          = - val_connect_voiset_cel_cel[ind_bcl_cel] ;
      }
      ind_bcl_cel++;

    }

    bool_ind_cel_vu = ECS_FALSE ;
    ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel_vis  ] - 1 ;

    while (   bool_ind_cel_vu == ECS_FALSE
           && ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel_vis+1] - 1) {

      if (   (ind_cel == val_connect_voiset_cel_cel[ind_bcl_cel] - 1)
          && (    ind_dir_per_fac_perio_tmp[ind_fac]
              == -ind_per_dir_voiset_cel_cel[ind_bcl_cel])) {
        bool_ind_cel_vu    = ECS_TRUE ;
        val_connect_voiset_cel_cel[ind_bcl_cel]
          = - val_connect_voiset_cel_cel[ind_bcl_cel] ;
      }

      ind_bcl_cel++ ;

    }

  }

  /* Libration du tableau de reprage */

  BFT_FREE(ind_dir_per_fac_perio_tmp) ;

  /* Elimination des cellules voisines vues au travers de faces
     du tableau de connectivit du voisinage tendu */

  nbr_cel_elimine = 0 ;
  ind_val_last = -1 ;
  ind_pos_last =  0 ;

  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

    for (ind_bcl_cel = ind_pos_last ;
         ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel+1] - 1 ;
         ind_bcl_cel++) {

      if (val_connect_voiset_cel_cel[ind_bcl_cel] > 0) {
        ind_val_last++;
        val_connect_voiset_cel_cel[ind_val_last]
          = val_connect_voiset_cel_cel[ind_bcl_cel] ;
        ind_per_dir_voiset_cel_cel[ind_val_last]
          = ind_per_dir_voiset_cel_cel[ind_bcl_cel] ;
      }
      else
        nbr_cel_elimine++;
    }

    ind_pos_last = pos_connect_voiset_cel_cel[ind_cel+1] - 1 ;
    pos_connect_voiset_cel_cel[ind_cel+1]
      = pos_connect_voiset_cel_cel[ind_cel+1] - nbr_cel_elimine ;

  }

  /* Rallocation avec la dimension exacte */

  nbr_val_connect_voiset_cel_cel
    = nbr_val_connect_voiset_cel_cel - nbr_cel_elimine ;
  BFT_REALLOC(val_connect_voiset_cel_cel,
              nbr_val_connect_voiset_cel_cel, ecs_int_t) ;
  BFT_REALLOC(ind_per_dir_voiset_cel_cel,
              nbr_val_connect_voiset_cel_cel, ecs_int_t) ;


  /* On dispose de la connectivit allge cellule -> cellules ;
     on la renvoie */

  *ppos_connect_voiset_cel_cel = pos_connect_voiset_cel_cel ;
  *pval_connect_voiset_cel_cel = val_connect_voiset_cel_cel ;
  *pind_per_dir_voiset_cel_cel = ind_per_dir_voiset_cel_cel ;


#if 0 && defined(DEBUG) && !defined(NDEBUG)
  printf("Vrification de la connectivit cellules cellule allge\n");
  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {
    printf("Cellule %3d :: Cellules :",ind_cel+1);
    for (ind_bcl_cel = pos_connect_voiset_cel_cel[ind_cel  ]-1 ;
         ind_bcl_cel < pos_connect_voiset_cel_cel[ind_cel+1]-1 ;
         ind_bcl_cel++) {
      ind_cel_1 = val_connect_voiset_cel_cel[ind_bcl_cel] - 1 ;
      printf(" %3d",ind_cel_1+1);
    }
    printf("\n");
  }
  printf("\n");
#endif

}
