/*============================================================================
 *  Dfinitions des fonctions
 *   associes aux structures `ecs_vec_int_t' et `ecs_vec_real_t' decrivant
 *   les vecteurs indexes entier et reel
 *   et propres aux vecteurs indexs
 *   lis aux champs auxiliaires de type "grandeur"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <limits.h>
#include <math.h>

#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' systeme
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab.h"

#include "ecs_elt_typ_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int.h"
#include "ecs_vec_real.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_grd.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int_priv.h"
#include "ecs_vec_real_priv.h"



/*============================================================================
 *                       Macros globales au fichier
 *============================================================================*/

#if !defined(FLT_MAX)
#define FLT_MAX HUGE_VAL
#endif

enum {
  X ,
  Y ,
  Z
} ;


#define ECS_LOC_PRODUIT_VECTORIEL(prod_vect, vect1, vect2) ( \
prod_vect[X] = vect1[Y] * vect2[Z] - vect2[Y] * vect1[Z] ,   \
prod_vect[Y] = vect2[X] * vect1[Z] - vect1[X] * vect2[Z] ,   \
prod_vect[Z] = vect1[X] * vect2[Y] - vect2[X] * vect1[Y]   )


#define ECS_LOC_PRODUIT_SCALAIRE(vect1, vect2)                        ( \
  vect1[X] * vect2[X] + vect1[Y] * vect2[Y] + vect1[Z] * vect2[Z] )


#define ECS_LOC_MODULE(vect) \
     sqrt(vect[X] * vect[X] + vect[Y] * vect[Y] + vect[Z] * vect[Z])



/*============================================================================
 *                       Prototypes de fonctions privees
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
  *  Fonction qui calcule les coordonnes minimales et maximales
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__calc_coo_ext
(
 ecs_vec_real_t * vec_som
)
{

  ecs_int_t   icoo ;
  ecs_int_t   ipos ;
  ecs_int_t   isom ;

  ecs_real_t  coo_min[3] ;
  ecs_real_t  coo_max[3] ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (vec_som->pos_nbr < 2)
    return ;

  ipos = 0 ;

  for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++) {
    coo_min[icoo] = vec_som->val_tab[ipos + icoo] ;
    coo_max[icoo] = vec_som->val_tab[ipos + icoo] ;
  }

  for (isom = 1 ; isom < vec_som->pos_nbr - 1 ; isom++) {

    ipos = vec_som->pos_pas * isom ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++) {
      if (vec_som->val_tab[ipos + icoo] < coo_min[icoo])
        coo_min[icoo] = vec_som->val_tab[ipos + icoo] ;
      else if (vec_som->val_tab[ipos + icoo] > coo_max[icoo])
        coo_max[icoo] = vec_som->val_tab[ipos + icoo] ;
    }

  }

  bft_printf(_("\n  Domain coordinate extents:\n\n")) ;

  bft_printf("  [% 10.5e, % 10.5e, % 10.5e]\n",
             coo_min[0], coo_min[1], coo_min[2]) ;
  bft_printf("  [% 10.5e, % 10.5e, % 10.5e]\n",
             coo_max[0], coo_max[1], coo_max[2]) ;

}


/*----------------------------------------------------------------------------
 * Fonction qui calcule les normales aux faces
 *
 *----------------------------------------------------------------------------
 *
 *                          Pi+1
 *              *---------*                   B  : barycentre du polygone P
 *             / .       . \
 *            /   .     .   \                 Pi : sommet     du polygone P
 *           /     .   .     \
 *          /       . .  Ti   \               Ti : triangle
 *         *.........B.........* Pi
 *     Pn-1 \       . .       /
 *           \     .   .     /
 *            \   .     .   /
 *             \ .   T0  . /
 *              *---------*
 *            P0
 *
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__calc_normales
(
       ecs_vec_real_t *const vec_fac_normale ,
 const ecs_vec_int_t  *const vec_fac_def     ,
 const ecs_vec_int_t  *const vec_are_def     ,
 const ecs_vec_real_t *const vec_som_def
)
{

  ecs_int_t  icoo     ;
  ecs_int_t  isom     ;
  ecs_int_t  iare     ;
  ecs_int_t  ifac     ;
  ecs_int_t  itri     ;

  ecs_int_t  icoo_inf ;
  ecs_int_t  iare_inf ;
  ecs_int_t  iare_sup ;

  ecs_int_t  nbr_max_som_fac ;
  ecs_int_t  nbr_som_fac ;
  ecs_int_t  nbr_fac  ;
  ecs_int_t  ind_som  ;
  ecs_int_t  ind_are  ;

  ecs_point_t  vect1  ;
  ecs_point_t  vect2  ;

  ecs_point_t *coo_som_fac    ;
  ecs_point_t  barycentre_fac ;
  ecs_point_t  normale_fac    ;
  ecs_point_t  normale_tri    ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(vec_som_def->pos_pas == ECS_DIM_3) ;

  nbr_fac = vec_fac_def->pos_nbr - 1 ;

  vec_fac_normale->pos_pas = ECS_DIM_3 ;


  nbr_max_som_fac = 0 ;

  for (ifac = 0 ; ifac < nbr_fac ; ifac++) {

    nbr_som_fac = vec_fac_def->pos_tab[ifac + 1] - vec_fac_def->pos_tab[ifac] ;

    if (nbr_max_som_fac <= nbr_som_fac) nbr_max_som_fac = nbr_som_fac ;

  }

  BFT_MALLOC(coo_som_fac, nbr_max_som_fac + 1, ecs_point_t) ;


  /*=========================================================================*/
  /* Boucle sur les faces                                                    */
  /*=========================================================================*/


  for (ifac = 0 ; ifac < nbr_fac ; ifac++) {


    /*------------------------------------------------------------------------*/
    /*                                                                        */
    /* Definition du polygone P en fonction des sommets Pi de la face         */
    /*                                                                        */
    /*------------------------------------------------------------------------*/


    iare_inf = vec_fac_def->pos_tab[ifac]     - 1 ;
    iare_sup = vec_fac_def->pos_tab[ifac + 1] - 1 ;

    nbr_som_fac = 0 ;


    for (iare = iare_inf ; iare < iare_sup ; iare++) {

      ind_are  = vec_fac_def->val_tab[iare] ;

      if (ind_are > 0)
        isom = vec_are_def->pos_tab[ind_are          - 1]  - 1 ;
      else
        isom = vec_are_def->pos_tab[ECS_ABS(ind_are) - 1]      ;

      ind_som  = vec_are_def->val_tab[isom] - 1 ;

      icoo_inf = vec_som_def->pos_pas * ind_som ;


      for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {

        coo_som_fac[nbr_som_fac][icoo] = vec_som_def->val_tab[icoo_inf + icoo] ;


      } /* Fin : boucle sur les coordonnees du sommet */


      nbr_som_fac++ ;



    } /* Fin : boucle sur les aretes de la face */


    for (icoo = X ; icoo < ECS_DIM_3 ; icoo++)
      coo_som_fac[nbr_som_fac][icoo] = coo_som_fac[0][icoo] ;



    /*------------------------------------------------------------------------*/
    /*                                                                        */
    /* Calcul des coordonnees du barycentre B du polygone P                   */
    /*                                                                        */
    /*                                                                        */
    /*  -->    1   n-1  -->                                                   */
    /*  OB  =  -  Somme OPi                                                   */
    /*         n   i=0                                                        */
    /*                                                                        */
    /*                                                                        */
    /*------------------------------------------------------------------------*/


    for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {

      barycentre_fac[icoo] = 0. ;

      for (isom = 0 ; isom < nbr_som_fac ; isom++)
        barycentre_fac[icoo] += coo_som_fac[isom][icoo] ;

      barycentre_fac[icoo] /= nbr_som_fac ;

    }


    for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {
      normale_fac[icoo] = 0. ;
    }


    /* Premiere boucle sur les triangles de la face (calcul des normales)     */
    /*========================================================================*/


    for (itri = 0 ; itri < nbr_som_fac ; itri++) {


      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /* Calcul de la normale au triangle Ti :                                */
      /*                                                                      */
      /*  ->            -->   -->                                             */
      /*  N(Ti) = 1/2 ( BPi X BPi+1 )                                         */
      /*                                                                      */
      /*----------------------------------------------------------------------*/


      for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {

        vect1[icoo] = coo_som_fac[itri    ][icoo] - barycentre_fac[icoo] ;
        vect2[icoo] = coo_som_fac[itri + 1][icoo] - barycentre_fac[icoo] ;

      }

      ECS_LOC_PRODUIT_VECTORIEL(normale_tri, vect1, vect2) ;

      for (icoo = X ; icoo < ECS_DIM_3 ; icoo++)
        normale_tri[icoo] /= 2. ;


      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /* Calcul de la normale au polygone                                     */
      /*  comme somme vectorielle des normales aux triangles :                */
      /*                                                                      */
      /*  ->      n-1   ->                                                    */
      /*  N(P) = Somme( N(Ti) )                                               */
      /*          i=0                                                         */
      /*                                                                      */
      /*----------------------------------------------------------------------*/


      for (icoo = X ; icoo < ECS_DIM_3 ; icoo++)
        normale_fac[icoo] += normale_tri[icoo] ;


    } /* Fin de la premiere boucle sur les triangles de la face */


    /* Stockage du resultat dans la structure appropriee */
    /*---------------------------------------------------*/

    for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {

      vec_fac_normale->val_tab[ifac * ECS_DIM_3 + icoo]
        = normale_fac[icoo] ;

    }


  } /* Fin : boucle sur les faces */


  BFT_FREE(coo_som_fac) ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les distorsions de faces que l'on peut filtrer
 *  via une liste. On renvoie galement une liste des faces dont la
 *  distorsion est suprieure  un angle 'max_warp_fac' pour un ventuel
 *  affichage et/ou redcoupage de ces faces en triangles.
 *
 *  On calcule l'angle entre chacune des artes d'une face et la normale
 *  de la face (calcule prcdemment). L'angle maximal obtenu sera
 *  pris comme tant la distorsion de la face.
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__fac_distorsion
(
 const ecs_vec_int_t   *const vec_fac_def     ,
 const ecs_vec_int_t   *const vec_are_def     ,
 const ecs_vec_real_t  *const vec_som_def     ,
 const ecs_vec_real_t  *const vec_fac_normale ,
 const ecs_tab_int_t   *const liste_filtre    ,
 const ecs_real_t             max_warp_fac    ,
       ecs_tab_int_t   *const liste_fac_warp  ,
       ecs_tab_real_t  *const tab_val_warp
)
{

#define ECS_LOC_FCT_COSINUS(vect1, vect2)                     ( \
   ECS_LOC_PRODUIT_SCALAIRE(vect1, vect2)                       \
 / (ECS_LOC_MODULE(vect1) * ECS_LOC_MODULE(vect2)) )

  ecs_int_t  ipos     ;
  ecs_int_t  isom_0   ;
  ecs_int_t  isom_1   ;
  ecs_int_t  iare     ;
  ecs_int_t  ifac     ;
  ecs_int_t  icoo     ;

  ecs_int_t  iare_inf ;
  ecs_int_t  iare_sup ;
  ecs_int_t  ind_fac  ;
  ecs_int_t  ind_are  ;

  ecs_int_t  nbr_fac  ;

  ecs_real_t  cos_warp_are  ;
  ecs_real_t  cos_warp_fac  ;
  ecs_real_t  warp_fac      ;
  ecs_real_t  rad_vers_deg  ;

  ecs_point_t  dir_are  ;
  ecs_point_t  normale_fac  ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  assert(vec_fac_def != NULL) ;
  assert(vec_are_def != NULL) ;
  assert(vec_som_def != NULL) ;

  if (liste_filtre == NULL)
    nbr_fac = vec_fac_def->pos_nbr - 1 ;
  else
    nbr_fac = liste_filtre->nbr ;

  assert(vec_som_def->pos_pas == ECS_DIM_3) ;

  rad_vers_deg = 180. / acos(-1.) ;


  /* Boucle gnrale sur les faces de la liste */
  /*===========================================*/


  liste_fac_warp->nbr = 0 ;

  for (ind_fac = 0 ; ind_fac < nbr_fac ; ind_fac++) {

    if (liste_filtre == NULL)
      ifac = ind_fac ;
    else
      ifac = liste_filtre->val[ind_fac] ;

    /* Normale de la face */

    for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {
      normale_fac[icoo] = vec_fac_normale->val_tab[ifac*ECS_DIM_3 + icoo] ;
    }

    /* Calcul des angles des artes avec la normale de la face */
    /*---------------------------------------------------------*/

    iare_inf = vec_fac_def->pos_tab[ifac]     - 1 ;
    iare_sup = vec_fac_def->pos_tab[ifac + 1] - 1 ;

    cos_warp_fac = 1. ;


    for (iare = iare_inf ; iare < iare_sup ; iare++) {

      ind_are  = ECS_ABS(vec_fac_def->val_tab[iare]) - 1 ;

      isom_0 = vec_are_def->val_tab[ind_are*2    ]  - 1 ;
      isom_1 = vec_are_def->val_tab[ind_are*2 + 1]  - 1 ;

      for (icoo = X ; icoo < ECS_DIM_3 ; icoo++) {
        dir_are[icoo] =   vec_som_def->val_tab[isom_1*ECS_DIM_3 + icoo]
                        - vec_som_def->val_tab[isom_0*ECS_DIM_3 + icoo] ;
      }


      /* Calcul de la distorsion locale de l'arte */

      cos_warp_are = ECS_LOC_FCT_COSINUS(normale_fac, dir_are) ;
      cos_warp_are = ECS_ABS(cos_warp_are) ;

      assert(cos_warp_are <= 1.001) ;


      /* Calcul de la distorsion globale de la face courante */

      cos_warp_fac = ECS_MIN(cos_warp_are, cos_warp_fac) ;


    } /* Fin : boucle sur les artes de la face */


    /* stockage en degrs des distorsions et des faces  trater */
    /*========================================================== */

    warp_fac = 90.0 - (acos(cos_warp_fac) * rad_vers_deg) ;

    tab_val_warp->val[ind_fac] = warp_fac ;

    if (warp_fac >= max_warp_fac) liste_fac_warp->nbr += 1 ;


  } /* Fin : boucle sur les faces */


  /* On renvoie la liste des faces ayant une */
  /* distorsion suprieure  max_warp_fac    */
  /* ======================================= */

  if (liste_fac_warp != NULL && liste_fac_warp->nbr >= 1) {

    BFT_MALLOC(liste_fac_warp->val, liste_fac_warp->nbr, ecs_int_t) ;

    ipos = 0 ;

    for (ind_fac = 0 ; ind_fac < nbr_fac ; ind_fac++) {

      if (liste_filtre == NULL)
        ifac = ind_fac ;
      else
        ifac = liste_filtre->val[ind_fac] ;

      if (tab_val_warp->val[ind_fac] >= max_warp_fac)
        liste_fac_warp->val[ipos++] = ifac ;

    }

  }


}


/*----------------------------------------------------------------------------
 *  Fonction qui calcule le maximum associ  une cellule d'un tableau de
 *   valeurs associes aux faces internes.
 *----------------------------------------------------------------------------*/

ecs_tab_real_t ecs_vec_grd__max_fac_int
(
 const ecs_int_t                nbr_cel           ,
 const ecs_vec_int_t     *const vec_def_fac_cel   ,
 const ecs_tab_int_t     *const liste_fac_interne ,
 const ecs_tab_real_t    *const val_fac_interne   ,
 const ecs_real_t               val_min
)
{

  ecs_int_t  icel     ;
  ecs_int_t  ifac     ;
  ecs_int_t  ifac_int ;
  ecs_int_t  icel_inf ;
  ecs_int_t  icel_sup ;

  ecs_int_t  ind_cel_inf     ;
  ecs_int_t  ind_cel_sup     ;

  ecs_tab_real_t tab_val_cel ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  assert(vec_def_fac_cel   != NULL) ;
  assert(liste_fac_interne != NULL) ;
  assert(val_fac_interne   != NULL) ;


  tab_val_cel.nbr = nbr_cel ;
  BFT_MALLOC(tab_val_cel.val, tab_val_cel.nbr, ecs_real_t) ;

  for (icel = 0 ; icel < tab_val_cel.nbr ; icel++)
    tab_val_cel.val[icel] = val_min ;


  /* Boucle gnrale sur les faces */
  /*===============================*/


  for (ifac_int = 0 ; ifac_int < liste_fac_interne->nbr  ; ifac_int++) {

    ifac = liste_fac_interne->val[ifac_int] ;

    assert((  vec_def_fac_cel->pos_tab[ifac + 1]
            - vec_def_fac_cel->pos_tab[ifac    ]) == 2) ;


    /* Numros des cellules qui encadrent la face interne */
    /*----------------------------------------------------*/

    icel_inf = vec_def_fac_cel->pos_tab[ifac] - 1 ;
    icel_sup = vec_def_fac_cel->pos_tab[ifac] ;

    ind_cel_inf = ECS_ABS(vec_def_fac_cel->val_tab[icel_inf]) - 1 ;
    ind_cel_sup = ECS_ABS(vec_def_fac_cel->val_tab[icel_sup]) - 1 ;

    /* Si on est en priodicit, on a cr des cellules "fantomes"
       supplmentaires pour pouvoir les considrer en tant que faces
       internes. On fait donc un test sur le numro de cellule.      */

    if (ind_cel_inf < nbr_cel &&
        tab_val_cel.val[ind_cel_inf] < val_fac_interne->val[ifac_int])
      tab_val_cel.val[ind_cel_inf] = val_fac_interne->val[ifac_int] ;

    if (ind_cel_sup < nbr_cel &&
        tab_val_cel.val[ind_cel_sup] < val_fac_interne->val[ifac_int])
      tab_val_cel.val[ind_cel_sup] = val_fac_interne->val[ifac_int] ;

  } /* Fin : boucle sur les faces */


  return tab_val_cel ;


}


/*----------------------------------------------------------------------------*
 *  Calcul de la matrice d'une rotation dfinie par un angle theta (en
 *   radians) et un vecteur unitaire direction.
 *   La matrice est calcule via la formule suivante :
 *
 *  R = (1-cos(theta))direction.transp(direction) + cos(theta)I + sin(theta)V
 *
 *           [ 0            -direction(3)  direction(2)]
 *  avec V = [ direction(3)       0       -direction(1)]
 *           [-direction(2)  direction(1)       0      ]
 *
 *  Remarque : le calcul de la matrice necessite 10 additions et 24 produits.
 *             on pourrait rduire le nombre de multiplications (tout en
 *             augmentant le nombre d'additions) en passant par les
 *             quaternions. On pourrait aussi rduire le nombre de produits
 *              18 en stockant 6 produits dans des variables intermdiares.
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__mat_rot_3d
(
 const ecs_real_t  angle,        /* --> Angle de rotation en degres           */
       ecs_real_t  direction[3], /* --> Vecteur directeur (unitaire) de l'axe */
       ecs_real_t  matrice[3][3] /* <-- Matrice definissant la rotation       */
)
{

  /* initialisations */

  ecs_int_t        icoo ;
  ecs_real_t       norme_dir ;

  const ecs_real_t pi = 4 * atan(1) ;
  const ecs_real_t theta = pi * angle/180 ;
  const ecs_real_t cost = cos(theta) ;
  const ecs_real_t sint = sin(theta) ;
  const ecs_real_t unmcost = (1.0 - cost) ;


  if (ECS_ABS(angle) < 1.e-12)
    return ;


  norme_dir = ECS_LOC_MODULE(direction) ;
  assert(norme_dir > 1.e-12) ;

  for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
    direction[icoo] /= norme_dir ;


  /* premire ligne de la matrice de rotation */

  matrice[0][0] = (unmcost) * direction[0]*direction[0] + cost                ;
  matrice[0][1] = (unmcost) * direction[0]*direction[1] - sint * direction[2] ;
  matrice[0][2] = (unmcost) * direction[0]*direction[2] + sint * direction[1] ;

  /* deuxime ligne de la matrice de rotation */

  matrice[1][0] = (unmcost) * direction[1]*direction[0] + sint * direction[2] ;
  matrice[1][1] = (unmcost) * direction[1]*direction[1] + cost                ;
  matrice[1][2] = (unmcost) * direction[1]*direction[2] - sint * direction[0] ;

  /* troisime ligne de la matrice de rotation */

  matrice[2][0] = (unmcost) * direction[2]*direction[0] - sint * direction[1] ;
  matrice[2][1] = (unmcost) * direction[2]*direction[1] + sint * direction[0] ;
  matrice[2][2] = (unmcost) * direction[2]*direction[2] + cost                ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les nouvelles coordonnees d'un vecteur ou d'un point
 *  dans le cas d'une periodicite.
 *----------------------------------------------------------------------------*/

ecs_vec_real_t * ecs_vec_grd__vec_new_coor_perio
(
 const ecs_vec_real_t    * vec_def             ,
 const ecs_real_t          translation[3]      ,
 const ecs_real_t          point_inv[3]        ,
 const ecs_real_t          matrice[3][3]       ,
 const ecs_bool_t          periodicite_inverse
)
{

  ecs_int_t         ielt ;
  ecs_int_t         icoo ;
  ecs_int_t         irot ;
  ecs_int_t         ipos ;

  ecs_real_t        vect[3] ;

  ecs_vec_real_t  * vec_def_new ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(vec_def->pos_nbr != 0) ;

  vec_def_new = ecs_vec_real__alloue(vec_def->pos_nbr,
                                     (vec_def->pos_nbr - 1) * ECS_DIM_3) ;


  if (periodicite_inverse == ECS_FALSE)


    for (ielt = 0 ; ielt < vec_def->pos_nbr - 1 ; ielt++) {

      ipos = vec_def->pos_pas * ielt ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vect[icoo] = 0. ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        for (irot = 0 ; irot < ECS_DIM_3 ; irot++)
          vect[icoo] = vect[icoo] + matrice[icoo][irot] *
            (vec_def->val_tab[ipos+irot]-point_inv[irot] + translation[irot]) ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vec_def_new->val_tab[ipos+icoo] =  vect[icoo] + point_inv[icoo] ;

    }


  else


    for (ielt = 0 ; ielt < vec_def->pos_nbr - 1 ; ielt++) {

      ipos = vec_def->pos_pas * ielt ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vect[icoo] = 0. ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        for (irot = 0 ; irot < ECS_DIM_3 ; irot++)
          vect[icoo] = vect[icoo] + matrice[irot][icoo] *
            (vec_def->val_tab[ipos+irot]-point_inv[irot]) ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vec_def_new->val_tab[ipos+icoo] =  vect[icoo] + point_inv[icoo]
                                           - translation[icoo] ;

    }



  return vec_def_new ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les nouvelles coordonnees d'un vecteur ou d'un point
 *  dans le cas d'une periodicite.
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__point_new_coor_perio
(
 const ecs_point_t         coord_point_old     ,
       ecs_point_t         coord_point_new     ,
 const ecs_real_t          translation[3]      ,
 const ecs_real_t          point_inv[3]        ,
 const ecs_real_t          matrice[3][3]       ,
 const ecs_bool_t          periodicite_inverse
)
{

  ecs_int_t     icoo ;
  ecs_int_t     irot ;

  ecs_real_t    vect[3] ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (periodicite_inverse == ECS_FALSE) {


    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      vect[icoo] = 0. ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      for (irot = 0 ; irot < ECS_DIM_3 ; irot++)
        vect[icoo] = vect[icoo] + matrice[icoo][irot] *
          (coord_point_old[irot] - point_inv[irot] + translation[irot]) ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      coord_point_new[icoo] = vect[icoo] + point_inv[icoo] ;


  }
  else {


    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      vect[icoo] = 0. ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      for (irot = 0 ; irot < ECS_DIM_3 ; irot++)
        vect[icoo] = vect[icoo] + matrice[irot][icoo] *
          (coord_point_old[irot] - point_inv[irot]) ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      coord_point_new[icoo] = vect[icoo] + point_inv[icoo] - translation[icoo] ;


  }


}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/

