/*============================================================================
 *  Dfinitions des fonctions
 *   associes  la structure `ecs_champ_t' dcrivant un champ
 *   et ralisant les sorties pour post-traitement EnSight
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h>

#include <bft_mem.h>
#include <bft_file.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_chaine_glob.h"
#include "ecs_elt_typ_liste.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Post-Traitement"
 *----------------------------------------------------------------------------*/

#include "ecs_post.h"
#include "ecs_post_ens.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_famille_chaine.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associ au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_post_ens.h"
#include "ecs_champ_post_ens.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' privs   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_post_ens_priv.h"
#include "ecs_champ_priv.h"


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui crit les connectivits nodales des lments
 *   selon leur type gometrique
 *
 *  Les lements doivent avoir t tris suivant leur type gometrique
 *----------------------------------------------------------------------------*/

void ecs_loc_champ_post_ens__ecr_part
(
 const char            *const nom_maillage    ,
 const char            *const nom_part        ,
       ecs_champ_t     *const champ_som       ,
       ecs_champ_t     *const champ_def       ,
 const ecs_tab_int_t   *const liste_filtre    ,
 const ecs_tab_int_t   *const tab_elt_typ_geo ,
       ecs_bool_t             ecrire_parent   ,
       ecs_post_ens_t  *const cas_ens
) ;


/*----------------------------------------------------------------------------
 *  Fonction d'extraction des numros de familles
 *---------------------------------------------------------------------------*/

static ecs_tab_int_t ecs_loc_champ_post_ens__famille
(
 const ecs_champ_t  *const champ_def     ,
       ecs_champ_t  *const champ_famille
) ;


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui crit les connectivits nodales des lments
 *   selon leur type gometrique
 *
 *  Les lements doivent avoir t tris suivant leur type gometrique
 *----------------------------------------------------------------------------*/

void ecs_champ_post_ens__ecr_part
(
 const char            *const nom_maillage    ,
       ecs_champ_t     *const champ_som       ,
       ecs_champ_t     *const champ_def       ,
       ecs_champ_t     *const champ_famille   ,
 const ecs_famille_t   *const famille_tete    ,
 const ecs_tab_int_t   *const tab_elt_typ_geo ,
       ecs_post_ens_t  *const cas_ens
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_som != NULL) ;
  assert(champ_def != NULL) ;

  assert(cas_ens != NULL) ;


  /* Ecriture en sous-parts */
  /*------------------------*/

  if (   cas_ens->single == ECS_FALSE
      && champ_famille != NULL
      && famille_tete != NULL) {

    ecs_int_t      idescr ;
    ecs_tab_int_t  tab_famille ;

    ecs_tab_char_t    tab_nom_descr ;
    ecs_tab_int_t   * tab_lst_descr ;  /* Tableau de pointeurs */

    /* Rcupration et tri des numros de familles */

    tab_famille = ecs_loc_champ_post_ens__famille(champ_def,
                                                  champ_famille) ;

    ecs_famille_chaine__att_fam_elt(famille_tete,
                                    tab_famille,
                                    _("Color "),
                                    "",
                                    ECS_TRUE,
                                    &tab_nom_descr,
                                    &tab_lst_descr) ;

    BFT_FREE(tab_famille.val) ;

    for (idescr = 0 ; idescr < tab_nom_descr.nbr ; idescr++) {

      if (tab_lst_descr[idescr].nbr >0) {

        ecs_bool_t  ecrire_parent  = ECS_FALSE ;
        const char prefixe_vol[]   = N_("vol") ;
        const char prefixe_bord[]  = N_("bdr") ;
        const char prefixe_int[]   = N_("int") ;
        const char nom_descr_nul[] = N_("no attribute") ;
        const char separ[]  = N_(": ") ;

        const char *prefixe_nom = nom_maillage ;
        const char *nom_descr = NULL ;
        char *nom_part = NULL ;

        if (!strcmp(nom_maillage, _(ECS_MAILLAGE_NOM_PRINCIPAL))) {
            prefixe_nom = prefixe_vol ;
            ecrire_parent = ECS_TRUE ;
        }
        else if (!strcmp(nom_maillage, _(ECS_MAILLAGE_NOM_BORD)))
            prefixe_nom = prefixe_bord ;
        else if (!strcmp(nom_maillage, _(ECS_NOM_FACES_SELECT)))
            prefixe_nom = prefixe_int ;

        if (tab_nom_descr.val[idescr] != NULL)
          nom_descr = tab_nom_descr.val[idescr] ;
        else
          nom_descr = nom_descr_nul ;

        BFT_MALLOC(nom_part,
                   ( strlen(_(prefixe_nom)) + strlen(_(separ))
                     + strlen(_(nom_descr)) + 1),
                   char) ;
        sprintf(nom_part, "%s%s%s",
                _(prefixe_nom), _(separ), _(nom_descr)) ;

        ecs_loc_champ_post_ens__ecr_part(nom_maillage,
                                         nom_part,
                                         champ_som,
                                         champ_def,
                                         tab_lst_descr + idescr,
                                         tab_elt_typ_geo,
                                         ecrire_parent,
                                         cas_ens) ;

        BFT_FREE(nom_part) ;

      }
    }

    /* Libration mmoire */

    for (idescr = 0 ; idescr < tab_nom_descr.nbr ; idescr++) {
      BFT_FREE(tab_nom_descr.val[idescr]) ;
      BFT_FREE(tab_lst_descr[idescr].val) ;
    }
    BFT_FREE(tab_nom_descr.val) ;
    BFT_FREE(tab_lst_descr) ;

  }

  /* Ecriture en une seule part */
  /*----------------------------*/

  else {

    ecs_loc_champ_post_ens__ecr_part(nom_maillage,
                                     nom_maillage,
                                     champ_som,
                                     champ_def,
                                     NULL,
                                     tab_elt_typ_geo,
                                     ECS_FALSE,
                                     cas_ens) ;

    /* Familles des lments */

    if (champ_famille != NULL) {

      ecs_tab_int_t  tab_famille ;

      tab_famille = ecs_loc_champ_post_ens__famille(champ_def,
                                                  champ_famille) ;

      ecs_champ_post_ens__ecr_val(&tab_famille,
                                  nom_maillage,
                                  _("family"),
                                  cas_ens) ;

      BFT_FREE(tab_famille.val) ;
      tab_famille.nbr = 0 ;

    }

  }

}


/*----------------------------------------------------------------------------
 *  Fonction crivant le champ  sortir au format Ensight
 *---------------------------------------------------------------------------*/

void ecs_champ_post_ens__ecr_val
(
 const ecs_tab_int_t   *const tab_val       ,
 const char            *const nom_maillage  ,
 const char            *const nom_champ     ,
       ecs_post_ens_t  *const cas_ens
)
{

  ecs_int_t  ielt         ;
  ecs_int_t  ind_part     ;

  ecs_int_t  ielt_max     ;
  ecs_int_t  ind_typ_ele  ;
  ecs_int_t  cpt_elt_loc  ;

  ecs_post_ens_part_t * part_loc ;

  bft_file_t        * fic_champ ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(tab_val != NULL) ;


  /* Recherche du `part' correspondant */
  /*-----------------------------------*/

  for (ind_part = 0 ;
       (   ind_part < cas_ens->nbr_part
        && strcmp((cas_ens->tab_part[ind_part])->nom_maillage,
                  nom_maillage) != 0) ;
       ind_part++) ;

  if (ind_part >= cas_ens->nbr_part)
    return ;

  fic_champ = ecs_post_ens__ecrit_fic_var(cas_ens,
                                          nom_champ) ;

  while (   ind_part < cas_ens->nbr_part
         && strcmp((cas_ens->tab_part[ind_part])->nom_maillage,
                   nom_maillage) == 0) {

    part_loc = cas_ens->tab_part[ind_part] ;
    ind_part++ ;


    /* criture de l'entte */
    /*----------------------*/

    bft_file_printf(fic_champ,
                    "part\n" "%10d\n",
                    (int)(part_loc->num_part)) ;


    /* Initialisations */
    /*-----------------*/

    ielt_max = -1 ;


    /* Boucle sur le type d'lment */
    /*------------------------------*/

    ielt = 0 ;

    for (ind_typ_ele = 0 ;
         ind_typ_ele < part_loc->nbr_typ_ele ;
         ind_typ_ele++) {

      bft_file_printf(fic_champ, "%s\n",
                      part_loc->nom_typ_ele[ind_typ_ele]) ;


      cpt_elt_loc = 0 ;


      /* criture des valeurs */

      if (part_loc->lst_parents == NULL) {

        while (cpt_elt_loc <  part_loc->nbr_ele_typ[ind_typ_ele]) {

          bft_file_printf(fic_champ, "%12.5E\n", (double)(tab_val->val[ielt])) ;
          cpt_elt_loc++ ;

          ielt++ ;

        }

      }
      else {

        while (cpt_elt_loc <  part_loc->nbr_ele_typ[ind_typ_ele]) {

          bft_file_printf(fic_champ, "%12.5E\n",
                          (double)(tab_val->val[part_loc->lst_parents[ielt]])) ;
          cpt_elt_loc++ ;

          ielt++ ;

        }

      }

    }

  } /* Fin de la boucle sur les parts associs au maillage */

  bft_file_free(fic_champ) ;

}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui crit les connectivits nodales des lments
 *   selon leur type gometrique
 *
 *  Les lements doivent avoir t tris suivant leur type gometrique
 *----------------------------------------------------------------------------*/

void ecs_loc_champ_post_ens__ecr_part
(
 const char            *const nom_maillage    ,
 const char            *const nom_part        ,
       ecs_champ_t     *const champ_som       ,
       ecs_champ_t     *const champ_def       ,
 const ecs_tab_int_t   *const liste_filtre    ,
 const ecs_tab_int_t   *const tab_elt_typ_geo ,
       ecs_bool_t             ecrire_parent   ,
       ecs_post_ens_t  *const cas_ens
)
{

  ecs_int_t  ielt ;
  ecs_int_t  ielt_loc ;
  ecs_int_t  ival ;
  ecs_int_t  ival_deb ;
  ecs_int_t  ival_fin ;
  ecs_int_t  isom ;
  ecs_int_t  icoo ;

  ecs_int_t  cpt_elt ;
  ecs_int_t  cpt_elt_fin ;
  ecs_int_t  cpt_elt_tot ;
  ecs_int_t  nbr_coo ;
  ecs_int_t  nbr_elt ;
  ecs_int_t  nbr_som ;
  ecs_int_t  elt_typ_ref ;
  ecs_int_t  marqueur_fin ;
  ecs_int_t  nbr_elt_typ_geo ;
  ecs_int_t  nbr_fac_loc ;
  ecs_int_t  nbr_som_loc ;

  ecs_int_t  * def_pos_tab ;
  ecs_int_t  * def_val_tab ;

  ecs_real_t * som_val_tab ;

  bft_file_t * fic_imp ;

  ecs_int_t  * indice_som = NULL ;

  ecs_post_ens_part_t  * this_part = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_som != NULL) ;
  assert(champ_def != NULL) ;

  assert(cas_ens != NULL) ;


  /* Cration de la structure d'information sur la part */
  /*----------------------------------------------------*/

  BFT_REALLOC(cas_ens->tab_part, cas_ens->nbr_part + 1, ecs_post_ens_part_t *) ;
  cas_ens->nbr_part += 1 ;

  BFT_MALLOC(this_part, 1, ecs_post_ens_part_t) ;
  cas_ens->tab_part[cas_ens->nbr_part - 1] = this_part ;

  this_part->num_part = cas_ens->nbr_part ;
  BFT_MALLOC(this_part->nom_maillage, strlen(nom_maillage) + 1, char) ;
  strcpy(this_part->nom_maillage, nom_maillage) ;
  BFT_MALLOC(this_part->nom_part, strlen(nom_part) + 1, char) ;
  strcpy(this_part->nom_part, nom_part) ;

  this_part->lst_parents = NULL ;


  /* Initialisations */
  /*-----------------*/

  fic_imp = ecs_post_ens__ecrit_fic_geo(cas_ens) ;

  assert(cas_ens->fic_geo != NULL) ;


  def_pos_tab =              ecs_champ__ret_pos_tab(champ_def) ;
  def_val_tab = (ecs_int_t *)(champ_def->val_tab) ;


  /* criture de l'entte */
  /*----------------------*/

  bft_file_printf(fic_imp,
                  "part\n" "%10d\n" "%s\n",
                  (int)(this_part->num_part), this_part->nom_part) ;


  /* criture du nombre de sommets */

  if (liste_filtre == NULL) {

    nbr_som = champ_som->nbr_elt ;

  }
  else {

    BFT_MALLOC(indice_som, champ_som->nbr_elt, ecs_int_t) ;

    for (isom = 0 ; isom < champ_som->nbr_elt ; isom++)
      indice_som[isom] = -1 ;

    nbr_som = 0 ;

    for (ielt_loc = 0 ; ielt_loc < liste_filtre->nbr ; ielt_loc++) {

      ielt = liste_filtre->val[ielt_loc] ;

      for (ival = def_pos_tab[ielt    ] - 1 ;
           ival < def_pos_tab[ielt + 1] - 1 ;
           ival++) {
        isom = def_val_tab[ival] - 1 ;
        if (indice_som[isom] == -1)
          indice_som[isom] = 0;
      }
    }

    for (isom = 0 ; isom < champ_som->nbr_elt ; isom++) {
      if (indice_som[isom] > -1) {
        indice_som[isom] = nbr_som++ ;
      }
    }

  }

  bft_file_printf(fic_imp, "coordinates\n" "%10d\n",
                  (int)nbr_som) ;

  this_part->nbr_som = nbr_som ;


  /* criture des coordonnes */
  /*--------------------------*/

  /* On est en dimension 3 */

  assert(champ_som->pos_pas == 2 || champ_som->pos_pas == 3) ;
  assert(champ_som->typ_val == ECS_TYPE_ecs_real_t) ;

  nbr_coo = champ_som->pos_pas ;
  som_val_tab = (ecs_real_t *)(champ_som->val_tab) ;

  for (icoo = 0 ; icoo < nbr_coo ; icoo++) {

    if (indice_som == NULL) {
      for (isom = 0 ; isom < champ_som->nbr_elt ; isom++)
        bft_file_printf(fic_imp, "%12.5E\n",
                        (float)som_val_tab[isom * nbr_coo + icoo]) ;
    }
    else {
      for (isom = 0 ; isom < champ_som->nbr_elt ; isom++) {
        if (indice_som[isom] > -1)
          bft_file_printf(fic_imp, "%12.5E\n",
                          (float)som_val_tab[isom*nbr_coo + icoo]) ;
      }
    }

  }

  /* On complte les coordonnes par Z = 0 en 2D */

  if (nbr_coo == 2) {

    for (isom = 0 ; isom < nbr_som ; isom++)
      bft_file_printf(fic_imp, "%12.5E\n", 0.0) ;

  }


  /* criture des lments */
  /*------------------------*/

  this_part->nbr_typ_ele = 0 ;
  this_part->nbr_ele_typ = NULL ;
  this_part->nom_typ_ele = NULL ;

  cpt_elt = 0 ;

  if (liste_filtre == NULL)
    nbr_elt = champ_def->nbr_elt ;
  else
    nbr_elt = liste_filtre->nbr ;

  cpt_elt_tot = 0 ;

  elt_typ_ref = -1 ;


  /* Boucle sur le type d'lment */

  while (cpt_elt < nbr_elt) {

    /* Recherche du prochain type d'lment utilis */

    elt_typ_ref += 1 ;

    while (tab_elt_typ_geo->val[elt_typ_ref] == 0)
      elt_typ_ref++ ;

    if (liste_filtre == NULL) {
      cpt_elt_fin = cpt_elt + tab_elt_typ_geo->val[elt_typ_ref] ;
    }
    else {
      cpt_elt_fin = cpt_elt ;
      cpt_elt_tot += tab_elt_typ_geo->val[elt_typ_ref] ;
      for (ielt_loc = cpt_elt ;
           (   ielt_loc < liste_filtre->nbr
            && liste_filtre->val[ielt_loc] < cpt_elt_tot) ;
           ielt_loc++)
        cpt_elt_fin++ ;
    }

    nbr_elt_typ_geo = cpt_elt_fin - cpt_elt ;

    if (nbr_elt_typ_geo == 0)
      continue ;

    /* criture du type d'lment */

    switch (elt_typ_ref) {

    case ECS_ELT_TYP_FAC_POLY:
      bft_file_printf(fic_imp, "nsided\n") ;
      break ;

    case ECS_ELT_TYP_CEL_POLY:
      bft_file_printf(fic_imp, "nfaced\n") ;
      break ;

    default:
      bft_file_printf(fic_imp, "%s\n",
                      ecs_fic_elt_typ_liste_c[elt_typ_ref].nom) ;

    }

    bft_file_printf(fic_imp, "%10d\n", (int)nbr_elt_typ_geo) ;

    this_part->nbr_typ_ele += 1 ;
    BFT_REALLOC(this_part->nbr_ele_typ, this_part->nbr_typ_ele, ecs_int_t) ;
    BFT_REALLOC(this_part->nom_typ_ele, this_part->nbr_typ_ele, char *) ;
    BFT_MALLOC(this_part->nom_typ_ele[this_part->nbr_typ_ele - 1],
               strlen(ecs_fic_elt_typ_liste_c[elt_typ_ref].nom) + 1, char) ;
    this_part->nbr_ele_typ[this_part->nbr_typ_ele - 1] = nbr_elt_typ_geo ;

    switch (elt_typ_ref) {

    case ECS_ELT_TYP_FAC_POLY:
      strcpy(this_part->nom_typ_ele[this_part->nbr_typ_ele - 1],
             "nsided") ;
      break ;

    case ECS_ELT_TYP_CEL_POLY:
      strcpy(this_part->nom_typ_ele[this_part->nbr_typ_ele - 1],
             "nfaced") ;
      break ;

    default:
      strcpy(this_part->nom_typ_ele[this_part->nbr_typ_ele - 1],
             ecs_fic_elt_typ_liste_c[elt_typ_ref].nom) ;

    }


    /* criture des connectivits */
    /*----------------------------*/

    switch(elt_typ_ref) {

    case ECS_ELT_TYP_FAC_POLY: /* Polygones */

      /* Nombre de sommets par lment */

      if (liste_filtre == NULL) {
        for (ielt = cpt_elt ; ielt < cpt_elt_fin ; ielt++)
          bft_file_printf(fic_imp, "%10d\n",
                          (int)(  def_pos_tab[ielt + 1]
                                  - def_pos_tab[ielt]    )) ;

      }
      else {
        for (ielt_loc = cpt_elt ; ielt_loc < cpt_elt_fin ; ielt_loc++) {
          ielt = liste_filtre->val[ielt_loc] ;
          bft_file_printf(fic_imp, "%10d\n",
                          (int)(  def_pos_tab[ielt + 1]
                                  - def_pos_tab[ielt]    )) ;
        }
      }

      /* Connectivit */

      if (liste_filtre == NULL) {
        for (ielt = cpt_elt ; ielt < cpt_elt_fin ; ielt++) {
          for (ival = def_pos_tab[ielt + 1] - 2 ;
               ival > def_pos_tab[ielt    ] - 2 ;
               ival--)
            bft_file_printf(fic_imp, "%10d",
                            (int)(def_val_tab[ival])) ;
          bft_file_printf(fic_imp, "\n") ;
        }
      }
      else {
        for (ielt_loc = cpt_elt ; ielt_loc < cpt_elt_fin ; ielt_loc++) {
          ielt = liste_filtre->val[ielt_loc] ;
          for (ival = def_pos_tab[ielt + 1] - 2 ;
               ival > def_pos_tab[ielt    ] - 2 ;
               ival--)
            bft_file_printf(fic_imp, "%10d",
                            (int)(indice_som[def_val_tab[ival] - 1] + 1)) ;
          bft_file_printf(fic_imp, "\n") ;
        }
      }

      break ;

    case ECS_ELT_TYP_CEL_POLY: /* Polydres */

      /* Convention : dfinition nodale cellule->sommets avec numros de
         premiers sommets rpts en fin de liste pour marquer la fin
         de chaque face */

      /* Faces par lment */

      if (liste_filtre == NULL) {
        for (ielt = cpt_elt ; ielt < cpt_elt_fin ; ielt++) {
          marqueur_fin = -1 ;
          nbr_fac_loc = 0 ;
          for (ival = def_pos_tab[ielt    ] - 1 ;
               ival < def_pos_tab[ielt + 1] - 1 ;
               ival++) {
            if (def_val_tab[ival] != marqueur_fin) {
              if (marqueur_fin == -1)
                marqueur_fin = def_val_tab[ival] ;
            }
            else {
              marqueur_fin = -1 ;
              nbr_fac_loc += 1 ;
            }
          }
          bft_file_printf(fic_imp, "%10d\n", (int)nbr_fac_loc) ;
        }
      }
      else {
        for (ielt_loc = cpt_elt ; ielt_loc < cpt_elt_fin ; ielt_loc++) {
          ielt = liste_filtre->val[ielt_loc] ;
          marqueur_fin = -1 ;
          nbr_fac_loc = 0 ;
          for (ival = def_pos_tab[ielt    ] - 1 ;
               ival < def_pos_tab[ielt + 1] - 1 ;
               ival++) {
            if (def_val_tab[ival] != marqueur_fin) {
              if (marqueur_fin == -1)
                marqueur_fin = def_val_tab[ival] ;
            }
            else {
              marqueur_fin = -1 ;
              nbr_fac_loc += 1 ;
            }
          }
          bft_file_printf(fic_imp, "%10d\n", (int)nbr_fac_loc) ;
        }
      }

      /* Sommets par face */

      marqueur_fin = -1 ;
      nbr_som_loc = 0 ;

      if (liste_filtre == NULL) {

        for (ival = def_pos_tab[cpt_elt    ] - 1 ;
             ival < def_pos_tab[cpt_elt_fin] - 1 ;
             ival++) {

          if (def_val_tab[ival] != marqueur_fin) {
            nbr_som_loc += 1 ;
            if (marqueur_fin == -1)
              marqueur_fin = def_val_tab[ival] ;
          }
          else {
            marqueur_fin = -1 ;
            bft_file_printf(fic_imp, "%10d\n", (int)nbr_som_loc) ;
            nbr_som_loc = 0 ;
          }

        }

      }
      else {

        for (ielt_loc = cpt_elt ; ielt_loc < cpt_elt_fin ; ielt_loc++) {

          ielt = liste_filtre->val[ielt_loc] ;
          ival_deb = def_pos_tab[ielt    ] - 1 ;
          ival_fin = def_pos_tab[ielt + 1] - 1 ;

          for (ival = ival_deb ; ival < ival_fin ; ival++) {
            if (def_val_tab[ival] != marqueur_fin) {
              nbr_som_loc += 1 ;
              if (marqueur_fin == -1)
                marqueur_fin = def_val_tab[ival] ;
            }
            else {
              marqueur_fin = -1 ;
              bft_file_printf(fic_imp, "%10d\n", (int)nbr_som_loc) ;
              nbr_som_loc = 0 ;
            }
          }

        }

      }

      /* Connectivit */

      marqueur_fin = -1 ;

      if (liste_filtre == NULL) {

        for (ival = def_pos_tab[cpt_elt    ] - 1 ;
             ival < def_pos_tab[cpt_elt_fin] - 1 ;
             ival++) {

          if (def_val_tab[ival] != marqueur_fin) {
            bft_file_printf(fic_imp, "%10d",
                            (int)(def_val_tab[ival])) ;
            if (marqueur_fin == -1)
              marqueur_fin = def_val_tab[ival] ;
          }
          else {
            marqueur_fin = -1 ;
            bft_file_printf(fic_imp, "\n") ;
          }

        }

      }
      else {

        for (ielt_loc = cpt_elt ; ielt_loc < cpt_elt_fin ; ielt_loc++) {

          ielt = liste_filtre->val[ielt_loc] ;
          ival_deb = def_pos_tab[ielt    ] - 1 ;
          ival_fin = def_pos_tab[ielt + 1] - 1 ;

          for (ival = ival_deb ; ival < ival_fin ; ival++) {

            if (def_val_tab[ival] != marqueur_fin) {
              bft_file_printf(fic_imp, "%10d",
                              (int)(indice_som[def_val_tab[ival] - 1] + 1)) ;
              if (marqueur_fin == -1)
                marqueur_fin = def_val_tab[ival] ;
            }
            else {
              marqueur_fin = -1 ;
              bft_file_printf(fic_imp, "\n") ;
            }

          }

        }

      }

      break ;

    default: /* lments "standard" */

      if (liste_filtre == NULL) {

        for (ielt = cpt_elt ; ielt < cpt_elt_fin ; ielt++) {

          for (ival = def_pos_tab[ielt]     - 1 ;
               ival < def_pos_tab[ielt + 1] - 1 ;
               ival++)

            bft_file_printf(fic_imp, "%10d",
                            (int)(def_val_tab[ival])) ;

          bft_file_printf(fic_imp, "\n") ;

        }

      }
      else {

        for (ielt_loc = cpt_elt ; ielt_loc < cpt_elt_fin ; ielt_loc++) {

          ielt = liste_filtre->val[ielt_loc] ;

          for (ival = def_pos_tab[ielt]     - 1 ;
               ival < def_pos_tab[ielt + 1] - 1 ;
               ival++)

            bft_file_printf(fic_imp, "%10d",
                            (int)(indice_som[def_val_tab[ival] - 1] + 1)) ;

          bft_file_printf(fic_imp, "\n") ;

        }

      }

      break;

    }


    /* On s'apprte  passer au type d'lment suivant */

    cpt_elt += nbr_elt_typ_geo ;

  }

  bft_file_flush(fic_imp) ;


  /* Nettoyage avant la sortie */
  /*---------------------------*/

  if (indice_som != NULL)
    BFT_FREE(indice_som) ;

  ecs_champ__libere_pos_tab(champ_def, def_pos_tab) ;


  /* Fermeture du fichier de gomtrie */

  bft_file_close_stream(cas_ens->fic_geo) ;

  if (ecrire_parent == ECS_TRUE && liste_filtre != NULL) {
    BFT_MALLOC(this_part->lst_parents, liste_filtre->nbr, ecs_int_t) ;
    memcpy(this_part->lst_parents,
           liste_filtre->val,
           liste_filtre->nbr * sizeof(ecs_int_t)) ;
  }

}


/*----------------------------------------------------------------------------
 *  Fonction d'extraction des numros de familles
 *---------------------------------------------------------------------------*/

static ecs_tab_int_t ecs_loc_champ_post_ens__famille
(
 const ecs_champ_t  *const champ_def     ,
       ecs_champ_t  *const champ_famille
)
{

  ecs_int_t   ielt ;
  ecs_int_t   pos_elt ;
  ecs_int_t   pos_elt_sup ;

  ecs_int_t * famille_pos_tab ;
  ecs_int_t * famille_val_tab ;

  ecs_tab_int_t  tab_fam ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_famille != NULL) ;


  /* Familles des lments */
  /*-----------------------*/

  tab_fam.nbr = champ_def->nbr_elt ;
  BFT_MALLOC(tab_fam.val, tab_fam.nbr, ecs_int_t) ;


  famille_val_tab = (ecs_int_t *)(champ_famille->val_tab) ;

  if (champ_famille->pos_pas != 1) {

    /* Certains lments n'ont pas de famille */

    /* On attribue aux lments qui n'ont pas de famille, */
    /*  le numro de famille `0'                          */

    famille_pos_tab = ecs_champ__ret_pos_tab(champ_famille) ;

    for (ielt = 0 ; ielt < tab_fam.nbr ; ielt++) {

      pos_elt     = famille_pos_tab[ielt    ] - 1 ;
      pos_elt_sup = famille_pos_tab[ielt + 1] - 1 ;

      if (pos_elt == pos_elt_sup) {

        tab_fam.val[ielt] = 0 ;

      }
      else {

        tab_fam.val[ielt] = famille_val_tab[pos_elt] ;

      }

    } /* Fin : boucle sur les elements */

    ecs_champ__libere_pos_tab(champ_famille, famille_pos_tab) ;

  }
  else { /* Tous les elements ont une famille */

    for (ielt = 0 ; ielt < tab_fam.nbr ; ielt++)
      tab_fam.val[ielt] = famille_val_tab[ielt] ;

  }


  return tab_fam ;

}

